package com.alibaba.fastjson.asm;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

public class ClassReader {
   public final byte[] b;
   private final int[] items;
   private final String[] strings;
   private final int maxStringLength;
   public final int header;

   public ClassReader(InputStream is) throws IOException {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      byte[] buf = new byte[1024];

      while(true) {
         int index = is.read(buf);
         if (index == -1) {
            is.close();
            this.b = out.toByteArray();
            this.items = new int[this.readUnsignedShort(8)];
            int n = this.items.length;
            this.strings = new String[n];
            int max = 0;
            index = 10;

            for(int i = 1; i < n; ++i) {
               this.items[i] = index + 1;
               int size;
               switch(this.b[index]) {
               case 1:
                  size = 3 + this.readUnsignedShort(index + 1);
                  if (size > max) {
                     max = size;
                  }
                  break;
               case 2:
               case 7:
               case 8:
               case 13:
               case 14:
               case 16:
               case 17:
               default:
                  size = 3;
                  break;
               case 3:
               case 4:
               case 9:
               case 10:
               case 11:
               case 12:
               case 18:
                  size = 5;
                  break;
               case 5:
               case 6:
                  size = 9;
                  ++i;
                  break;
               case 15:
                  size = 4;
               }

               index += size;
            }

            this.maxStringLength = max;
            this.header = index;
            return;
         }

         if (index > 0) {
            out.write(buf, 0, index);
         }
      }
   }

   public void accept(TypeCollector classVisitor) {
      char[] c = new char[this.maxStringLength];
      int anns = false;
      int ianns = false;
      int u = this.header;
      int v = this.items[this.readUnsignedShort(u + 4)];
      int len = this.readUnsignedShort(u + 6);
      int w = false;
      u += 8;

      int i;
      for(i = 0; i < len; ++i) {
         u += 2;
      }

      i = this.readUnsignedShort(u);

      int j;
      for(v = u + 2; i > 0; --i) {
         j = this.readUnsignedShort(v + 6);

         for(v += 8; j > 0; --j) {
            v += 6 + this.readInt(v + 2);
         }
      }

      i = this.readUnsignedShort(v);

      for(v += 2; i > 0; --i) {
         j = this.readUnsignedShort(v + 6);

         for(v += 8; j > 0; --j) {
            v += 6 + this.readInt(v + 2);
         }
      }

      i = this.readUnsignedShort(v);

      for(v += 2; i > 0; --i) {
         v += 6 + this.readInt(v + 2);
      }

      i = this.readUnsignedShort(u);

      for(u += 2; i > 0; --i) {
         j = this.readUnsignedShort(u + 6);

         for(u += 8; j > 0; --j) {
            u += 6 + this.readInt(u + 2);
         }
      }

      i = this.readUnsignedShort(u);

      for(u += 2; i > 0; --i) {
         u = this.readMethod(classVisitor, c, u);
      }

   }

   private int readMethod(TypeCollector classVisitor, char[] c, int u) {
      int access = this.readUnsignedShort(u);
      String name = this.readUTF8(u + 2, c);
      String desc = this.readUTF8(u + 4, c);
      int v = 0;
      int w = 0;
      int j = this.readUnsignedShort(u + 6);

      String attrName;
      for(u += 8; j > 0; --j) {
         attrName = this.readUTF8(u, c);
         int attrSize = this.readInt(u + 2);
         u += 6;
         if (attrName.equals("Code")) {
            v = u;
         }

         u += attrSize;
      }

      int w;
      if (w != 0) {
         w = w + 2;

         for(j = 0; j < this.readUnsignedShort(w); ++j) {
            w += 2;
         }
      }

      MethodCollector mv = classVisitor.visitMethod(access, name, desc);
      if (mv != null && v != 0) {
         int codeLength = this.readInt(v + 4);
         v += 8;
         int codeEnd = v + codeLength;
         j = this.readUnsignedShort(codeEnd);

         for(v = codeEnd + 2; j > 0; --j) {
            v += 8;
         }

         int varTable = 0;
         int varTypeTable = 0;
         j = this.readUnsignedShort(v);

         for(v += 2; j > 0; --j) {
            attrName = this.readUTF8(v, c);
            if (attrName.equals("LocalVariableTable")) {
               varTable = v + 6;
            } else if (attrName.equals("LocalVariableTypeTable")) {
               varTypeTable = v + 6;
            }

            v += 6 + this.readInt(v + 2);
         }

         if (varTable != 0) {
            int k;
            if (varTypeTable != 0) {
               k = this.readUnsignedShort(varTypeTable) * 3;
               w = varTypeTable + 2;

               for(int[] typeTable = new int[k]; k > 0; w += 10) {
                  --k;
                  typeTable[k] = w + 6;
                  --k;
                  typeTable[k] = this.readUnsignedShort(w + 8);
                  --k;
                  typeTable[k] = this.readUnsignedShort(w);
               }
            }

            k = this.readUnsignedShort(varTable);

            for(w = varTable + 2; k > 0; --k) {
               int index = this.readUnsignedShort(w + 8);
               mv.visitLocalVariable(this.readUTF8(w + 4, c), index);
               w += 10;
            }
         }
      }

      return u;
   }

   private int readUnsignedShort(int index) {
      byte[] b = this.b;
      return (b[index] & 255) << 8 | b[index + 1] & 255;
   }

   private int readInt(int index) {
      byte[] b = this.b;
      return (b[index] & 255) << 24 | (b[index + 1] & 255) << 16 | (b[index + 2] & 255) << 8 | b[index + 3] & 255;
   }

   private String readUTF8(int index, char[] buf) {
      int item = this.readUnsignedShort(index);
      String s = this.strings[item];
      if (s != null) {
         return s;
      } else {
         index = this.items[item];
         return this.strings[item] = this.readUTF(index + 2, this.readUnsignedShort(index), buf);
      }
   }

   private String readUTF(int index, int utfLen, char[] buf) {
      int endIndex = index + utfLen;
      byte[] b = this.b;
      int strLen = 0;
      int st = 0;
      char cc = 0;

      while(true) {
         while(index < endIndex) {
            int c = b[index++];
            switch(st) {
            case 0:
               int c = c & 255;
               if (c < 128) {
                  buf[strLen++] = (char)c;
               } else {
                  if (c < 224 && c > 191) {
                     cc = (char)(c & 31);
                     st = 1;
                     continue;
                  }

                  cc = (char)(c & 15);
                  st = 2;
               }
               break;
            case 1:
               buf[strLen++] = (char)(cc << 6 | c & 63);
               st = 0;
               break;
            case 2:
               cc = (char)(cc << 6 | c & 63);
               st = 1;
            }
         }

         return new String(buf, 0, strLen);
      }
   }
}
