package com.alibaba.fastjson.asm;

import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Map;

public class TypeCollector {
   private static final Map<String, String> primitives = new HashMap<String, String>() {
      {
         this.put("int", "I");
         this.put("boolean", "Z");
         this.put("byte", "B");
         this.put("char", "C");
         this.put("short", "S");
         this.put("float", "F");
         this.put("long", "J");
         this.put("double", "D");
      }
   };
   private final String methodName;
   private final Class<?>[] parameterTypes;
   protected MethodCollector collector;

   public TypeCollector(String methodName, Class<?>[] parameterTypes) {
      this.methodName = methodName;
      this.parameterTypes = parameterTypes;
      this.collector = null;
   }

   protected MethodCollector visitMethod(int access, String name, String desc) {
      if (this.collector != null) {
         return null;
      } else if (!name.equals(this.methodName)) {
         return null;
      } else {
         Type[] argTypes = Type.getArgumentTypes(desc);
         int longOrDoubleQuantity = 0;
         Type[] var6 = argTypes;
         int var7 = argTypes.length;

         for(int var8 = 0; var8 < var7; ++var8) {
            Type t = var6[var8];
            String className = t.getClassName();
            if (className.equals("long") || className.equals("double")) {
               ++longOrDoubleQuantity;
            }
         }

         if (argTypes.length != this.parameterTypes.length) {
            return null;
         } else {
            for(int i = 0; i < argTypes.length; ++i) {
               if (!this.correctTypeName(argTypes[i], this.parameterTypes[i].getName())) {
                  return null;
               }
            }

            return this.collector = new MethodCollector(Modifier.isStatic(access) ? 0 : 1, argTypes.length + longOrDoubleQuantity);
         }
      }
   }

   private boolean correctTypeName(Type type, String paramTypeName) {
      String s = type.getClassName();

      String braces;
      for(braces = ""; s.endsWith("[]"); s = s.substring(0, s.length() - 2)) {
         braces = braces + "[";
      }

      if (!braces.equals("")) {
         if (primitives.containsKey(s)) {
            s = braces + (String)primitives.get(s);
         } else {
            s = braces + "L" + s + ";";
         }
      }

      return s.equals(paramTypeName);
   }

   public String[] getParameterNamesForMethod() {
      return this.collector != null && this.collector.debugInfoPresent ? this.collector.getResult().split(",") : new String[0];
   }
}
