package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.JSONScanner;
import java.lang.reflect.Type;
import java.text.ParseException;
import java.text.SimpleDateFormat;

public abstract class AbstractDateDeserializer extends ContextObjectDeserializer implements ObjectDeserializer {
   public <T> T deserialze(DefaultJSONParser parser, Type clazz, Object fieldName) {
      return this.deserialze(parser, clazz, fieldName, (String)null, 0);
   }

   public <T> T deserialze(DefaultJSONParser parser, Type clazz, Object fieldName, String format, int features) {
      JSONLexer lexer = parser.lexer;
      Object val;
      if (lexer.token() == 2) {
         val = lexer.longValue();
         lexer.nextToken(16);
      } else {
         String strVal;
         if (lexer.token() == 4) {
            strVal = lexer.stringVal();
            if (format != null) {
               SimpleDateFormat simpleDateFormat = null;

               try {
                  simpleDateFormat = new SimpleDateFormat(format, JSON.defaultLocale);
               } catch (IllegalArgumentException var14) {
                  if (format.equals("yyyy-MM-ddTHH:mm:ss.SSS")) {
                     format = "yyyy-MM-dd'T'HH:mm:ss.SSS";
                     simpleDateFormat = new SimpleDateFormat(format);
                  } else if (format.equals("yyyy-MM-ddTHH:mm:ss")) {
                     format = "yyyy-MM-dd'T'HH:mm:ss";
                     simpleDateFormat = new SimpleDateFormat(format);
                  }
               }

               try {
                  val = simpleDateFormat.parse(strVal);
               } catch (ParseException var13) {
                  if (format.equals("yyyy-MM-dd'T'HH:mm:ss.SSS") && strVal.length() == 19) {
                     try {
                        val = (new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")).parse(strVal);
                     } catch (ParseException var12) {
                        val = null;
                     }
                  } else {
                     val = null;
                  }
               }
            } else {
               val = null;
            }

            if (val == null) {
               val = strVal;
               lexer.nextToken(16);
               if (lexer.isEnabled(Feature.AllowISO8601DateFormat)) {
                  JSONScanner iso8601Lexer = new JSONScanner(strVal);
                  if (iso8601Lexer.scanISO8601DateIfMatch()) {
                     val = iso8601Lexer.getCalendar().getTime();
                  }

                  iso8601Lexer.close();
               }
            }
         } else if (lexer.token() == 8) {
            lexer.nextToken();
            val = null;
         } else if (lexer.token() == 12) {
            lexer.nextToken();
            if (lexer.token() != 4) {
               throw new JSONException("syntax error");
            }

            strVal = lexer.stringVal();
            if (JSON.DEFAULT_TYPE_KEY.equals(strVal)) {
               lexer.nextToken();
               parser.accept(17);
               String typeName = lexer.stringVal();
               Class<?> type = parser.getConfig().checkAutoType(typeName, (Class)null, lexer.getFeatures());
               if (type != null) {
                  clazz = type;
               }

               parser.accept(4);
               parser.accept(16);
            }

            lexer.nextTokenWithColon(2);
            if (lexer.token() != 2) {
               throw new JSONException("syntax error : " + lexer.tokenName());
            }

            long timeMillis = lexer.longValue();
            lexer.nextToken();
            val = timeMillis;
            parser.accept(13);
         } else if (parser.getResolveStatus() == 2) {
            parser.setResolveStatus(0);
            parser.accept(16);
            if (lexer.token() != 4) {
               throw new JSONException("syntax error");
            }

            if (!"val".equals(lexer.stringVal())) {
               throw new JSONException("syntax error");
            }

            lexer.nextToken();
            parser.accept(17);
            val = parser.parse();
            parser.accept(13);
         } else {
            val = parser.parse();
         }
      }

      return this.cast(parser, (Type)clazz, fieldName, val);
   }

   protected abstract <T> T cast(DefaultJSONParser var1, Type var2, Object var3, Object var4);
}
