package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.ParseContext;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.util.FieldInfo;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Map;
import java.util.zip.GZIPInputStream;

public class DefaultFieldDeserializer extends FieldDeserializer {
   protected ObjectDeserializer fieldValueDeserilizer;
   protected boolean customDeserilizer = false;

   public DefaultFieldDeserializer(ParserConfig config, Class<?> clazz, FieldInfo fieldInfo) {
      super(clazz, fieldInfo);
      JSONField annotation = fieldInfo.getAnnotation();
      if (annotation != null) {
         Class<?> deserializeUsing = annotation.deserializeUsing();
         this.customDeserilizer = deserializeUsing != null && deserializeUsing != Void.class;
      }

   }

   public ObjectDeserializer getFieldValueDeserilizer(ParserConfig config) {
      if (this.fieldValueDeserilizer == null) {
         JSONField annotation = this.fieldInfo.getAnnotation();
         if (annotation != null && annotation.deserializeUsing() != Void.class) {
            Class deserializeUsing = annotation.deserializeUsing();

            try {
               this.fieldValueDeserilizer = (ObjectDeserializer)deserializeUsing.newInstance();
            } catch (Exception var5) {
               throw new JSONException("create deserializeUsing ObjectDeserializer error", var5);
            }
         } else {
            this.fieldValueDeserilizer = config.getDeserializer(this.fieldInfo.fieldClass, this.fieldInfo.fieldType);
         }
      }

      return this.fieldValueDeserilizer;
   }

   public void parseField(DefaultJSONParser parser, Object object, Type objectType, Map<String, Object> fieldValues) {
      if (this.fieldValueDeserilizer == null) {
         this.getFieldValueDeserilizer(parser.getConfig());
      }

      ObjectDeserializer fieldValueDeserilizer = this.fieldValueDeserilizer;
      Type fieldType = this.fieldInfo.fieldType;
      if (objectType instanceof ParameterizedType) {
         ParseContext objContext = parser.getContext();
         if (objContext != null) {
            objContext.type = objectType;
         }

         if (fieldType != objectType) {
            fieldType = FieldInfo.getFieldType(this.clazz, objectType, fieldType);
            fieldValueDeserilizer = parser.getConfig().getDeserializer(fieldType);
         }
      }

      Object value;
      if (fieldValueDeserilizer instanceof JavaBeanDeserializer && this.fieldInfo.parserFeatures != 0) {
         JavaBeanDeserializer javaBeanDeser = (JavaBeanDeserializer)fieldValueDeserilizer;
         value = javaBeanDeser.deserialze(parser, fieldType, this.fieldInfo.name, this.fieldInfo.parserFeatures);
      } else if (this.fieldInfo.format != null && fieldValueDeserilizer instanceof ContextObjectDeserializer) {
         value = ((ContextObjectDeserializer)fieldValueDeserilizer).deserialze(parser, fieldType, this.fieldInfo.name, this.fieldInfo.format, this.fieldInfo.parserFeatures);
      } else {
         value = fieldValueDeserilizer.deserialze(parser, fieldType, this.fieldInfo.name);
      }

      if (value instanceof byte[] && ("gzip".equals(this.fieldInfo.format) || "gzip,base64".equals(this.fieldInfo.format))) {
         byte[] bytes = (byte[])((byte[])value);
         GZIPInputStream gzipIn = null;

         try {
            gzipIn = new GZIPInputStream(new ByteArrayInputStream(bytes));
            ByteArrayOutputStream byteOut = new ByteArrayOutputStream();

            while(true) {
               byte[] buf = new byte[1024];
               int len = gzipIn.read(buf);
               if (len == -1) {
                  value = byteOut.toByteArray();
                  break;
               }

               if (len > 0) {
                  byteOut.write(buf, 0, len);
               }
            }
         } catch (IOException var13) {
            throw new JSONException("unzip bytes error.", var13);
         }
      }

      if (parser.getResolveStatus() == 1) {
         DefaultJSONParser.ResolveTask task = parser.getLastResolveTask();
         task.fieldDeserializer = this;
         task.ownerContext = parser.getContext();
         parser.setResolveStatus(0);
      } else if (object == null) {
         fieldValues.put(this.fieldInfo.name, value);
      } else {
         this.setValue(object, value);
      }

   }

   public int getFastMatchToken() {
      return this.fieldValueDeserilizer != null ? this.fieldValueDeserilizer.getFastMatchToken() : 2;
   }

   public void parseFieldUnwrapped(DefaultJSONParser parser, Object object, Type objectType, Map<String, Object> fieldValues) {
      throw new JSONException("TODO");
   }
}
