package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class EnumDeserializer implements ObjectDeserializer {
   protected final Class<?> enumClass;
   protected final Enum[] enums;
   protected final Enum[] ordinalEnums;
   protected long[] enumNameHashCodes;

   public EnumDeserializer(Class<?> enumClass) {
      this.enumClass = enumClass;
      this.ordinalEnums = (Enum[])((Enum[])enumClass.getEnumConstants());
      Map<Long, Enum> enumMap = new HashMap();

      int i;
      for(i = 0; i < this.ordinalEnums.length; ++i) {
         Enum e = this.ordinalEnums[i];
         String name = e.name();
         JSONField jsonField = null;

         try {
            Field field = enumClass.getField(name);
            jsonField = (JSONField)field.getAnnotation(JSONField.class);
            if (jsonField != null) {
               String jsonFieldName = jsonField.name();
               if (jsonFieldName != null && jsonFieldName.length() > 0) {
                  name = jsonFieldName;
               }
            }
         } catch (Exception var19) {
         }

         long hash = -3750763034362895579L;
         long hash_lower = -3750763034362895579L;

         for(int j = 0; j < name.length(); ++j) {
            char ch = name.charAt(j);
            hash ^= (long)ch;
            hash_lower ^= (long)(ch >= 'A' && ch <= 'Z' ? ch + 32 : ch);
            hash *= 1099511628211L;
            hash_lower *= 1099511628211L;
         }

         enumMap.put(hash, e);
         if (hash != hash_lower) {
            enumMap.put(hash_lower, e);
         }

         if (jsonField != null) {
            String[] var25 = jsonField.alternateNames();
            int var26 = var25.length;

            for(int var13 = 0; var13 < var26; ++var13) {
               String alterName = var25[var13];
               long alterNameHash = -3750763034362895579L;

               for(int j = 0; j < alterName.length(); ++j) {
                  char ch = alterName.charAt(j);
                  alterNameHash ^= (long)ch;
                  alterNameHash *= 1099511628211L;
               }

               if (alterNameHash != hash && alterNameHash != hash_lower) {
                  enumMap.put(alterNameHash, e);
               }
            }
         }
      }

      this.enumNameHashCodes = new long[enumMap.size()];
      i = 0;

      Long h;
      for(Iterator var20 = enumMap.keySet().iterator(); var20.hasNext(); this.enumNameHashCodes[i++] = h) {
         h = (Long)var20.next();
      }

      Arrays.sort(this.enumNameHashCodes);
      this.enums = new Enum[this.enumNameHashCodes.length];

      for(i = 0; i < this.enumNameHashCodes.length; ++i) {
         long hash = this.enumNameHashCodes[i];
         Enum e = (Enum)enumMap.get(hash);
         this.enums[i] = e;
      }

   }

   public Enum getEnumByHashCode(long hashCode) {
      if (this.enums == null) {
         return null;
      } else {
         int enumIndex = Arrays.binarySearch(this.enumNameHashCodes, hashCode);
         return enumIndex < 0 ? null : this.enums[enumIndex];
      }
   }

   public Enum<?> valueOf(int ordinal) {
      return this.ordinalEnums[ordinal];
   }

   public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName) {
      try {
         JSONLexer lexer = parser.lexer;
         int token = lexer.token();
         if (token == 2) {
            int intValue = lexer.intValue();
            lexer.nextToken(16);
            if (intValue >= 0 && intValue <= this.ordinalEnums.length) {
               return this.ordinalEnums[intValue];
            } else {
               throw new JSONException("parse enum " + this.enumClass.getName() + " error, value : " + intValue);
            }
         } else if (token != 4) {
            Object value;
            if (token == 8) {
               value = null;
               lexer.nextToken(16);
               return null;
            } else {
               value = parser.parse();
               throw new JSONException("parse enum " + this.enumClass.getName() + " error, value : " + value);
            }
         } else {
            String name = lexer.stringVal();
            lexer.nextToken(16);
            if (name.length() == 0) {
               return null;
            } else {
               long hash = -3750763034362895579L;

               for(int j = 0; j < name.length(); ++j) {
                  char ch = name.charAt(j);
                  hash ^= (long)ch;
                  hash *= 1099511628211L;
               }

               return this.getEnumByHashCode(hash);
            }
         }
      } catch (JSONException var12) {
         throw var12;
      } catch (Exception var13) {
         throw new JSONException(var13.getMessage(), var13);
      }
   }

   public int getFastMatchToken() {
      return 2;
   }
}
