package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.serializer.BeanContext;
import com.alibaba.fastjson.util.FieldInfo;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

public abstract class FieldDeserializer {
   public final FieldInfo fieldInfo;
   protected final Class<?> clazz;
   protected BeanContext beanContext;

   public FieldDeserializer(Class<?> clazz, FieldInfo fieldInfo) {
      this.clazz = clazz;
      this.fieldInfo = fieldInfo;
   }

   public abstract void parseField(DefaultJSONParser var1, Object var2, Type var3, Map<String, Object> var4);

   public int getFastMatchToken() {
      return 0;
   }

   public void setValue(Object object, boolean value) {
      this.setValue(object, (Object)value);
   }

   public void setValue(Object object, int value) {
      this.setValue(object, (Object)value);
   }

   public void setValue(Object object, long value) {
      this.setValue(object, (Object)value);
   }

   public void setValue(Object object, String value) {
      this.setValue(object, (Object)value);
   }

   public void setValue(Object object, Object value) {
      if (value != null || !this.fieldInfo.fieldClass.isPrimitive()) {
         if (this.fieldInfo.fieldClass == String.class && this.fieldInfo.format != null && this.fieldInfo.format.equals("trim")) {
            value = ((String)value).trim();
         }

         try {
            Method method = this.fieldInfo.method;
            if (method != null) {
               if (this.fieldInfo.getOnly) {
                  if (this.fieldInfo.fieldClass == AtomicInteger.class) {
                     AtomicInteger atomic = (AtomicInteger)method.invoke(object);
                     if (atomic != null) {
                        atomic.set(((AtomicInteger)value).get());
                     }
                  } else if (this.fieldInfo.fieldClass == AtomicLong.class) {
                     AtomicLong atomic = (AtomicLong)method.invoke(object);
                     if (atomic != null) {
                        atomic.set(((AtomicLong)value).get());
                     }
                  } else if (this.fieldInfo.fieldClass == AtomicBoolean.class) {
                     AtomicBoolean atomic = (AtomicBoolean)method.invoke(object);
                     if (atomic != null) {
                        atomic.set(((AtomicBoolean)value).get());
                     }
                  } else if (Map.class.isAssignableFrom(method.getReturnType())) {
                     Map map = (Map)method.invoke(object);
                     if (map != null) {
                        if (map == Collections.emptyMap() || map.getClass().getName().startsWith("java.util.Collections$Unmodifiable")) {
                           return;
                        }

                        map.putAll((Map)value);
                     }
                  } else {
                     Collection collection = (Collection)method.invoke(object);
                     if (collection != null && value != null) {
                        if (collection == Collections.emptySet() || collection == Collections.emptyList() || collection.getClass().getName().startsWith("java.util.Collections$Unmodifiable")) {
                           return;
                        }

                        collection.clear();
                        collection.addAll((Collection)value);
                     }
                  }
               } else {
                  method.invoke(object, value);
               }
            } else {
               Field field = this.fieldInfo.field;
               if (this.fieldInfo.getOnly) {
                  if (this.fieldInfo.fieldClass == AtomicInteger.class) {
                     AtomicInteger atomic = (AtomicInteger)field.get(object);
                     if (atomic != null) {
                        atomic.set(((AtomicInteger)value).get());
                     }
                  } else if (this.fieldInfo.fieldClass == AtomicLong.class) {
                     AtomicLong atomic = (AtomicLong)field.get(object);
                     if (atomic != null) {
                        atomic.set(((AtomicLong)value).get());
                     }
                  } else if (this.fieldInfo.fieldClass == AtomicBoolean.class) {
                     AtomicBoolean atomic = (AtomicBoolean)field.get(object);
                     if (atomic != null) {
                        atomic.set(((AtomicBoolean)value).get());
                     }
                  } else if (Map.class.isAssignableFrom(this.fieldInfo.fieldClass)) {
                     Map map = (Map)field.get(object);
                     if (map != null) {
                        if (map == Collections.emptyMap() || map.getClass().getName().startsWith("java.util.Collections$Unmodifiable")) {
                           return;
                        }

                        map.putAll((Map)value);
                     }
                  } else {
                     Collection collection = (Collection)field.get(object);
                     if (collection != null && value != null) {
                        if (collection == Collections.emptySet() || collection == Collections.emptyList() || collection.getClass().getName().startsWith("java.util.Collections$Unmodifiable")) {
                           return;
                        }

                        collection.clear();
                        collection.addAll((Collection)value);
                     }
                  }
               } else if (field != null) {
                  field.set(object, value);
               }
            }

         } catch (Exception var6) {
            throw new JSONException("set property error, " + this.fieldInfo.name, var6);
         }
      }
   }

   public void setWrappedValue(String key, Object value) {
      throw new JSONException("TODO");
   }
}
