package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.serializer.BeanContext;
import com.alibaba.fastjson.serializer.ContextObjectSerializer;
import com.alibaba.fastjson.serializer.JSONSerializer;
import com.alibaba.fastjson.serializer.ObjectSerializer;
import com.alibaba.fastjson.serializer.SerializeWriter;
import com.alibaba.fastjson.serializer.SerializerFeature;
import java.io.IOException;
import java.lang.reflect.Type;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.OffsetDateTime;
import java.time.OffsetTime;
import java.time.Period;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAccessor;
import java.util.Locale;

public class Jdk8DateCodec extends ContextObjectDeserializer implements ObjectSerializer, ContextObjectSerializer, ObjectDeserializer {
   public static final Jdk8DateCodec instance = new Jdk8DateCodec();
   private static final String defaultPatttern = "yyyy-MM-dd HH:mm:ss";
   private static final DateTimeFormatter defaultFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
   private static final DateTimeFormatter formatter_dt19_tw = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
   private static final DateTimeFormatter formatter_dt19_cn = DateTimeFormatter.ofPattern("yyyy年M月d日 HH:mm:ss");
   private static final DateTimeFormatter formatter_dt19_cn_1 = DateTimeFormatter.ofPattern("yyyy年M月d日 H时m分s秒");
   private static final DateTimeFormatter formatter_dt19_kr = DateTimeFormatter.ofPattern("yyyy년M월d일 HH:mm:ss");
   private static final DateTimeFormatter formatter_dt19_us = DateTimeFormatter.ofPattern("MM/dd/yyyy HH:mm:ss");
   private static final DateTimeFormatter formatter_dt19_eur = DateTimeFormatter.ofPattern("dd/MM/yyyy HH:mm:ss");
   private static final DateTimeFormatter formatter_dt19_de = DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss");
   private static final DateTimeFormatter formatter_dt19_in = DateTimeFormatter.ofPattern("dd-MM-yyyy HH:mm:ss");
   private static final DateTimeFormatter formatter_d8 = DateTimeFormatter.ofPattern("yyyyMMdd");
   private static final DateTimeFormatter formatter_d10_tw = DateTimeFormatter.ofPattern("yyyy/MM/dd");
   private static final DateTimeFormatter formatter_d10_cn = DateTimeFormatter.ofPattern("yyyy年M月d日");
   private static final DateTimeFormatter formatter_d10_kr = DateTimeFormatter.ofPattern("yyyy년M월d일");
   private static final DateTimeFormatter formatter_d10_us = DateTimeFormatter.ofPattern("MM/dd/yyyy");
   private static final DateTimeFormatter formatter_d10_eur = DateTimeFormatter.ofPattern("dd/MM/yyyy");
   private static final DateTimeFormatter formatter_d10_de = DateTimeFormatter.ofPattern("dd.MM.yyyy");
   private static final DateTimeFormatter formatter_d10_in = DateTimeFormatter.ofPattern("dd-MM-yyyy");
   private static final DateTimeFormatter ISO_FIXED_FORMAT = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").withZone(ZoneId.systemDefault());
   private static final String formatter_iso8601_pattern = "yyyy-MM-dd'T'HH:mm:ss";
   private static final DateTimeFormatter formatter_iso8601 = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");

   public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName, String format, int feature) {
      JSONLexer lexer = parser.lexer;
      if (lexer.token() == 8) {
         lexer.nextToken();
         return null;
      } else if (lexer.token() == 4) {
         String text = lexer.stringVal();
         lexer.nextToken();
         DateTimeFormatter formatter = null;
         if (format != null) {
            if ("yyyy-MM-dd HH:mm:ss".equals(format)) {
               formatter = defaultFormatter;
            } else {
               formatter = DateTimeFormatter.ofPattern(format);
            }
         }

         if ("".equals(text)) {
            return null;
         } else if (type != LocalDateTime.class) {
            LocalDateTime localDateTime;
            if (type == LocalDate.class) {
               LocalDate localDate;
               if (text.length() == 23) {
                  localDateTime = LocalDateTime.parse(text);
                  localDate = LocalDate.of(localDateTime.getYear(), localDateTime.getMonthValue(), localDateTime.getDayOfMonth());
               } else {
                  localDate = this.parseLocalDate(text, format, formatter);
               }

               return localDate;
            } else if (type == LocalTime.class) {
               LocalTime localDate;
               if (text.length() == 23) {
                  localDateTime = LocalDateTime.parse(text);
                  localDate = LocalTime.of(localDateTime.getHour(), localDateTime.getMinute(), localDateTime.getSecond(), localDateTime.getNano());
               } else {
                  localDate = LocalTime.parse(text);
               }

               return localDate;
            } else if (type == ZonedDateTime.class) {
               if (formatter == defaultFormatter) {
                  formatter = ISO_FIXED_FORMAT;
               }

               ZonedDateTime zonedDateTime = this.parseZonedDateTime(text, formatter);
               return zonedDateTime;
            } else if (type == OffsetDateTime.class) {
               OffsetDateTime offsetDateTime = OffsetDateTime.parse(text);
               return offsetDateTime;
            } else if (type == OffsetTime.class) {
               OffsetTime offsetTime = OffsetTime.parse(text);
               return offsetTime;
            } else if (type == ZoneId.class) {
               ZoneId offsetTime = ZoneId.of(text);
               return offsetTime;
            } else if (type == Period.class) {
               Period period = Period.parse(text);
               return period;
            } else if (type == Duration.class) {
               Duration duration = Duration.parse(text);
               return duration;
            } else if (type == Instant.class) {
               Instant instant = Instant.parse(text);
               return instant;
            } else {
               return null;
            }
         } else {
            LocalDateTime localDateTime;
            if (text.length() != 10 && text.length() != 8) {
               localDateTime = this.parseDateTime(text, formatter);
            } else {
               LocalDate localDate = this.parseLocalDate(text, format, formatter);
               localDateTime = LocalDateTime.of(localDate, LocalTime.MIN);
            }

            return localDateTime;
         }
      } else if (lexer.token() == 2) {
         long millis = lexer.longValue();
         lexer.nextToken();
         if (type == LocalDateTime.class) {
            return LocalDateTime.ofInstant(Instant.ofEpochMilli(millis), JSON.defaultTimeZone.toZoneId());
         } else if (type == LocalDate.class) {
            return LocalDateTime.ofInstant(Instant.ofEpochMilli(millis), JSON.defaultTimeZone.toZoneId()).toLocalDate();
         } else if (type == LocalTime.class) {
            return LocalDateTime.ofInstant(Instant.ofEpochMilli(millis), JSON.defaultTimeZone.toZoneId()).toLocalTime();
         } else {
            throw new UnsupportedOperationException();
         }
      } else {
         throw new UnsupportedOperationException();
      }
   }

   protected LocalDateTime parseDateTime(String text, DateTimeFormatter formatter) {
      if (formatter == null) {
         char c4;
         if (text.length() == 19) {
            c4 = text.charAt(4);
            char c7 = text.charAt(7);
            char c10 = text.charAt(10);
            char c13 = text.charAt(13);
            char c16 = text.charAt(16);
            if (c13 == ':' && c16 == ':') {
               if (c4 == '-' && c7 == '-') {
                  if (c10 == 'T') {
                     formatter = DateTimeFormatter.ISO_LOCAL_DATE_TIME;
                  } else if (c10 == ' ') {
                     formatter = defaultFormatter;
                  }
               } else if (c4 == '-' && c7 == '-') {
                  formatter = defaultFormatter;
               } else if (c4 == '/' && c7 == '/') {
                  formatter = formatter_dt19_tw;
               } else {
                  char c0 = text.charAt(0);
                  char c1 = text.charAt(1);
                  char c2 = text.charAt(2);
                  char c3 = text.charAt(3);
                  char c5 = text.charAt(5);
                  if (c2 == '/' && c5 == '/') {
                     int v0 = (c0 - 48) * 10 + (c1 - 48);
                     int v1 = (c3 - 48) * 10 + (c4 - 48);
                     if (v0 > 12) {
                        formatter = formatter_dt19_eur;
                     } else if (v1 > 12) {
                        formatter = formatter_dt19_us;
                     } else {
                        String country = Locale.getDefault().getCountry();
                        if (country.equals("US")) {
                           formatter = formatter_dt19_us;
                        } else if (country.equals("BR") || country.equals("AU")) {
                           formatter = formatter_dt19_eur;
                        }
                     }
                  } else if (c2 == '.' && c5 == '.') {
                     formatter = formatter_dt19_de;
                  } else if (c2 == '-' && c5 == '-') {
                     formatter = formatter_dt19_in;
                  }
               }
            }
         }

         if (text.length() >= 17) {
            c4 = text.charAt(4);
            if (c4 == 24180) {
               if (text.charAt(text.length() - 1) == 31186) {
                  formatter = formatter_dt19_cn_1;
               } else {
                  formatter = formatter_dt19_cn;
               }
            } else if (c4 == '년') {
               formatter = formatter_dt19_kr;
            }
         }
      }

      return formatter == null ? LocalDateTime.parse(text) : LocalDateTime.parse(text, formatter);
   }

   protected LocalDate parseLocalDate(String text, String format, DateTimeFormatter formatter) {
      if (formatter == null) {
         if (text.length() == 8) {
            formatter = formatter_d8;
         }

         char c4;
         if (text.length() == 10) {
            c4 = text.charAt(4);
            char c7 = text.charAt(7);
            if (c4 == '/' && c7 == '/') {
               formatter = formatter_d10_tw;
            }

            char c0 = text.charAt(0);
            char c1 = text.charAt(1);
            char c2 = text.charAt(2);
            char c3 = text.charAt(3);
            char c5 = text.charAt(5);
            if (c2 == '/' && c5 == '/') {
               int v0 = (c0 - 48) * 10 + (c1 - 48);
               int v1 = (c3 - 48) * 10 + (c4 - 48);
               if (v0 > 12) {
                  formatter = formatter_d10_eur;
               } else if (v1 > 12) {
                  formatter = formatter_d10_us;
               } else {
                  String country = Locale.getDefault().getCountry();
                  if (country.equals("US")) {
                     formatter = formatter_d10_us;
                  } else if (country.equals("BR") || country.equals("AU")) {
                     formatter = formatter_d10_eur;
                  }
               }
            } else if (c2 == '.' && c5 == '.') {
               formatter = formatter_d10_de;
            } else if (c2 == '-' && c5 == '-') {
               formatter = formatter_d10_in;
            }
         }

         if (text.length() >= 9) {
            c4 = text.charAt(4);
            if (c4 == 24180) {
               formatter = formatter_d10_cn;
            } else if (c4 == '년') {
               formatter = formatter_d10_kr;
            }
         }
      }

      return formatter == null ? LocalDate.parse(text) : LocalDate.parse(text, formatter);
   }

   protected ZonedDateTime parseZonedDateTime(String text, DateTimeFormatter formatter) {
      if (formatter == null) {
         char c4;
         if (text.length() == 19) {
            c4 = text.charAt(4);
            char c7 = text.charAt(7);
            char c10 = text.charAt(10);
            char c13 = text.charAt(13);
            char c16 = text.charAt(16);
            if (c13 == ':' && c16 == ':') {
               if (c4 == '-' && c7 == '-') {
                  if (c10 == 'T') {
                     formatter = DateTimeFormatter.ISO_LOCAL_DATE_TIME;
                  } else if (c10 == ' ') {
                     formatter = defaultFormatter;
                  }
               } else if (c4 == '-' && c7 == '-') {
                  formatter = defaultFormatter;
               } else if (c4 == '/' && c7 == '/') {
                  formatter = formatter_dt19_tw;
               } else {
                  char c0 = text.charAt(0);
                  char c1 = text.charAt(1);
                  char c2 = text.charAt(2);
                  char c3 = text.charAt(3);
                  char c5 = text.charAt(5);
                  if (c2 == '/' && c5 == '/') {
                     int v0 = (c0 - 48) * 10 + (c1 - 48);
                     int v1 = (c3 - 48) * 10 + (c4 - 48);
                     if (v0 > 12) {
                        formatter = formatter_dt19_eur;
                     } else if (v1 > 12) {
                        formatter = formatter_dt19_us;
                     } else {
                        String country = Locale.getDefault().getCountry();
                        if (country.equals("US")) {
                           formatter = formatter_dt19_us;
                        } else if (country.equals("BR") || country.equals("AU")) {
                           formatter = formatter_dt19_eur;
                        }
                     }
                  } else if (c2 == '.' && c5 == '.') {
                     formatter = formatter_dt19_de;
                  } else if (c2 == '-' && c5 == '-') {
                     formatter = formatter_dt19_in;
                  }
               }
            }
         }

         if (text.length() >= 17) {
            c4 = text.charAt(4);
            if (c4 == 24180) {
               if (text.charAt(text.length() - 1) == 31186) {
                  formatter = formatter_dt19_cn_1;
               } else {
                  formatter = formatter_dt19_cn;
               }
            } else if (c4 == '년') {
               formatter = formatter_dt19_kr;
            }
         }
      }

      return formatter == null ? ZonedDateTime.parse(text) : ZonedDateTime.parse(text, formatter);
   }

   public int getFastMatchToken() {
      return 4;
   }

   public void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      SerializeWriter out = serializer.out;
      if (object == null) {
         out.writeNull();
      } else {
         if (fieldType == null) {
            fieldType = object.getClass();
         }

         if (fieldType == LocalDateTime.class) {
            int mask = SerializerFeature.UseISO8601DateFormat.getMask();
            LocalDateTime dateTime = (LocalDateTime)object;
            String format = serializer.getDateFormatPattern();
            if (format == null && (features & mask) != 0 || serializer.isEnabled(SerializerFeature.UseISO8601DateFormat)) {
               format = "yyyy-MM-dd'T'HH:mm:ss";
            }

            if (dateTime.getNano() != 0 && format == null) {
               if (out.isEnabled(SerializerFeature.WriteDateUseDateFormat)) {
                  this.write((SerializeWriter)out, (TemporalAccessor)dateTime, (String)JSON.DEFFAULT_DATE_FORMAT);
               } else {
                  out.writeLong(dateTime.atZone(JSON.defaultTimeZone.toZoneId()).toInstant().toEpochMilli());
               }
            } else {
               if (format == null) {
                  format = JSON.DEFFAULT_DATE_FORMAT;
               }

               this.write((SerializeWriter)out, (TemporalAccessor)dateTime, (String)format);
            }
         } else {
            out.writeString(object.toString());
         }
      }

   }

   public void write(JSONSerializer serializer, Object object, BeanContext context) throws IOException {
      SerializeWriter out = serializer.out;
      String format = context.getFormat();
      this.write(out, (TemporalAccessor)object, format);
   }

   private void write(SerializeWriter out, TemporalAccessor object, String format) {
      DateTimeFormatter formatter;
      if (format == "yyyy-MM-dd'T'HH:mm:ss") {
         formatter = formatter_iso8601;
      } else {
         formatter = DateTimeFormatter.ofPattern(format);
      }

      String text = formatter.format(object);
      out.writeString(text);
   }
}
