package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.util.TypeUtils;
import java.lang.reflect.Type;
import java.math.BigDecimal;

public class NumberDeserializer implements ObjectDeserializer {
   public static final NumberDeserializer instance = new NumberDeserializer();

   public <T> T deserialze(DefaultJSONParser parser, Type clazz, Object fieldName) {
      JSONLexer lexer = parser.lexer;
      String val;
      if (lexer.token() == 2) {
         if (clazz != Double.TYPE && clazz != Double.class) {
            long val = lexer.longValue();
            lexer.nextToken(16);
            if (clazz != Short.TYPE && clazz != Short.class) {
               if (clazz != Byte.TYPE && clazz != Byte.class) {
                  return val >= -2147483648L && val <= 2147483647L ? (int)val : val;
               } else if (val <= 127L && val >= -128L) {
                  return (byte)((int)val);
               } else {
                  throw new JSONException("short overflow : " + val);
               }
            } else if (val <= 32767L && val >= -32768L) {
               return (short)((int)val);
            } else {
               throw new JSONException("short overflow : " + val);
            }
         } else {
            val = lexer.numberString();
            lexer.nextToken(16);
            return Double.parseDouble(val);
         }
      } else if (lexer.token() == 3) {
         if (clazz != Double.TYPE && clazz != Double.class) {
            BigDecimal val = lexer.decimalValue();
            lexer.nextToken(16);
            if (clazz != Short.TYPE && clazz != Short.class) {
               return clazz != Byte.TYPE && clazz != Byte.class ? val : val.byteValue();
            } else if (val.compareTo(BigDecimal.valueOf(32767L)) <= 0 && val.compareTo(BigDecimal.valueOf(-32768L)) >= 0) {
               return val.shortValue();
            } else {
               throw new JSONException("short overflow : " + val);
            }
         } else {
            val = lexer.numberString();
            lexer.nextToken(16);
            return Double.parseDouble(val);
         }
      } else {
         Object value;
         if (lexer.token() == 18 && "NaN".equals(lexer.stringVal())) {
            lexer.nextToken();
            value = null;
            if (clazz == Double.class) {
               value = Double.NaN;
            } else if (clazz == Float.class) {
               value = Float.NaN;
            }

            return value;
         } else {
            value = parser.parse();
            if (value == null) {
               return null;
            } else if (clazz != Double.TYPE && clazz != Double.class) {
               if (clazz != Short.TYPE && clazz != Short.class) {
                  if (clazz != Byte.TYPE && clazz != Byte.class) {
                     return TypeUtils.castToBigDecimal(value);
                  } else {
                     try {
                        return TypeUtils.castToByte(value);
                     } catch (Exception var7) {
                        throw new JSONException("parseByte error, field : " + fieldName, var7);
                     }
                  }
               } else {
                  try {
                     return TypeUtils.castToShort(value);
                  } catch (Exception var8) {
                     throw new JSONException("parseShort error, field : " + fieldName, var8);
                  }
               }
            } else {
               try {
                  return TypeUtils.castToDouble(value);
               } catch (Exception var9) {
                  throw new JSONException("parseDouble error, field : " + fieldName, var9);
               }
            }
         }
      }
   }

   public int getFastMatchToken() {
      return 2;
   }
}
