package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONScanner;
import java.lang.reflect.Type;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;

public class SqlDateDeserializer extends AbstractDateDeserializer implements ObjectDeserializer {
   public static final SqlDateDeserializer instance = new SqlDateDeserializer();
   public static final SqlDateDeserializer instance_timestamp = new SqlDateDeserializer(true);
   private boolean timestamp = false;

   public SqlDateDeserializer() {
   }

   public SqlDateDeserializer(boolean timestmap) {
      this.timestamp = true;
   }

   protected <T> T cast(DefaultJSONParser parser, Type clazz, Object fieldName, Object val) {
      if (this.timestamp) {
         return this.castTimestamp(parser, clazz, fieldName, val);
      } else if (val == null) {
         return null;
      } else {
         Date val;
         if (val instanceof java.util.Date) {
            val = new Date(((java.util.Date)val).getTime());
         } else {
            if (!(val instanceof Number)) {
               if (!(val instanceof String)) {
                  throw new JSONException("parse error : " + val);
               }

               String strVal = (String)val;
               if (strVal.length() == 0) {
                  return null;
               }

               JSONScanner dateLexer = new JSONScanner(strVal);

               long longVal;
               try {
                  if (dateLexer.scanISO8601DateIfMatch()) {
                     longVal = dateLexer.getCalendar().getTimeInMillis();
                  } else {
                     DateFormat dateFormat = parser.getDateFormat();

                     try {
                        java.util.Date date = dateFormat.parse(strVal);
                        Date sqlDate = new Date(date.getTime());
                        Date var12 = sqlDate;
                        return var12;
                     } catch (ParseException var16) {
                        longVal = Long.parseLong(strVal);
                     }
                  }
               } finally {
                  dateLexer.close();
               }

               return new Date(longVal);
            }

            val = new Date(((Number)val).longValue());
         }

         return val;
      }
   }

   protected <T> T castTimestamp(DefaultJSONParser parser, Type clazz, Object fieldName, Object val) {
      if (val == null) {
         return null;
      } else if (val instanceof java.util.Date) {
         return new Timestamp(((java.util.Date)val).getTime());
      } else if (val instanceof Number) {
         return new Timestamp(((Number)val).longValue());
      } else if (!(val instanceof String)) {
         throw new JSONException("parse error");
      } else {
         String strVal = (String)val;
         if (strVal.length() == 0) {
            return null;
         } else {
            JSONScanner dateLexer = new JSONScanner(strVal);

            long longVal;
            try {
               if (dateLexer.scanISO8601DateIfMatch()) {
                  longVal = dateLexer.getCalendar().getTimeInMillis();
               } else {
                  DateFormat dateFormat = parser.getDateFormat();

                  try {
                     java.util.Date date = dateFormat.parse(strVal);
                     Timestamp sqlDate = new Timestamp(date.getTime());
                     Timestamp var12 = sqlDate;
                     return var12;
                  } catch (ParseException var16) {
                     longVal = Long.parseLong(strVal);
                  }
               }
            } finally {
               dateLexer.close();
            }

            return new Timestamp(longVal);
         }
      }
   }

   public int getFastMatchToken() {
      return 2;
   }
}
