package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.IOException;
import java.lang.reflect.Type;
import java.math.BigDecimal;

public class BigDecimalCodec implements ObjectSerializer, ObjectDeserializer {
   public static final BigDecimalCodec instance = new BigDecimalCodec();

   public void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      SerializeWriter out = serializer.out;
      if (object == null) {
         out.writeNull(SerializerFeature.WriteNullNumberAsZero);
      } else {
         BigDecimal val = (BigDecimal)object;
         String outText;
         if (out.isEnabled(SerializerFeature.WriteBigDecimalAsPlain)) {
            outText = val.toPlainString();
         } else {
            outText = val.toString();
         }

         out.write(outText);
         if (out.isEnabled(SerializerFeature.WriteClassName) && fieldType != BigDecimal.class && val.scale() == 0) {
            out.write(46);
         }
      }

   }

   public <T> T deserialze(DefaultJSONParser parser, Type clazz, Object fieldName) {
      try {
         return deserialze(parser);
      } catch (Exception var5) {
         throw new JSONException("parseDecimal error, field : " + fieldName, var5);
      }
   }

   public static <T> T deserialze(DefaultJSONParser parser) {
      JSONLexer lexer = parser.lexer;
      BigDecimal val;
      if (lexer.token() == 2) {
         val = lexer.decimalValue();
         lexer.nextToken(16);
         return val;
      } else if (lexer.token() == 3) {
         val = lexer.decimalValue();
         lexer.nextToken(16);
         return val;
      } else {
         Object value = parser.parse();
         return value == null ? null : TypeUtils.castToBigDecimal(value);
      }
   }

   public int getFastMatchToken() {
      return 2;
   }
}
