package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import com.alibaba.fastjson.util.IOUtils;
import java.io.IOException;
import java.lang.reflect.Type;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

public class CalendarCodec implements ObjectSerializer, ObjectDeserializer {
   public static final CalendarCodec instance = new CalendarCodec();
   private DatatypeFactory dateFactory;

   public void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      SerializeWriter out = serializer.out;
      if (object == null) {
         out.writeNull();
      } else {
         Object calendar;
         if (object instanceof XMLGregorianCalendar) {
            calendar = ((XMLGregorianCalendar)object).toGregorianCalendar();
         } else {
            calendar = (Calendar)object;
         }

         if (out.isEnabled(SerializerFeature.UseISO8601DateFormat)) {
            char quote = out.isEnabled(SerializerFeature.UseSingleQuotes) ? 39 : 34;
            out.append((char)quote);
            int year = ((Calendar)calendar).get(1);
            int month = ((Calendar)calendar).get(2) + 1;
            int day = ((Calendar)calendar).get(5);
            int hour = ((Calendar)calendar).get(11);
            int minute = ((Calendar)calendar).get(12);
            int second = ((Calendar)calendar).get(13);
            int millis = ((Calendar)calendar).get(14);
            char[] buf;
            if (millis != 0) {
               buf = "0000-00-00T00:00:00.000".toCharArray();
               IOUtils.getChars((int)millis, 23, buf);
               IOUtils.getChars((int)second, 19, buf);
               IOUtils.getChars((int)minute, 16, buf);
               IOUtils.getChars((int)hour, 13, buf);
               IOUtils.getChars((int)day, 10, buf);
               IOUtils.getChars((int)month, 7, buf);
               IOUtils.getChars((int)year, 4, buf);
            } else if (second == 0 && minute == 0 && hour == 0) {
               buf = "0000-00-00".toCharArray();
               IOUtils.getChars((int)day, 10, buf);
               IOUtils.getChars((int)month, 7, buf);
               IOUtils.getChars((int)year, 4, buf);
            } else {
               buf = "0000-00-00T00:00:00".toCharArray();
               IOUtils.getChars((int)second, 19, buf);
               IOUtils.getChars((int)minute, 16, buf);
               IOUtils.getChars((int)hour, 13, buf);
               IOUtils.getChars((int)day, 10, buf);
               IOUtils.getChars((int)month, 7, buf);
               IOUtils.getChars((int)year, 4, buf);
            }

            out.write((char[])buf);
            int timeZone = ((Calendar)calendar).getTimeZone().getRawOffset() / 3600000;
            if (timeZone == 0) {
               out.append("Z");
            } else if (timeZone > 0) {
               out.append("+").append(String.format("%02d", timeZone)).append(":00");
            } else {
               out.append("-").append(String.format("%02d", -timeZone)).append(":00");
            }

            out.append((char)quote);
         } else {
            Date date = ((Calendar)calendar).getTime();
            serializer.write((Object)date);
         }

      }
   }

   public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName) {
      Object value = DateCodec.instance.deserialze(parser, type, fieldName);
      if (value instanceof Calendar) {
         return value;
      } else {
         Date date = (Date)value;
         if (date == null) {
            return null;
         } else {
            JSONLexer lexer = parser.lexer;
            Calendar calendar = Calendar.getInstance(lexer.getTimeZone(), lexer.getLocale());
            calendar.setTime(date);
            return type == XMLGregorianCalendar.class ? this.createXMLGregorianCalendar((GregorianCalendar)calendar) : calendar;
         }
      }
   }

   public XMLGregorianCalendar createXMLGregorianCalendar(Calendar calendar) {
      if (this.dateFactory == null) {
         try {
            this.dateFactory = DatatypeFactory.newInstance();
         } catch (DatatypeConfigurationException var3) {
            throw new IllegalStateException("Could not obtain an instance of DatatypeFactory.", var3);
         }
      }

      return this.dateFactory.newXMLGregorianCalendar((GregorianCalendar)calendar);
   }

   public int getFastMatchToken() {
      return 2;
   }
}
