package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Iterator;

public class CharArrayCodec implements ObjectDeserializer {
   public <T> T deserialze(DefaultJSONParser parser, Type clazz, Object fieldName) {
      return deserialze(parser);
   }

   public static <T> T deserialze(DefaultJSONParser parser) {
      JSONLexer lexer = parser.lexer;
      if (lexer.token() == 4) {
         String val = lexer.stringVal();
         lexer.nextToken(16);
         return val.toCharArray();
      } else if (lexer.token() == 2) {
         Number val = lexer.integerValue();
         lexer.nextToken(16);
         return val.toString().toCharArray();
      } else {
         Object value = parser.parse();
         if (value instanceof String) {
            return ((String)value).toCharArray();
         } else if (!(value instanceof Collection)) {
            return value == null ? null : JSON.toJSONString(value).toCharArray();
         } else {
            Collection<?> collection = (Collection)value;
            boolean accept = true;
            Iterator var5 = collection.iterator();

            while(var5.hasNext()) {
               Object item = var5.next();
               if (item instanceof String) {
                  int itemLength = ((String)item).length();
                  if (itemLength != 1) {
                     accept = false;
                     break;
                  }
               }
            }

            if (!accept) {
               throw new JSONException("can not cast to char[]");
            } else {
               char[] chars = new char[collection.size()];
               int pos = 0;

               Object item;
               for(Iterator var13 = collection.iterator(); var13.hasNext(); chars[pos++] = ((String)item).charAt(0)) {
                  item = var13.next();
               }

               return chars;
            }
         }
      }
   }

   public int getFastMatchToken() {
      return 4;
   }
}
