package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONScanner;
import com.alibaba.fastjson.parser.deserializer.AbstractDateDeserializer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import com.alibaba.fastjson.util.IOUtils;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.IOException;
import java.lang.reflect.Type;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

public class DateCodec extends AbstractDateDeserializer implements ObjectSerializer, ObjectDeserializer {
   public static final DateCodec instance = new DateCodec();

   public void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      SerializeWriter out = serializer.out;
      if (object == null) {
         out.writeNull();
      } else {
         Date date;
         if (object instanceof Date) {
            date = (Date)object;
         } else {
            date = TypeUtils.castToDate(object);
         }

         if (out.isEnabled(SerializerFeature.WriteDateUseDateFormat)) {
            DateFormat format = serializer.getDateFormat();
            if (format == null) {
               format = new SimpleDateFormat(JSON.DEFFAULT_DATE_FORMAT, serializer.locale);
               ((DateFormat)format).setTimeZone(serializer.timeZone);
            }

            String text = ((DateFormat)format).format(date);
            out.writeString(text);
         } else if (out.isEnabled(SerializerFeature.WriteClassName) && object.getClass() != fieldType) {
            if (object.getClass() == Date.class) {
               out.write("new Date(");
               out.writeLong(((Date)object).getTime());
               out.write(41);
            } else {
               out.write(123);
               out.writeFieldName(JSON.DEFAULT_TYPE_KEY);
               serializer.write(object.getClass().getName());
               out.writeFieldValue(',', "val", ((Date)object).getTime());
               out.write(125);
            }

         } else {
            long time = date.getTime();
            if (out.isEnabled(SerializerFeature.UseISO8601DateFormat)) {
               char quote = out.isEnabled(SerializerFeature.UseSingleQuotes) ? 39 : 34;
               out.write(quote);
               Calendar calendar = Calendar.getInstance(serializer.timeZone, serializer.locale);
               calendar.setTimeInMillis(time);
               int year = calendar.get(1);
               int month = calendar.get(2) + 1;
               int day = calendar.get(5);
               int hour = calendar.get(11);
               int minute = calendar.get(12);
               int second = calendar.get(13);
               int millis = calendar.get(14);
               char[] buf;
               if (millis != 0) {
                  buf = "0000-00-00T00:00:00.000".toCharArray();
                  IOUtils.getChars((int)millis, 23, buf);
                  IOUtils.getChars((int)second, 19, buf);
                  IOUtils.getChars((int)minute, 16, buf);
                  IOUtils.getChars((int)hour, 13, buf);
                  IOUtils.getChars((int)day, 10, buf);
                  IOUtils.getChars((int)month, 7, buf);
                  IOUtils.getChars((int)year, 4, buf);
               } else if (second == 0 && minute == 0 && hour == 0) {
                  buf = "0000-00-00".toCharArray();
                  IOUtils.getChars((int)day, 10, buf);
                  IOUtils.getChars((int)month, 7, buf);
                  IOUtils.getChars((int)year, 4, buf);
               } else {
                  buf = "0000-00-00T00:00:00".toCharArray();
                  IOUtils.getChars((int)second, 19, buf);
                  IOUtils.getChars((int)minute, 16, buf);
                  IOUtils.getChars((int)hour, 13, buf);
                  IOUtils.getChars((int)day, 10, buf);
                  IOUtils.getChars((int)month, 7, buf);
                  IOUtils.getChars((int)year, 4, buf);
               }

               out.write((char[])buf);
               int timeZone = calendar.getTimeZone().getRawOffset() / 3600000;
               if (timeZone == 0) {
                  out.write(90);
               } else {
                  if (timeZone > 9) {
                     out.write(43);
                     out.writeInt(timeZone);
                  } else if (timeZone > 0) {
                     out.write(43);
                     out.write(48);
                     out.writeInt(timeZone);
                  } else if (timeZone < -9) {
                     out.write(45);
                     out.writeInt(timeZone);
                  } else if (timeZone < 0) {
                     out.write(45);
                     out.write(48);
                     out.writeInt(timeZone);
                  }

                  out.append(":00");
               }

               out.write(quote);
            } else {
               out.writeLong(time);
            }

         }
      }
   }

   public <T> T cast(DefaultJSONParser parser, Type clazz, Object fieldName, Object val) {
      if (val == null) {
         return null;
      } else if (val instanceof Date) {
         return val;
      } else if (val instanceof Number) {
         return new Date(((Number)val).longValue());
      } else if (!(val instanceof String)) {
         throw new JSONException("parse error");
      } else {
         String strVal = (String)val;
         if (strVal.length() == 0) {
            return null;
         } else {
            JSONScanner dateLexer = new JSONScanner(strVal);

            label346: {
               Date var8;
               try {
                  if (!dateLexer.scanISO8601DateIfMatch(false)) {
                     break label346;
                  }

                  Calendar calendar = dateLexer.getCalendar();
                  if (clazz == Calendar.class) {
                     Calendar var28 = calendar;
                     return var28;
                  }

                  var8 = calendar.getTime();
               } finally {
                  dateLexer.close();
               }

               return var8;
            }

            if (strVal.length() == parser.getDateFomartPattern().length() || strVal.length() == 22 && parser.getDateFomartPattern().equals("yyyyMMddHHmmssSSSZ")) {
               DateFormat dateFormat = parser.getDateFormat();

               try {
                  return dateFormat.parse(strVal);
               } catch (ParseException var21) {
               }
            }

            if (strVal.startsWith("/Date(") && strVal.endsWith(")/")) {
               String dotnetDateStr = strVal.substring(6, strVal.length() - 2);
               strVal = dotnetDateStr;
            }

            if (!"0000-00-00".equals(strVal) && !"0000-00-00T00:00:00".equalsIgnoreCase(strVal) && !"0001-01-01T00:00:00+08:00".equalsIgnoreCase(strVal)) {
               int index = strVal.lastIndexOf(124);
               if (index > 20) {
                  String tzStr = strVal.substring(index + 1);
                  TimeZone timeZone = TimeZone.getTimeZone(tzStr);
                  if (!"GMT".equals(timeZone.getID())) {
                     String subStr = strVal.substring(0, index);
                     JSONScanner dateLexer = new JSONScanner(subStr);

                     try {
                        if (dateLexer.scanISO8601DateIfMatch(false)) {
                           Calendar calendar = dateLexer.getCalendar();
                           calendar.setTimeZone(timeZone);
                           if (clazz == Calendar.class) {
                              Calendar var30 = calendar;
                              return var30;
                           }

                           Date var12 = calendar.getTime();
                           return var12;
                        }
                     } finally {
                        dateLexer.close();
                     }
                  }
               }

               long longVal = Long.parseLong(strVal);
               return new Date(longVal);
            } else {
               return null;
            }
         }
      }
   }

   public int getFastMatchToken() {
      return 2;
   }
}
