package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.annotation.JSONType;
import com.alibaba.fastjson.util.FieldInfo;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;

public class FieldSerializer implements Comparable<FieldSerializer> {
   public final FieldInfo fieldInfo;
   protected final boolean writeNull;
   protected int features;
   private final String double_quoted_fieldPrefix;
   private String single_quoted_fieldPrefix;
   private String un_quoted_fieldPrefix;
   protected BeanContext fieldContext;
   private String format;
   protected boolean writeEnumUsingToString = false;
   protected boolean writeEnumUsingName = false;
   protected boolean disableCircularReferenceDetect = false;
   protected boolean serializeUsing = false;
   protected boolean persistenceXToMany = false;
   protected boolean browserCompatible;
   private FieldSerializer.RuntimeSerializerInfo runtimeInfo;

   public FieldSerializer(Class<?> beanType, FieldInfo fieldInfo) {
      this.fieldInfo = fieldInfo;
      this.fieldContext = new BeanContext(beanType, fieldInfo);
      int var6;
      if (beanType != null && (fieldInfo.isEnum || fieldInfo.fieldClass == Long.TYPE || fieldInfo.fieldClass == Long.class)) {
         JSONType jsonType = (JSONType)TypeUtils.getAnnotation(beanType, JSONType.class);
         if (jsonType != null) {
            SerializerFeature[] var4 = jsonType.serialzeFeatures();
            int var5 = var4.length;

            for(var6 = 0; var6 < var5; ++var6) {
               SerializerFeature feature = var4[var6];
               if (feature == SerializerFeature.WriteEnumUsingToString) {
                  this.writeEnumUsingToString = true;
               } else if (feature == SerializerFeature.WriteEnumUsingName) {
                  this.writeEnumUsingName = true;
               } else if (feature == SerializerFeature.DisableCircularReferenceDetect) {
                  this.disableCircularReferenceDetect = true;
               } else if (feature == SerializerFeature.BrowserCompatible) {
                  this.browserCompatible = true;
               }
            }
         }
      }

      fieldInfo.setAccessible();
      this.double_quoted_fieldPrefix = '"' + fieldInfo.name + "\":";
      boolean writeNull = false;
      JSONField annotation = fieldInfo.getAnnotation();
      if (annotation != null) {
         SerializerFeature[] var11 = annotation.serialzeFeatures();
         var6 = var11.length;

         SerializerFeature feature;
         int var12;
         for(var12 = 0; var12 < var6; ++var12) {
            feature = var11[var12];
            if ((feature.getMask() & SerializerFeature.WRITE_MAP_NULL_FEATURES) != 0) {
               writeNull = true;
               break;
            }
         }

         this.format = annotation.format();
         if (this.format.trim().length() == 0) {
            this.format = null;
         }

         var11 = annotation.serialzeFeatures();
         var6 = var11.length;

         for(var12 = 0; var12 < var6; ++var12) {
            feature = var11[var12];
            if (feature == SerializerFeature.WriteEnumUsingToString) {
               this.writeEnumUsingToString = true;
            } else if (feature == SerializerFeature.WriteEnumUsingName) {
               this.writeEnumUsingName = true;
            } else if (feature == SerializerFeature.DisableCircularReferenceDetect) {
               this.disableCircularReferenceDetect = true;
            } else if (feature == SerializerFeature.BrowserCompatible) {
               this.browserCompatible = true;
            }
         }

         this.features = SerializerFeature.of(annotation.serialzeFeatures());
      }

      this.writeNull = writeNull;
      this.persistenceXToMany = TypeUtils.isAnnotationPresentOneToMany(fieldInfo.method) || TypeUtils.isAnnotationPresentManyToMany(fieldInfo.method);
   }

   public void writePrefix(JSONSerializer serializer) throws IOException {
      SerializeWriter out = serializer.out;
      if (out.quoteFieldNames) {
         if (out.useSingleQuotes) {
            if (this.single_quoted_fieldPrefix == null) {
               this.single_quoted_fieldPrefix = '\'' + this.fieldInfo.name + "':";
            }

            out.write(this.single_quoted_fieldPrefix);
         } else {
            out.write(this.double_quoted_fieldPrefix);
         }
      } else {
         if (this.un_quoted_fieldPrefix == null) {
            this.un_quoted_fieldPrefix = this.fieldInfo.name + ":";
         }

         out.write(this.un_quoted_fieldPrefix);
      }

   }

   public Object getPropertyValueDirect(Object object) throws InvocationTargetException, IllegalAccessException {
      Object fieldValue = this.fieldInfo.get(object);
      return this.persistenceXToMany && !TypeUtils.isHibernateInitialized(fieldValue) ? null : fieldValue;
   }

   public Object getPropertyValue(Object object) throws InvocationTargetException, IllegalAccessException {
      Object propertyValue = this.fieldInfo.get(object);
      if (this.format != null && propertyValue != null && this.fieldInfo.fieldClass == Date.class) {
         SimpleDateFormat dateFormat = new SimpleDateFormat(this.format);
         dateFormat.setTimeZone(JSON.defaultTimeZone);
         return dateFormat.format(propertyValue);
      } else {
         return propertyValue;
      }
   }

   public int compareTo(FieldSerializer o) {
      return this.fieldInfo.compareTo(o.fieldInfo);
   }

   public void writeValue(JSONSerializer serializer, Object propertyValue) throws Exception {
      if (this.runtimeInfo == null) {
         Class runtimeFieldClass;
         if (propertyValue == null) {
            runtimeFieldClass = this.fieldInfo.fieldClass;
         } else {
            runtimeFieldClass = propertyValue.getClass();
         }

         ObjectSerializer fieldSerializer = null;
         JSONField fieldAnnotation = this.fieldInfo.getAnnotation();
         if (fieldAnnotation != null && fieldAnnotation.serializeUsing() != Void.class) {
            fieldSerializer = (ObjectSerializer)fieldAnnotation.serializeUsing().newInstance();
            this.serializeUsing = true;
         } else {
            if (this.format != null) {
               if (runtimeFieldClass != Double.TYPE && runtimeFieldClass != Double.class) {
                  if (runtimeFieldClass == Float.TYPE || runtimeFieldClass == Float.class) {
                     fieldSerializer = new FloatCodec(this.format);
                  }
               } else {
                  fieldSerializer = new DoubleSerializer(this.format);
               }
            }

            if (fieldSerializer == null) {
               fieldSerializer = serializer.getObjectWriter(runtimeFieldClass);
            }
         }

         this.runtimeInfo = new FieldSerializer.RuntimeSerializerInfo((ObjectSerializer)fieldSerializer, runtimeFieldClass);
      }

      FieldSerializer.RuntimeSerializerInfo runtimeInfo = this.runtimeInfo;
      int fieldFeatures = this.disableCircularReferenceDetect ? this.fieldInfo.serialzeFeatures | SerializerFeature.DisableCircularReferenceDetect.getMask() : this.fieldInfo.serialzeFeatures;
      if (propertyValue == null) {
         SerializeWriter out = serializer.out;
         if (this.fieldInfo.fieldClass == Object.class && out.isEnabled(SerializerFeature.WRITE_MAP_NULL_FEATURES)) {
            out.writeNull();
         } else {
            Class<?> runtimeFieldClass = runtimeInfo.runtimeFieldClass;
            if (Number.class.isAssignableFrom(runtimeFieldClass)) {
               out.writeNull(this.features, SerializerFeature.WriteNullNumberAsZero.mask);
            } else if (String.class == runtimeFieldClass) {
               out.writeNull(this.features, SerializerFeature.WriteNullStringAsEmpty.mask);
            } else if (Boolean.class == runtimeFieldClass) {
               out.writeNull(this.features, SerializerFeature.WriteNullBooleanAsFalse.mask);
            } else if (Collection.class.isAssignableFrom(runtimeFieldClass)) {
               out.writeNull(this.features, SerializerFeature.WriteNullListAsEmpty.mask);
            } else {
               ObjectSerializer fieldSerializer = runtimeInfo.fieldSerializer;
               if (out.isEnabled(SerializerFeature.WRITE_MAP_NULL_FEATURES) && fieldSerializer instanceof JavaBeanSerializer) {
                  out.writeNull();
               } else {
                  fieldSerializer.write(serializer, (Object)null, this.fieldInfo.name, this.fieldInfo.fieldType, fieldFeatures);
               }
            }
         }
      } else {
         if (this.fieldInfo.isEnum) {
            if (this.writeEnumUsingName) {
               serializer.out.writeString(((Enum)propertyValue).name());
               return;
            }

            if (this.writeEnumUsingToString) {
               serializer.out.writeString(((Enum)propertyValue).toString());
               return;
            }
         }

         Class<?> valueClass = propertyValue.getClass();
         ObjectSerializer valueSerializer;
         if (valueClass != runtimeInfo.runtimeFieldClass && !this.serializeUsing) {
            valueSerializer = serializer.getObjectWriter(valueClass);
         } else {
            valueSerializer = runtimeInfo.fieldSerializer;
         }

         if (this.format != null && !(valueSerializer instanceof DoubleSerializer) && !(valueSerializer instanceof FloatCodec)) {
            if (valueSerializer instanceof ContextObjectSerializer) {
               ((ContextObjectSerializer)valueSerializer).write(serializer, propertyValue, this.fieldContext);
            } else {
               serializer.writeWithFormat(propertyValue, this.format);
            }

         } else {
            if (this.fieldInfo.unwrapped) {
               if (valueSerializer instanceof JavaBeanSerializer) {
                  JavaBeanSerializer javaBeanSerializer = (JavaBeanSerializer)valueSerializer;
                  javaBeanSerializer.write(serializer, propertyValue, this.fieldInfo.name, this.fieldInfo.fieldType, fieldFeatures, true);
                  return;
               }

               if (valueSerializer instanceof MapSerializer) {
                  MapSerializer mapSerializer = (MapSerializer)valueSerializer;
                  mapSerializer.write(serializer, propertyValue, this.fieldInfo.name, this.fieldInfo.fieldType, fieldFeatures, true);
                  return;
               }
            }

            if ((this.features & SerializerFeature.WriteClassName.mask) != 0 && valueClass != this.fieldInfo.fieldClass && JavaBeanSerializer.class.isInstance(valueSerializer)) {
               ((JavaBeanSerializer)valueSerializer).write(serializer, propertyValue, this.fieldInfo.name, this.fieldInfo.fieldType, fieldFeatures, false);
            } else {
               if (this.browserCompatible && propertyValue != null && (this.fieldInfo.fieldClass == Long.TYPE || this.fieldInfo.fieldClass == Long.class)) {
                  long value = (Long)propertyValue;
                  if (value > 9007199254740991L || value < -9007199254740991L) {
                     serializer.getWriter().writeString(Long.toString(value));
                     return;
                  }
               }

               valueSerializer.write(serializer, propertyValue, this.fieldInfo.name, this.fieldInfo.fieldType, fieldFeatures);
            }
         }
      }
   }

   static class RuntimeSerializerInfo {
      final ObjectSerializer fieldSerializer;
      final Class<?> runtimeFieldClass;

      public RuntimeSerializerInfo(ObjectSerializer fieldSerializer, Class<?> runtimeFieldClass) {
         this.fieldSerializer = fieldSerializer;
         this.runtimeFieldClass = runtimeFieldClass;
      }
   }
}
