package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.util.FieldInfo;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Type;
import java.lang.reflect.WildcardType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class JavaBeanSerializer extends SerializeFilterable implements ObjectSerializer {
   protected final FieldSerializer[] getters;
   protected final FieldSerializer[] sortedGetters;
   protected SerializeBeanInfo beanInfo;
   private transient volatile long[] hashArray;
   private transient volatile short[] hashArrayMapping;

   public JavaBeanSerializer(Class<?> beanType) {
      this(beanType, (Map)null);
   }

   public JavaBeanSerializer(Class<?> beanType, String... aliasList) {
      this(beanType, createAliasMap(aliasList));
   }

   static Map<String, String> createAliasMap(String... aliasList) {
      Map<String, String> aliasMap = new HashMap();
      String[] var2 = aliasList;
      int var3 = aliasList.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         String alias = var2[var4];
         aliasMap.put(alias, alias);
      }

      return aliasMap;
   }

   public Class<?> getType() {
      return this.beanInfo.beanType;
   }

   public JavaBeanSerializer(Class<?> beanType, Map<String, String> aliasMap) {
      this(TypeUtils.buildBeanInfo(beanType, aliasMap, (PropertyNamingStrategy)null));
   }

   public JavaBeanSerializer(SerializeBeanInfo beanInfo) {
      this.beanInfo = beanInfo;
      this.sortedGetters = new FieldSerializer[beanInfo.sortedFields.length];

      int i;
      for(i = 0; i < this.sortedGetters.length; ++i) {
         this.sortedGetters[i] = new FieldSerializer(beanInfo.beanType, beanInfo.sortedFields[i]);
      }

      if (beanInfo.fields == beanInfo.sortedFields) {
         this.getters = this.sortedGetters;
      } else {
         this.getters = new FieldSerializer[beanInfo.fields.length];

         for(i = 0; i < this.getters.length; ++i) {
            this.getters[i] = this.getFieldSerializer(beanInfo.fields[i].name);
         }
      }

   }

   public void writeDirectNonContext(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      this.write(serializer, object, fieldName, fieldType, features);
   }

   public void writeAsArray(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      this.write(serializer, object, fieldName, fieldType, features);
   }

   public void writeAsArrayNonContext(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      this.write(serializer, object, fieldName, fieldType, features);
   }

   public void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      this.write(serializer, object, fieldName, fieldType, features, false);
   }

   public void writeNoneASM(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      this.write(serializer, object, fieldName, fieldType, features, false);
   }

   protected void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features, boolean unwrapped) throws IOException {
      SerializeWriter out = serializer.out;
      if (object == null) {
         out.writeNull();
      } else if (!this.writeReference(serializer, object, features)) {
         FieldSerializer[] getters;
         if (out.sortField) {
            getters = this.sortedGetters;
         } else {
            getters = this.getters;
         }

         SerialContext parent = serializer.context;
         if (!this.beanInfo.beanType.isEnum()) {
            serializer.setContext(parent, object, fieldName, this.beanInfo.features, features);
         }

         boolean writeAsArray = this.isWriteAsArray(serializer, features);

         try {
            char startSeperator = writeAsArray ? 91 : 123;
            char endSeperator = writeAsArray ? 93 : 125;
            if (!unwrapped) {
               out.append((char)startSeperator);
            }

            if (getters.length > 0 && out.isEnabled(SerializerFeature.PrettyFormat)) {
               serializer.incrementIndent();
               serializer.println();
            }

            boolean commaFlag = false;
            if ((this.beanInfo.features & SerializerFeature.WriteClassName.mask) != 0 || (features & SerializerFeature.WriteClassName.mask) != 0 || serializer.isWriteClassName(fieldType, object)) {
               Class<?> objClass = object.getClass();
               Object type;
               if (objClass != fieldType && fieldType instanceof WildcardType) {
                  type = TypeUtils.getClass(fieldType);
               } else {
                  type = fieldType;
               }

               if (objClass != type) {
                  this.writeClassName(serializer, this.beanInfo.typeKey, object);
                  commaFlag = true;
               }
            }

            char seperator = commaFlag ? 44 : 0;
            boolean directWritePrefix = out.quoteFieldNames && !out.useSingleQuotes;
            char newSeperator = this.writeBefore(serializer, object, (char)seperator);
            commaFlag = newSeperator == ',';
            boolean skipTransient = out.isEnabled(SerializerFeature.SkipTransientField);
            boolean ignoreNonFieldGetter = out.isEnabled(SerializerFeature.IgnoreNonFieldGetter);
            int i = 0;

            while(true) {
               if (i >= getters.length) {
                  this.writeAfter(serializer, object, (char)(commaFlag ? ',' : '\u0000'));
                  if (getters.length > 0 && out.isEnabled(SerializerFeature.PrettyFormat)) {
                     serializer.decrementIdent();
                     serializer.println();
                  }

                  if (!unwrapped) {
                     out.append((char)endSeperator);
                  }
                  break;
               }

               FieldSerializer fieldSerializer = getters[i];
               Field field = fieldSerializer.fieldInfo.field;
               FieldInfo fieldInfo = fieldSerializer.fieldInfo;
               String fieldInfoName = fieldInfo.name;
               Class<?> fieldClass = fieldInfo.fieldClass;
               if ((!skipTransient || field == null || !fieldInfo.fieldTransient) && (!ignoreNonFieldGetter || field != null)) {
                  label966: {
                     boolean notApply = false;
                     if (!this.applyName(serializer, object, fieldInfoName) || !this.applyLabel(serializer, fieldInfo.label)) {
                        if (!writeAsArray) {
                           break label966;
                        }

                        notApply = true;
                     }

                     if (this.beanInfo.typeKey == null || !fieldInfoName.equals(this.beanInfo.typeKey) || !serializer.isWriteClassName(fieldType, object)) {
                        Object propertyValue;
                        if (notApply) {
                           propertyValue = null;
                        } else {
                           try {
                              propertyValue = fieldSerializer.getPropertyValueDirect(object);
                           } catch (InvocationTargetException var38) {
                              if (!out.isEnabled(SerializerFeature.IgnoreErrorGetter)) {
                                 throw var38;
                              }

                              propertyValue = null;
                           }
                        }

                        if (this.apply(serializer, object, fieldInfoName, propertyValue)) {
                           label951: {
                              if (fieldClass == String.class && "trim".equals(fieldInfo.format) && propertyValue != null) {
                                 propertyValue = ((String)propertyValue).trim();
                              }

                              String key = this.processKey(serializer, object, fieldInfoName, propertyValue);
                              Object originalValue = propertyValue;
                              propertyValue = this.processValue(serializer, fieldSerializer.fieldContext, object, fieldInfoName, propertyValue);
                              if (propertyValue == null) {
                                 int serialzeFeatures = fieldInfo.serialzeFeatures;
                                 if (this.beanInfo.jsonType != null) {
                                    serialzeFeatures |= SerializerFeature.of(this.beanInfo.jsonType.serialzeFeatures());
                                 }

                                 int defaultMask;
                                 int mask;
                                 if (fieldClass == Boolean.class) {
                                    defaultMask = SerializerFeature.WriteNullBooleanAsFalse.mask;
                                    mask = defaultMask | SerializerFeature.WriteMapNullValue.mask;
                                    if (!writeAsArray && (serialzeFeatures & mask) == 0 && (out.features & mask) == 0) {
                                       break label951;
                                    }

                                    if ((serialzeFeatures & defaultMask) != 0 || (out.features & defaultMask) != 0) {
                                       propertyValue = false;
                                    }
                                 } else if (fieldClass == String.class) {
                                    defaultMask = SerializerFeature.WriteNullStringAsEmpty.mask;
                                    mask = defaultMask | SerializerFeature.WriteMapNullValue.mask;
                                    if (!writeAsArray && (serialzeFeatures & mask) == 0 && (out.features & mask) == 0) {
                                       break label951;
                                    }

                                    if ((serialzeFeatures & defaultMask) != 0 || (out.features & defaultMask) != 0) {
                                       propertyValue = "";
                                    }
                                 } else if (Number.class.isAssignableFrom(fieldClass)) {
                                    defaultMask = SerializerFeature.WriteNullNumberAsZero.mask;
                                    mask = defaultMask | SerializerFeature.WriteMapNullValue.mask;
                                    if (!writeAsArray && (serialzeFeatures & mask) == 0 && (out.features & mask) == 0) {
                                       break label951;
                                    }

                                    if ((serialzeFeatures & defaultMask) != 0 || (out.features & defaultMask) != 0) {
                                       propertyValue = 0;
                                    }
                                 } else if (Collection.class.isAssignableFrom(fieldClass)) {
                                    defaultMask = SerializerFeature.WriteNullListAsEmpty.mask;
                                    mask = defaultMask | SerializerFeature.WriteMapNullValue.mask;
                                    if (!writeAsArray && (serialzeFeatures & mask) == 0 && (out.features & mask) == 0) {
                                       break label951;
                                    }

                                    if ((serialzeFeatures & defaultMask) != 0 || (out.features & defaultMask) != 0) {
                                       propertyValue = Collections.emptyList();
                                    }
                                 } else if (!writeAsArray && !fieldSerializer.writeNull && !out.isEnabled(SerializerFeature.WriteMapNullValue.mask)) {
                                    break label951;
                                 }
                              }

                              if (propertyValue != null && (out.notWriteDefaultValue || (fieldInfo.serialzeFeatures & SerializerFeature.NotWriteDefaultValue.mask) != 0 || (this.beanInfo.features & SerializerFeature.NotWriteDefaultValue.mask) != 0)) {
                                 Class<?> fieldCLass = fieldInfo.fieldClass;
                                 if (fieldCLass == Byte.TYPE && propertyValue instanceof Byte && (Byte)propertyValue == 0 || fieldCLass == Short.TYPE && propertyValue instanceof Short && (Short)propertyValue == 0 || fieldCLass == Integer.TYPE && propertyValue instanceof Integer && (Integer)propertyValue == 0 || fieldCLass == Long.TYPE && propertyValue instanceof Long && (Long)propertyValue == 0L || fieldCLass == Float.TYPE && propertyValue instanceof Float && (Float)propertyValue == 0.0F || fieldCLass == Double.TYPE && propertyValue instanceof Double && (Double)propertyValue == 0.0D || fieldCLass == Boolean.TYPE && propertyValue instanceof Boolean && !(Boolean)propertyValue) {
                                    break label951;
                                 }
                              }

                              if (commaFlag) {
                                 if (fieldInfo.unwrapped && propertyValue instanceof Map && ((Map)propertyValue).size() == 0) {
                                    break label951;
                                 }

                                 out.write(44);
                                 if (out.isEnabled(SerializerFeature.PrettyFormat)) {
                                    serializer.println();
                                 }
                              }

                              if (key != fieldInfoName) {
                                 if (!writeAsArray) {
                                    out.writeFieldName(key, true);
                                 }

                                 serializer.write(propertyValue);
                              } else if (originalValue != propertyValue) {
                                 if (!writeAsArray) {
                                    fieldSerializer.writePrefix(serializer);
                                 }

                                 serializer.write(propertyValue);
                              } else {
                                 if (!writeAsArray && !fieldInfo.unwrapped) {
                                    if (directWritePrefix) {
                                       out.write((char[])fieldInfo.name_chars, 0, fieldInfo.name_chars.length);
                                    } else {
                                       fieldSerializer.writePrefix(serializer);
                                    }
                                 }

                                 if (!writeAsArray) {
                                    JSONField fieldAnnotation = fieldInfo.getAnnotation();
                                    if (fieldClass == String.class && (fieldAnnotation == null || fieldAnnotation.serializeUsing() == Void.class)) {
                                       if (propertyValue == null) {
                                          if ((out.features & SerializerFeature.WriteNullStringAsEmpty.mask) == 0 && (fieldSerializer.features & SerializerFeature.WriteNullStringAsEmpty.mask) == 0) {
                                             out.writeNull();
                                          } else {
                                             out.writeString("");
                                          }
                                       } else {
                                          String propertyValueString = (String)propertyValue;
                                          if (out.useSingleQuotes) {
                                             out.writeStringWithSingleQuote(propertyValueString);
                                          } else {
                                             out.writeStringWithDoubleQuote(propertyValueString, '\u0000');
                                          }
                                       }
                                    } else {
                                       if (fieldInfo.unwrapped && propertyValue instanceof Map && ((Map)propertyValue).size() == 0) {
                                          commaFlag = false;
                                          break label951;
                                       }

                                       fieldSerializer.writeValue(serializer, propertyValue);
                                    }
                                 } else {
                                    fieldSerializer.writeValue(serializer, propertyValue);
                                 }
                              }

                              boolean fieldUnwrappedNull = false;
                              if (fieldInfo.unwrapped && propertyValue instanceof Map) {
                                 Map map = (Map)propertyValue;
                                 if (map.size() == 0) {
                                    fieldUnwrappedNull = true;
                                 } else if (!serializer.isEnabled(SerializerFeature.WriteMapNullValue)) {
                                    boolean hasNotNull = false;
                                    Iterator var32 = map.values().iterator();

                                    while(var32.hasNext()) {
                                       Object value = var32.next();
                                       if (value != null) {
                                          hasNotNull = true;
                                          break;
                                       }
                                    }

                                    if (!hasNotNull) {
                                       fieldUnwrappedNull = true;
                                    }
                                 }
                              }

                              if (!fieldUnwrappedNull) {
                                 commaFlag = true;
                              }
                           }
                        }
                     }
                  }
               }

               ++i;
            }
         } catch (Exception var39) {
            String errorMessage = "write javaBean error, fastjson version 1.2.47";
            if (object != null) {
               errorMessage = errorMessage + ", class " + object.getClass().getName();
            }

            if (fieldName != null) {
               errorMessage = errorMessage + ", fieldName : " + fieldName;
            }

            if (var39.getMessage() != null) {
               errorMessage = errorMessage + ", " + var39.getMessage();
            }

            throw new JSONException(errorMessage, var39);
         } finally {
            serializer.context = parent;
         }

      }
   }

   protected void writeClassName(JSONSerializer serializer, String typeKey, Object object) {
      if (typeKey == null) {
         typeKey = serializer.config.typeKey;
      }

      serializer.out.writeFieldName(typeKey, false);
      String typeName = this.beanInfo.typeName;
      if (typeName == null) {
         Class<?> clazz = object.getClass();
         if (TypeUtils.isProxy(clazz)) {
            clazz = clazz.getSuperclass();
         }

         typeName = clazz.getName();
      }

      serializer.write(typeName);
   }

   public boolean writeReference(JSONSerializer serializer, Object object, int fieldFeatures) {
      SerialContext context = serializer.context;
      int mask = SerializerFeature.DisableCircularReferenceDetect.mask;
      if (context != null && (context.features & mask) == 0 && (fieldFeatures & mask) == 0) {
         if (serializer.references != null && serializer.references.containsKey(object)) {
            serializer.writeReference(object);
            return true;
         } else {
            return false;
         }
      } else {
         return false;
      }
   }

   protected boolean isWriteAsArray(JSONSerializer serializer) {
      return this.isWriteAsArray(serializer, 0);
   }

   protected boolean isWriteAsArray(JSONSerializer serializer, int fieldFeatrues) {
      int mask = SerializerFeature.BeanToArray.mask;
      return (this.beanInfo.features & mask) != 0 || serializer.out.beanToArray || (fieldFeatrues & mask) != 0;
   }

   public Object getFieldValue(Object object, String key) {
      FieldSerializer fieldDeser = this.getFieldSerializer(key);
      if (fieldDeser == null) {
         throw new JSONException("field not found. " + key);
      } else {
         try {
            return fieldDeser.getPropertyValue(object);
         } catch (InvocationTargetException var5) {
            throw new JSONException("getFieldValue error." + key, var5);
         } catch (IllegalAccessException var6) {
            throw new JSONException("getFieldValue error." + key, var6);
         }
      }
   }

   public Object getFieldValue(Object object, String key, long keyHash, boolean throwFieldNotFoundException) {
      FieldSerializer fieldDeser = this.getFieldSerializer(keyHash);
      if (fieldDeser == null) {
         if (throwFieldNotFoundException) {
            throw new JSONException("field not found. " + key);
         } else {
            return null;
         }
      } else {
         try {
            return fieldDeser.getPropertyValue(object);
         } catch (InvocationTargetException var8) {
            throw new JSONException("getFieldValue error." + key, var8);
         } catch (IllegalAccessException var9) {
            throw new JSONException("getFieldValue error." + key, var9);
         }
      }
   }

   public FieldSerializer getFieldSerializer(String key) {
      if (key == null) {
         return null;
      } else {
         int low = 0;
         int high = this.sortedGetters.length - 1;

         while(low <= high) {
            int mid = low + high >>> 1;
            String fieldName = this.sortedGetters[mid].fieldInfo.name;
            int cmp = fieldName.compareTo(key);
            if (cmp < 0) {
               low = mid + 1;
            } else {
               if (cmp <= 0) {
                  return this.sortedGetters[mid];
               }

               high = mid - 1;
            }
         }

         return null;
      }
   }

   public FieldSerializer getFieldSerializer(long hash) {
      PropertyNamingStrategy[] namingStrategies = null;
      int i;
      String name;
      int p;
      if (this.hashArray == null) {
         namingStrategies = PropertyNamingStrategy.values();
         long[] hashArray = new long[this.sortedGetters.length * namingStrategies.length];
         int index = 0;

         for(i = 0; i < this.sortedGetters.length; ++i) {
            name = this.sortedGetters[i].fieldInfo.name;
            hashArray[index++] = TypeUtils.fnv1a_64(name);

            for(p = 0; p < namingStrategies.length; ++p) {
               String name_t = namingStrategies[p].translate(name);
               if (!name.equals(name_t)) {
                  hashArray[index++] = TypeUtils.fnv1a_64(name_t);
               }
            }
         }

         Arrays.sort(hashArray, 0, index);
         this.hashArray = new long[index];
         System.arraycopy(hashArray, 0, this.hashArray, 0, index);
      }

      int pos = Arrays.binarySearch(this.hashArray, hash);
      if (pos < 0) {
         return null;
      } else {
         if (this.hashArrayMapping == null) {
            if (namingStrategies == null) {
               namingStrategies = PropertyNamingStrategy.values();
            }

            short[] mapping = new short[this.hashArray.length];
            Arrays.fill(mapping, (short)-1);

            for(i = 0; i < this.sortedGetters.length; ++i) {
               name = this.sortedGetters[i].fieldInfo.name;
               p = Arrays.binarySearch(this.hashArray, TypeUtils.fnv1a_64(name));
               if (p >= 0) {
                  mapping[p] = (short)i;
               }

               for(int j = 0; j < namingStrategies.length; ++j) {
                  String name_t = namingStrategies[j].translate(name);
                  if (!name.equals(name_t)) {
                     int p_t = Arrays.binarySearch(this.hashArray, TypeUtils.fnv1a_64(name_t));
                     if (p_t >= 0) {
                        mapping[p_t] = (short)i;
                     }
                  }
               }
            }

            this.hashArrayMapping = mapping;
         }

         int getterIndex = this.hashArrayMapping[pos];
         return getterIndex != -1 ? this.sortedGetters[getterIndex] : null;
      }
   }

   public List<Object> getFieldValues(Object object) throws Exception {
      List<Object> fieldValues = new ArrayList(this.sortedGetters.length);
      FieldSerializer[] var3 = this.sortedGetters;
      int var4 = var3.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         FieldSerializer getter = var3[var5];
         fieldValues.add(getter.getPropertyValue(object));
      }

      return fieldValues;
   }

   public List<Object> getObjectFieldValues(Object object) throws Exception {
      List<Object> fieldValues = new ArrayList(this.sortedGetters.length);
      FieldSerializer[] var3 = this.sortedGetters;
      int var4 = var3.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         FieldSerializer getter = var3[var5];
         Class fieldClass = getter.fieldInfo.fieldClass;
         if (!fieldClass.isPrimitive() && !fieldClass.getName().startsWith("java.lang.")) {
            fieldValues.add(getter.getPropertyValue(object));
         }
      }

      return fieldValues;
   }

   public int getSize(Object object) throws Exception {
      int size = 0;
      FieldSerializer[] var3 = this.sortedGetters;
      int var4 = var3.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         FieldSerializer getter = var3[var5];
         Object value = getter.getPropertyValueDirect(object);
         if (value != null) {
            ++size;
         }
      }

      return size;
   }

   public Map<String, Object> getFieldValuesMap(Object object) throws Exception {
      Map<String, Object> map = new LinkedHashMap(this.sortedGetters.length);
      FieldSerializer[] var3 = this.sortedGetters;
      int var4 = var3.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         FieldSerializer getter = var3[var5];
         map.put(getter.fieldInfo.name, getter.getPropertyValue(object));
      }

      return map;
   }

   protected BeanContext getBeanContext(int orinal) {
      return this.sortedGetters[orinal].fieldContext;
   }

   protected Type getFieldType(int ordinal) {
      return this.sortedGetters[ordinal].fieldInfo.fieldType;
   }

   protected char writeBefore(JSONSerializer jsonBeanDeser, Object object, char seperator) {
      Iterator var4;
      BeforeFilter beforeFilter;
      if (jsonBeanDeser.beforeFilters != null) {
         for(var4 = jsonBeanDeser.beforeFilters.iterator(); var4.hasNext(); seperator = beforeFilter.writeBefore(jsonBeanDeser, object, seperator)) {
            beforeFilter = (BeforeFilter)var4.next();
         }
      }

      if (this.beforeFilters != null) {
         for(var4 = this.beforeFilters.iterator(); var4.hasNext(); seperator = beforeFilter.writeBefore(jsonBeanDeser, object, seperator)) {
            beforeFilter = (BeforeFilter)var4.next();
         }
      }

      return seperator;
   }

   protected char writeAfter(JSONSerializer jsonBeanDeser, Object object, char seperator) {
      Iterator var4;
      AfterFilter afterFilter;
      if (jsonBeanDeser.afterFilters != null) {
         for(var4 = jsonBeanDeser.afterFilters.iterator(); var4.hasNext(); seperator = afterFilter.writeAfter(jsonBeanDeser, object, seperator)) {
            afterFilter = (AfterFilter)var4.next();
         }
      }

      if (this.afterFilters != null) {
         for(var4 = this.afterFilters.iterator(); var4.hasNext(); seperator = afterFilter.writeAfter(jsonBeanDeser, object, seperator)) {
            afterFilter = (AfterFilter)var4.next();
         }
      }

      return seperator;
   }

   protected boolean applyLabel(JSONSerializer jsonBeanDeser, String label) {
      Iterator var3;
      LabelFilter propertyFilter;
      if (jsonBeanDeser.labelFilters != null) {
         var3 = jsonBeanDeser.labelFilters.iterator();

         while(var3.hasNext()) {
            propertyFilter = (LabelFilter)var3.next();
            if (!propertyFilter.apply(label)) {
               return false;
            }
         }
      }

      if (this.labelFilters != null) {
         var3 = this.labelFilters.iterator();

         while(var3.hasNext()) {
            propertyFilter = (LabelFilter)var3.next();
            if (!propertyFilter.apply(label)) {
               return false;
            }
         }
      }

      return true;
   }
}
