package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import java.io.IOException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map.Entry;

public class MapSerializer extends SerializeFilterable implements ObjectSerializer {
   public static MapSerializer instance = new MapSerializer();
   private static final int NON_STRINGKEY_AS_STRING;

   public void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      this.write(serializer, object, fieldName, fieldType, features, false);
   }

   public void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features, boolean unwrapped) throws IOException {
      SerializeWriter out = serializer.out;
      if (object == null) {
         out.writeNull();
      } else {
         Map<?, ?> map = (Map)object;
         int mapSortFieldMask = SerializerFeature.MapSortField.mask;
         if ((out.features & mapSortFieldMask) != 0 || (features & mapSortFieldMask) != 0) {
            if (map instanceof JSONObject) {
               map = ((JSONObject)map).getInnerMap();
            }

            if (!(map instanceof SortedMap) && !(map instanceof LinkedHashMap)) {
               try {
                  map = new TreeMap((Map)map);
               } catch (Exception var25) {
               }
            }
         }

         if (serializer.containsReference(object)) {
            serializer.writeReference(object);
         } else {
            SerialContext parent = serializer.context;
            serializer.setContext(parent, object, fieldName, 0);

            try {
               if (!unwrapped) {
                  out.write(123);
               }

               serializer.incrementIndent();
               Class<?> preClazz = null;
               ObjectSerializer preWriter = null;
               boolean first = true;
               if (out.isEnabled(SerializerFeature.WriteClassName)) {
                  String typeKey = serializer.config.typeKey;
                  Class<?> mapClass = map.getClass();
                  boolean containsKey = (mapClass == JSONObject.class || mapClass == HashMap.class || mapClass == LinkedHashMap.class) && ((Map)map).containsKey(typeKey);
                  if (!containsKey) {
                     out.writeFieldName(typeKey);
                     out.writeString(object.getClass().getName());
                     first = false;
                  }
               }

               Iterator var27 = ((Map)map).entrySet().iterator();

               while(var27.hasNext()) {
                  Entry entry = (Entry)var27.next();
                  Object value = entry.getValue();
                  Object entryKey = entry.getKey();
                  List<PropertyPreFilter> nameFilters = serializer.propertyPreFilters;
                  String strKey;
                  if (nameFilters != null && nameFilters.size() > 0) {
                     if (entryKey != null && !(entryKey instanceof String)) {
                        if (entryKey.getClass().isPrimitive() || entryKey instanceof Number) {
                           strKey = JSON.toJSONString(entryKey);
                           if (!this.applyName(serializer, object, strKey)) {
                              continue;
                           }
                        }
                     } else if (!this.applyName(serializer, object, (String)entryKey)) {
                        continue;
                     }
                  }

                  nameFilters = this.propertyPreFilters;
                  if (nameFilters != null && nameFilters.size() > 0) {
                     if (entryKey != null && !(entryKey instanceof String)) {
                        if (entryKey.getClass().isPrimitive() || entryKey instanceof Number) {
                           strKey = JSON.toJSONString(entryKey);
                           if (!this.applyName(serializer, object, strKey)) {
                              continue;
                           }
                        }
                     } else if (!this.applyName(serializer, object, (String)entryKey)) {
                        continue;
                     }
                  }

                  nameFilters = serializer.propertyFilters;
                  if (nameFilters != null && nameFilters.size() > 0) {
                     if (entryKey != null && !(entryKey instanceof String)) {
                        if (entryKey.getClass().isPrimitive() || entryKey instanceof Number) {
                           strKey = JSON.toJSONString(entryKey);
                           if (!this.apply(serializer, object, strKey, value)) {
                              continue;
                           }
                        }
                     } else if (!this.apply(serializer, object, (String)entryKey, value)) {
                        continue;
                     }
                  }

                  nameFilters = this.propertyFilters;
                  if (nameFilters != null && nameFilters.size() > 0) {
                     if (entryKey != null && !(entryKey instanceof String)) {
                        if (entryKey.getClass().isPrimitive() || entryKey instanceof Number) {
                           strKey = JSON.toJSONString(entryKey);
                           if (!this.apply(serializer, object, strKey, value)) {
                              continue;
                           }
                        }
                     } else if (!this.apply(serializer, object, (String)entryKey, value)) {
                        continue;
                     }
                  }

                  nameFilters = serializer.nameFilters;
                  if (nameFilters != null && nameFilters.size() > 0) {
                     if (entryKey != null && !(entryKey instanceof String)) {
                        if (entryKey.getClass().isPrimitive() || entryKey instanceof Number) {
                           strKey = JSON.toJSONString(entryKey);
                           entryKey = this.processKey(serializer, object, strKey, value);
                        }
                     } else {
                        entryKey = this.processKey(serializer, object, (String)entryKey, value);
                     }
                  }

                  nameFilters = this.nameFilters;
                  if (nameFilters != null && nameFilters.size() > 0) {
                     if (entryKey != null && !(entryKey instanceof String)) {
                        if (entryKey.getClass().isPrimitive() || entryKey instanceof Number) {
                           strKey = JSON.toJSONString(entryKey);
                           entryKey = this.processKey(serializer, object, strKey, value);
                        }
                     } else {
                        entryKey = this.processKey(serializer, object, (String)entryKey, value);
                     }
                  }

                  if (entryKey != null && !(entryKey instanceof String)) {
                     boolean objectOrArray = entryKey instanceof Map || entryKey instanceof Collection;
                     if (!objectOrArray) {
                        strKey = JSON.toJSONString(entryKey);
                        value = this.processValue(serializer, (BeanContext)null, object, strKey, value);
                     }
                  } else {
                     value = this.processValue(serializer, (BeanContext)null, object, (String)entryKey, value);
                  }

                  if (value != null || out.isEnabled(SerializerFeature.WRITE_MAP_NULL_FEATURES)) {
                     String key;
                     if (entryKey instanceof String) {
                        key = (String)entryKey;
                        if (!first) {
                           out.write(44);
                        }

                        if (out.isEnabled(SerializerFeature.PrettyFormat)) {
                           serializer.println();
                        }

                        out.writeFieldName(key, true);
                     } else {
                        if (!first) {
                           out.write(44);
                        }

                        if (out.isEnabled(NON_STRINGKEY_AS_STRING) && !(entryKey instanceof Enum)) {
                           key = JSON.toJSONString(entryKey);
                           serializer.write(key);
                        } else {
                           serializer.write(entryKey);
                        }

                        out.write(58);
                     }

                     first = false;
                     if (value == null) {
                        out.writeNull();
                     } else {
                        Class<?> clazz = value.getClass();
                        if (clazz != preClazz) {
                           preClazz = clazz;
                           preWriter = serializer.getObjectWriter(clazz);
                        }

                        if (SerializerFeature.isEnabled(features, SerializerFeature.WriteClassName) && preWriter instanceof JavaBeanSerializer) {
                           Type valueType = null;
                           if (fieldType instanceof ParameterizedType) {
                              ParameterizedType parameterizedType = (ParameterizedType)fieldType;
                              Type[] actualTypeArguments = parameterizedType.getActualTypeArguments();
                              if (actualTypeArguments.length == 2) {
                                 valueType = actualTypeArguments[1];
                              }
                           }

                           JavaBeanSerializer javaBeanSerializer = (JavaBeanSerializer)preWriter;
                           javaBeanSerializer.writeNoneASM(serializer, value, entryKey, valueType, features);
                        } else {
                           preWriter.write(serializer, value, entryKey, (Type)null, features);
                        }
                     }
                  }
               }
            } finally {
               serializer.context = parent;
            }

            serializer.decrementIdent();
            if (out.isEnabled(SerializerFeature.PrettyFormat) && ((Map)map).size() > 0) {
               serializer.println();
            }

            if (!unwrapped) {
               out.write(125);
            }

         }
      }
   }

   static {
      NON_STRINGKEY_AS_STRING = SerializerFeature.of(new SerializerFeature[]{SerializerFeature.BrowserCompatible, SerializerFeature.WriteNonStringKeyAsString, SerializerFeature.BrowserSecure});
   }
}
