package com.alibaba.fastjson.serializer;

import java.io.IOException;
import java.lang.reflect.Type;

public class PrimitiveArraySerializer implements ObjectSerializer {
   public static PrimitiveArraySerializer instance = new PrimitiveArraySerializer();

   public final void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      SerializeWriter out = serializer.out;
      if (object == null) {
         out.writeNull(SerializerFeature.WriteNullListAsEmpty);
      } else {
         int i;
         if (object instanceof int[]) {
            int[] array = (int[])((int[])object);
            out.write(91);

            for(i = 0; i < array.length; ++i) {
               if (i != 0) {
                  out.write(44);
               }

               out.writeInt(array[i]);
            }

            out.write(93);
         } else if (object instanceof short[]) {
            short[] array = (short[])((short[])object);
            out.write(91);

            for(i = 0; i < array.length; ++i) {
               if (i != 0) {
                  out.write(44);
               }

               out.writeInt(array[i]);
            }

            out.write(93);
         } else if (object instanceof long[]) {
            long[] array = (long[])((long[])object);
            out.write(91);

            for(i = 0; i < array.length; ++i) {
               if (i != 0) {
                  out.write(44);
               }

               out.writeLong(array[i]);
            }

            out.write(93);
         } else if (object instanceof boolean[]) {
            boolean[] array = (boolean[])((boolean[])object);
            out.write(91);

            for(i = 0; i < array.length; ++i) {
               if (i != 0) {
                  out.write(44);
               }

               out.write(array[i]);
            }

            out.write(93);
         } else if (object instanceof float[]) {
            float[] array = (float[])((float[])object);
            out.write(91);

            for(i = 0; i < array.length; ++i) {
               if (i != 0) {
                  out.write(44);
               }

               float item = array[i];
               if (Float.isNaN(item)) {
                  out.writeNull();
               } else {
                  out.append(Float.toString(item));
               }
            }

            out.write(93);
         } else if (object instanceof double[]) {
            double[] array = (double[])((double[])object);
            out.write(91);

            for(i = 0; i < array.length; ++i) {
               if (i != 0) {
                  out.write(44);
               }

               double item = array[i];
               if (Double.isNaN(item)) {
                  out.writeNull();
               } else {
                  out.append(Double.toString(item));
               }
            }

            out.write(93);
         } else if (object instanceof byte[]) {
            byte[] array = (byte[])((byte[])object);
            out.writeByteArray(array);
         } else {
            char[] chars = (char[])((char[])object);
            out.writeString(chars);
         }
      }
   }
}
