package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONAware;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONStreamAware;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.annotation.JSONType;
import com.alibaba.fastjson.parser.deserializer.Jdk8DateCodec;
import com.alibaba.fastjson.parser.deserializer.OptionalCodec;
import com.alibaba.fastjson.support.springfox.SwaggerJsonSerializer;
import com.alibaba.fastjson.util.ASMUtils;
import com.alibaba.fastjson.util.FieldInfo;
import com.alibaba.fastjson.util.IdentityHashMap;
import com.alibaba.fastjson.util.ServiceLoader;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.File;
import java.io.Serializable;
import java.lang.ref.SoftReference;
import java.lang.ref.WeakReference;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.Proxy;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.URI;
import java.net.URL;
import java.nio.charset.Charset;
import java.sql.Clob;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Currency;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;
import java.util.UUID;
import java.util.Map.Entry;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicIntegerArray;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.AtomicLongArray;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Pattern;
import javax.xml.datatype.XMLGregorianCalendar;

public class SerializeConfig {
   public static final SerializeConfig globalInstance = new SerializeConfig();
   private static boolean awtError = false;
   private static boolean jdk8Error = false;
   private static boolean oracleJdbcError = false;
   private static boolean springfoxError = false;
   private static boolean guavaError = false;
   private static boolean jsonnullError = false;
   private boolean asm;
   private ASMSerializerFactory asmFactory;
   protected String typeKey;
   public PropertyNamingStrategy propertyNamingStrategy;
   private final IdentityHashMap<Type, ObjectSerializer> serializers;
   private final boolean fieldBased;

   public String getTypeKey() {
      return this.typeKey;
   }

   public void setTypeKey(String typeKey) {
      this.typeKey = typeKey;
   }

   private final JavaBeanSerializer createASMSerializer(SerializeBeanInfo beanInfo) throws Exception {
      JavaBeanSerializer serializer = this.asmFactory.createJavaBeanSerializer(beanInfo);

      for(int i = 0; i < serializer.sortedGetters.length; ++i) {
         FieldSerializer fieldDeser = serializer.sortedGetters[i];
         Class<?> fieldClass = fieldDeser.fieldInfo.fieldClass;
         if (fieldClass.isEnum()) {
            ObjectSerializer fieldSer = this.getObjectWriter(fieldClass);
            if (!(fieldSer instanceof EnumSerializer)) {
               serializer.writeDirect = false;
            }
         }
      }

      return serializer;
   }

   public final ObjectSerializer createJavaBeanSerializer(Class<?> clazz) {
      SerializeBeanInfo beanInfo = TypeUtils.buildBeanInfo(clazz, (Map)null, this.propertyNamingStrategy, this.fieldBased);
      return (ObjectSerializer)(beanInfo.fields.length == 0 && Iterable.class.isAssignableFrom(clazz) ? MiscCodec.instance : this.createJavaBeanSerializer(beanInfo));
   }

   public ObjectSerializer createJavaBeanSerializer(SerializeBeanInfo beanInfo) {
      JSONType jsonType = beanInfo.jsonType;
      boolean asm = this.asm && !this.fieldBased;
      Class clazz;
      int var6;
      int var7;
      if (jsonType != null) {
         clazz = jsonType.serializer();
         if (clazz != Void.class) {
            try {
               Object seralizer = clazz.newInstance();
               if (seralizer instanceof ObjectSerializer) {
                  return (ObjectSerializer)seralizer;
               }
            } catch (Throwable var21) {
            }
         }

         if (!jsonType.asm()) {
            asm = false;
         }

         SerializerFeature[] var22 = jsonType.serialzeFeatures();
         var6 = var22.length;

         for(var7 = 0; var7 < var6; ++var7) {
            SerializerFeature feature = var22[var7];
            if (SerializerFeature.WriteNonStringValueAsString == feature || SerializerFeature.WriteEnumUsingToString == feature || SerializerFeature.NotWriteDefaultValue == feature || SerializerFeature.BrowserCompatible == feature) {
               asm = false;
               break;
            }
         }
      }

      clazz = beanInfo.beanType;
      if (!Modifier.isPublic(beanInfo.beanType.getModifiers())) {
         return new JavaBeanSerializer(beanInfo);
      } else {
         if (asm && this.asmFactory.classLoader.isExternalClass(clazz) || clazz == Serializable.class || clazz == Object.class) {
            asm = false;
         }

         if (asm && !ASMUtils.checkName(clazz.getSimpleName())) {
            asm = false;
         }

         if (asm && beanInfo.beanType.isInterface()) {
            asm = false;
         }

         if (asm) {
            FieldInfo[] var23 = beanInfo.fields;
            var6 = var23.length;

            for(var7 = 0; var7 < var6; ++var7) {
               FieldInfo fieldInfo = var23[var7];
               Field field = fieldInfo.field;
               if (field != null && !field.getType().equals(fieldInfo.fieldClass)) {
                  asm = false;
                  break;
               }

               Method method = fieldInfo.method;
               if (method != null && !method.getReturnType().equals(fieldInfo.fieldClass)) {
                  asm = false;
                  break;
               }

               JSONField annotation = fieldInfo.getAnnotation();
               if (annotation != null) {
                  String format = annotation.format();
                  if (format.length() != 0 && (fieldInfo.fieldClass != String.class || !"trim".equals(format))) {
                     asm = false;
                     break;
                  }

                  if (!ASMUtils.checkName(annotation.name()) || annotation.jsonDirect() || annotation.serializeUsing() != Void.class || annotation.unwrapped()) {
                     asm = false;
                     break;
                  }

                  SerializerFeature[] var13 = annotation.serialzeFeatures();
                  int var14 = var13.length;

                  for(int var15 = 0; var15 < var14; ++var15) {
                     SerializerFeature feature = var13[var15];
                     if (SerializerFeature.WriteNonStringValueAsString == feature || SerializerFeature.WriteEnumUsingToString == feature || SerializerFeature.NotWriteDefaultValue == feature || SerializerFeature.BrowserCompatible == feature || SerializerFeature.WriteClassName == feature) {
                        asm = false;
                        break;
                     }
                  }

                  if (TypeUtils.isAnnotationPresentOneToMany(method) || TypeUtils.isAnnotationPresentManyToMany(method)) {
                     asm = true;
                     break;
                  }
               }
            }
         }

         if (asm) {
            try {
               ObjectSerializer asmSerializer = this.createASMSerializer(beanInfo);
               if (asmSerializer != null) {
                  return asmSerializer;
               }
            } catch (ClassNotFoundException var17) {
            } catch (ClassFormatError var18) {
            } catch (ClassCastException var19) {
            } catch (Throwable var20) {
               throw new JSONException("create asm serializer error, class " + clazz, var20);
            }
         }

         return new JavaBeanSerializer(beanInfo);
      }
   }

   public boolean isAsmEnable() {
      return this.asm;
   }

   public void setAsmEnable(boolean asmEnable) {
      if (!ASMUtils.IS_ANDROID) {
         this.asm = asmEnable;
      }
   }

   public static SerializeConfig getGlobalInstance() {
      return globalInstance;
   }

   public SerializeConfig() {
      this(8192);
   }

   public SerializeConfig(boolean fieldBase) {
      this(8192, fieldBase);
   }

   public SerializeConfig(int tableSize) {
      this(tableSize, false);
   }

   public SerializeConfig(int tableSize, boolean fieldBase) {
      this.asm = !ASMUtils.IS_ANDROID;
      this.typeKey = JSON.DEFAULT_TYPE_KEY;
      this.fieldBased = fieldBase;
      this.serializers = new IdentityHashMap(tableSize);

      try {
         if (this.asm) {
            this.asmFactory = new ASMSerializerFactory();
         }
      } catch (Throwable var4) {
         this.asm = false;
      }

      this.initSerializers();
   }

   private void initSerializers() {
      this.put((Type)Boolean.class, (ObjectSerializer)BooleanCodec.instance);
      this.put((Type)Character.class, (ObjectSerializer)CharacterCodec.instance);
      this.put((Type)Byte.class, (ObjectSerializer)IntegerCodec.instance);
      this.put((Type)Short.class, (ObjectSerializer)IntegerCodec.instance);
      this.put((Type)Integer.class, (ObjectSerializer)IntegerCodec.instance);
      this.put((Type)Long.class, (ObjectSerializer)LongCodec.instance);
      this.put((Type)Float.class, (ObjectSerializer)FloatCodec.instance);
      this.put((Type)Double.class, (ObjectSerializer)DoubleSerializer.instance);
      this.put((Type)BigDecimal.class, (ObjectSerializer)BigDecimalCodec.instance);
      this.put((Type)BigInteger.class, (ObjectSerializer)BigIntegerCodec.instance);
      this.put((Type)String.class, (ObjectSerializer)StringCodec.instance);
      this.put((Type)byte[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)short[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)int[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)long[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)float[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)double[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)boolean[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)char[].class, (ObjectSerializer)PrimitiveArraySerializer.instance);
      this.put((Type)Object[].class, (ObjectSerializer)ObjectArrayCodec.instance);
      this.put((Type)Class.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)SimpleDateFormat.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)Currency.class, (ObjectSerializer)(new MiscCodec()));
      this.put((Type)TimeZone.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)InetAddress.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)Inet4Address.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)Inet6Address.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)InetSocketAddress.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)File.class, (ObjectSerializer)MiscCodec.instance);
      this.put((Type)Appendable.class, (ObjectSerializer)AppendableSerializer.instance);
      this.put((Type)StringBuffer.class, (ObjectSerializer)AppendableSerializer.instance);
      this.put((Type)StringBuilder.class, (ObjectSerializer)AppendableSerializer.instance);
      this.put((Type)Charset.class, (ObjectSerializer)ToStringSerializer.instance);
      this.put((Type)Pattern.class, (ObjectSerializer)ToStringSerializer.instance);
      this.put((Type)Locale.class, (ObjectSerializer)ToStringSerializer.instance);
      this.put((Type)URI.class, (ObjectSerializer)ToStringSerializer.instance);
      this.put((Type)URL.class, (ObjectSerializer)ToStringSerializer.instance);
      this.put((Type)UUID.class, (ObjectSerializer)ToStringSerializer.instance);
      this.put((Type)AtomicBoolean.class, (ObjectSerializer)AtomicCodec.instance);
      this.put((Type)AtomicInteger.class, (ObjectSerializer)AtomicCodec.instance);
      this.put((Type)AtomicLong.class, (ObjectSerializer)AtomicCodec.instance);
      this.put((Type)AtomicReference.class, (ObjectSerializer)ReferenceCodec.instance);
      this.put((Type)AtomicIntegerArray.class, (ObjectSerializer)AtomicCodec.instance);
      this.put((Type)AtomicLongArray.class, (ObjectSerializer)AtomicCodec.instance);
      this.put((Type)WeakReference.class, (ObjectSerializer)ReferenceCodec.instance);
      this.put((Type)SoftReference.class, (ObjectSerializer)ReferenceCodec.instance);
      this.put((Type)LinkedList.class, (ObjectSerializer)CollectionCodec.instance);
   }

   public void addFilter(Class<?> clazz, SerializeFilter filter) {
      ObjectSerializer serializer = this.getObjectWriter(clazz);
      if (serializer instanceof SerializeFilterable) {
         SerializeFilterable filterable = (SerializeFilterable)serializer;
         if (this != globalInstance && filterable == MapSerializer.instance) {
            MapSerializer newMapSer = new MapSerializer();
            this.put((Type)clazz, (ObjectSerializer)newMapSer);
            newMapSer.addFilter(filter);
            return;
         }

         filterable.addFilter(filter);
      }

   }

   public void config(Class<?> clazz, SerializerFeature feature, boolean value) {
      ObjectSerializer serializer = this.getObjectWriter(clazz, false);
      if (serializer == null) {
         SerializeBeanInfo beanInfo = TypeUtils.buildBeanInfo(clazz, (Map)null, this.propertyNamingStrategy);
         if (value) {
            beanInfo.features |= feature.mask;
         } else {
            beanInfo.features &= ~feature.mask;
         }

         serializer = this.createJavaBeanSerializer(beanInfo);
         this.put((Type)clazz, (ObjectSerializer)serializer);
      } else {
         if (serializer instanceof JavaBeanSerializer) {
            JavaBeanSerializer javaBeanSerializer = (JavaBeanSerializer)serializer;
            SerializeBeanInfo beanInfo = javaBeanSerializer.beanInfo;
            int originalFeaturs = beanInfo.features;
            if (value) {
               beanInfo.features |= feature.mask;
            } else {
               beanInfo.features &= ~feature.mask;
            }

            if (originalFeaturs == beanInfo.features) {
               return;
            }

            Class<?> serializerClass = serializer.getClass();
            if (serializerClass != JavaBeanSerializer.class) {
               ObjectSerializer newSerializer = this.createJavaBeanSerializer(beanInfo);
               this.put((Type)clazz, (ObjectSerializer)newSerializer);
            }
         }

      }
   }

   public ObjectSerializer getObjectWriter(Class<?> clazz) {
      return this.getObjectWriter(clazz, true);
   }

   private ObjectSerializer getObjectWriter(Class<?> clazz, boolean create) {
      ObjectSerializer writer = (ObjectSerializer)this.serializers.get(clazz);
      ClassLoader classLoader;
      Iterator var5;
      Object o;
      AutowiredObjectSerializer autowired;
      Iterator var8;
      Type forType;
      if (writer == null) {
         try {
            classLoader = Thread.currentThread().getContextClassLoader();
            var5 = ServiceLoader.load(AutowiredObjectSerializer.class, classLoader).iterator();

            label361:
            while(true) {
               do {
                  if (!var5.hasNext()) {
                     break label361;
                  }

                  o = var5.next();
               } while(!(o instanceof AutowiredObjectSerializer));

               autowired = (AutowiredObjectSerializer)o;
               var8 = autowired.getAutowiredFor().iterator();

               while(var8.hasNext()) {
                  forType = (Type)var8.next();
                  this.put((Type)forType, (ObjectSerializer)autowired);
               }
            }
         } catch (ClassCastException var19) {
         }

         writer = (ObjectSerializer)this.serializers.get(clazz);
      }

      if (writer == null) {
         classLoader = JSON.class.getClassLoader();
         if (classLoader != Thread.currentThread().getContextClassLoader()) {
            try {
               var5 = ServiceLoader.load(AutowiredObjectSerializer.class, classLoader).iterator();

               label342:
               while(true) {
                  do {
                     if (!var5.hasNext()) {
                        break label342;
                     }

                     o = var5.next();
                  } while(!(o instanceof AutowiredObjectSerializer));

                  autowired = (AutowiredObjectSerializer)o;
                  var8 = autowired.getAutowiredFor().iterator();

                  while(var8.hasNext()) {
                     forType = (Type)var8.next();
                     this.put((Type)forType, (ObjectSerializer)autowired);
                  }
               }
            } catch (ClassCastException var18) {
            }

            writer = (ObjectSerializer)this.serializers.get(clazz);
         }
      }

      if (writer == null) {
         String className = clazz.getName();
         if (Map.class.isAssignableFrom(clazz)) {
            this.put((Type)clazz, (ObjectSerializer)(writer = MapSerializer.instance));
         } else if (List.class.isAssignableFrom(clazz)) {
            this.put((Type)clazz, (ObjectSerializer)(writer = ListSerializer.instance));
         } else if (Collection.class.isAssignableFrom(clazz)) {
            this.put((Type)clazz, (ObjectSerializer)(writer = CollectionCodec.instance));
         } else if (Date.class.isAssignableFrom(clazz)) {
            this.put((Type)clazz, (ObjectSerializer)(writer = DateCodec.instance));
         } else if (JSONAware.class.isAssignableFrom(clazz)) {
            this.put((Type)clazz, (ObjectSerializer)(writer = JSONAwareSerializer.instance));
         } else if (JSONSerializable.class.isAssignableFrom(clazz)) {
            this.put((Type)clazz, (ObjectSerializer)(writer = JSONSerializableSerializer.instance));
         } else if (JSONStreamAware.class.isAssignableFrom(clazz)) {
            this.put((Type)clazz, (ObjectSerializer)(writer = MiscCodec.instance));
         } else {
            JSONType jsonType;
            if (clazz.isEnum()) {
               jsonType = (JSONType)TypeUtils.getAnnotation(clazz, JSONType.class);
               if (jsonType != null && jsonType.serializeEnumAsJavaBean()) {
                  this.put((Type)clazz, (ObjectSerializer)(writer = this.createJavaBeanSerializer(clazz)));
               } else {
                  this.put((Type)clazz, (ObjectSerializer)(writer = EnumSerializer.instance));
               }
            } else {
               Class superClass;
               if ((superClass = clazz.getSuperclass()) != null && superClass.isEnum()) {
                  jsonType = (JSONType)TypeUtils.getAnnotation(superClass, JSONType.class);
                  if (jsonType != null && jsonType.serializeEnumAsJavaBean()) {
                     this.put((Type)clazz, (ObjectSerializer)(writer = this.createJavaBeanSerializer(clazz)));
                  } else {
                     this.put((Type)clazz, (ObjectSerializer)(writer = EnumSerializer.instance));
                  }
               } else if (clazz.isArray()) {
                  Class<?> componentType = clazz.getComponentType();
                  ObjectSerializer compObjectSerializer = this.getObjectWriter(componentType);
                  this.put((Type)clazz, (ObjectSerializer)(writer = new ArraySerializer(componentType, compObjectSerializer)));
               } else if (Throwable.class.isAssignableFrom(clazz)) {
                  SerializeBeanInfo beanInfo = TypeUtils.buildBeanInfo(clazz, (Map)null, this.propertyNamingStrategy);
                  beanInfo.features |= SerializerFeature.WriteClassName.mask;
                  this.put((Type)clazz, (ObjectSerializer)(writer = new JavaBeanSerializer(beanInfo)));
               } else if (!TimeZone.class.isAssignableFrom(clazz) && !Entry.class.isAssignableFrom(clazz)) {
                  if (Appendable.class.isAssignableFrom(clazz)) {
                     this.put((Type)clazz, (ObjectSerializer)(writer = AppendableSerializer.instance));
                  } else if (Charset.class.isAssignableFrom(clazz)) {
                     this.put((Type)clazz, (ObjectSerializer)(writer = ToStringSerializer.instance));
                  } else if (Enumeration.class.isAssignableFrom(clazz)) {
                     this.put((Type)clazz, (ObjectSerializer)(writer = EnumerationSerializer.instance));
                  } else if (!Calendar.class.isAssignableFrom(clazz) && !XMLGregorianCalendar.class.isAssignableFrom(clazz)) {
                     if (Clob.class.isAssignableFrom(clazz)) {
                        this.put((Type)clazz, (ObjectSerializer)(writer = ClobSeriliazer.instance));
                     } else if (TypeUtils.isPath(clazz)) {
                        this.put((Type)clazz, (ObjectSerializer)(writer = ToStringSerializer.instance));
                     } else if (Iterator.class.isAssignableFrom(clazz)) {
                        this.put((Type)clazz, (ObjectSerializer)(writer = MiscCodec.instance));
                     } else {
                        String name;
                        String[] names;
                        String[] var27;
                        int var28;
                        int var29;
                        if (className.startsWith("java.awt.") && AwtCodec.support(clazz) && !awtError) {
                           try {
                              names = new String[]{"java.awt.Color", "java.awt.Font", "java.awt.Point", "java.awt.Rectangle"};
                              var27 = names;
                              var28 = names.length;

                              for(var29 = 0; var29 < var28; ++var29) {
                                 name = var27[var29];
                                 if (name.equals(className)) {
                                    AwtCodec writer;
                                    this.put((Type)Class.forName(name), (ObjectSerializer)(writer = AwtCodec.instance));
                                    return writer;
                                 }
                              }
                           } catch (Throwable var17) {
                              awtError = true;
                           }
                        }

                        if (!jdk8Error && (className.startsWith("java.time.") || className.startsWith("java.util.Optional") || className.equals("java.util.concurrent.atomic.LongAdder") || className.equals("java.util.concurrent.atomic.DoubleAdder"))) {
                           try {
                              names = new String[]{"java.time.LocalDateTime", "java.time.LocalDate", "java.time.LocalTime", "java.time.ZonedDateTime", "java.time.OffsetDateTime", "java.time.OffsetTime", "java.time.ZoneOffset", "java.time.ZoneRegion", "java.time.Period", "java.time.Duration", "java.time.Instant"};
                              var27 = names;
                              var28 = names.length;

                              for(var29 = 0; var29 < var28; ++var29) {
                                 name = var27[var29];
                                 if (name.equals(className)) {
                                    Jdk8DateCodec writer;
                                    this.put((Type)Class.forName(name), (ObjectSerializer)(writer = Jdk8DateCodec.instance));
                                    return writer;
                                 }
                              }

                              names = new String[]{"java.util.Optional", "java.util.OptionalDouble", "java.util.OptionalInt", "java.util.OptionalLong"};
                              var27 = names;
                              var28 = names.length;

                              for(var29 = 0; var29 < var28; ++var29) {
                                 name = var27[var29];
                                 if (name.equals(className)) {
                                    OptionalCodec writer;
                                    this.put((Type)Class.forName(name), (ObjectSerializer)(writer = OptionalCodec.instance));
                                    return writer;
                                 }
                              }

                              names = new String[]{"java.util.concurrent.atomic.LongAdder", "java.util.concurrent.atomic.DoubleAdder"};
                              var27 = names;
                              var28 = names.length;

                              for(var29 = 0; var29 < var28; ++var29) {
                                 name = var27[var29];
                                 if (name.equals(className)) {
                                    AdderSerializer writer;
                                    this.put((Type)Class.forName(name), (ObjectSerializer)(writer = AdderSerializer.instance));
                                    return writer;
                                 }
                              }
                           } catch (Throwable var16) {
                              jdk8Error = true;
                           }
                        }

                        if (!oracleJdbcError && className.startsWith("oracle.sql.")) {
                           try {
                              names = new String[]{"oracle.sql.DATE", "oracle.sql.TIMESTAMP"};
                              var27 = names;
                              var28 = names.length;

                              for(var29 = 0; var29 < var28; ++var29) {
                                 name = var27[var29];
                                 if (name.equals(className)) {
                                    DateCodec writer;
                                    this.put((Type)Class.forName(name), (ObjectSerializer)(writer = DateCodec.instance));
                                    return writer;
                                 }
                              }
                           } catch (Throwable var15) {
                              oracleJdbcError = true;
                           }
                        }

                        if (!springfoxError && className.equals("springfox.documentation.spring.web.json.Json")) {
                           try {
                              SwaggerJsonSerializer writer;
                              this.put((Type)Class.forName("springfox.documentation.spring.web.json.Json"), (ObjectSerializer)(writer = SwaggerJsonSerializer.instance));
                              return writer;
                           } catch (ClassNotFoundException var13) {
                              springfoxError = true;
                           }
                        }

                        if (!guavaError && className.startsWith("com.google.common.collect.")) {
                           try {
                              names = new String[]{"com.google.common.collect.HashMultimap", "com.google.common.collect.LinkedListMultimap", "com.google.common.collect.ArrayListMultimap", "com.google.common.collect.TreeMultimap"};
                              var27 = names;
                              var28 = names.length;

                              for(var29 = 0; var29 < var28; ++var29) {
                                 name = var27[var29];
                                 if (name.equals(className)) {
                                    GuavaCodec writer;
                                    this.put((Type)Class.forName(name), (ObjectSerializer)(writer = GuavaCodec.instance));
                                    return writer;
                                 }
                              }
                           } catch (ClassNotFoundException var14) {
                              guavaError = true;
                           }
                        }

                        if (!jsonnullError && className.equals("net.sf.json.JSONNull")) {
                           try {
                              MiscCodec writer;
                              this.put((Type)Class.forName("net.sf.json.JSONNull"), (ObjectSerializer)(writer = MiscCodec.instance));
                              return writer;
                           } catch (ClassNotFoundException var12) {
                              jsonnullError = true;
                           }
                        }

                        Class[] interfaces = clazz.getInterfaces();
                        if (interfaces.length == 1 && interfaces[0].isAnnotation()) {
                           return AnnotationSerializer.instance;
                        }

                        Class handlerClass;
                        ObjectSerializer superWriter;
                        if (TypeUtils.isProxy(clazz)) {
                           handlerClass = clazz.getSuperclass();
                           superWriter = this.getObjectWriter(handlerClass);
                           this.put((Type)clazz, (ObjectSerializer)superWriter);
                           return superWriter;
                        }

                        if (Proxy.isProxyClass(clazz)) {
                           handlerClass = null;
                           if (interfaces.length == 2) {
                              handlerClass = interfaces[1];
                           } else {
                              Class[] var32 = interfaces;
                              var29 = interfaces.length;

                              for(int var38 = 0; var38 < var29; ++var38) {
                                 Class proxiedInterface = var32[var38];
                                 if (!proxiedInterface.getName().startsWith("org.springframework.aop.")) {
                                    if (handlerClass != null) {
                                       handlerClass = null;
                                       break;
                                    }

                                    handlerClass = proxiedInterface;
                                 }
                              }
                           }

                           if (handlerClass != null) {
                              superWriter = this.getObjectWriter(handlerClass);
                              this.put((Type)clazz, (ObjectSerializer)superWriter);
                              return superWriter;
                           }
                        }

                        if (create) {
                           writer = this.createJavaBeanSerializer(clazz);
                           this.put((Type)clazz, (ObjectSerializer)writer);
                        }
                     }
                  } else {
                     this.put((Type)clazz, (ObjectSerializer)(writer = CalendarCodec.instance));
                  }
               } else {
                  this.put((Type)clazz, (ObjectSerializer)(writer = MiscCodec.instance));
               }
            }
         }

         if (writer == null) {
            writer = (ObjectSerializer)this.serializers.get(clazz);
         }
      }

      return (ObjectSerializer)writer;
   }

   public final ObjectSerializer get(Type key) {
      return (ObjectSerializer)this.serializers.get(key);
   }

   public boolean put(Object type, Object value) {
      return this.put((Type)type, (ObjectSerializer)value);
   }

   public boolean put(Type type, ObjectSerializer value) {
      return this.serializers.put(type, value);
   }

   public void configEnumAsJavaBean(Class<? extends Enum>... enumClasses) {
      Class[] var2 = enumClasses;
      int var3 = enumClasses.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Class<? extends Enum> enumClass = var2[var4];
         this.put((Type)enumClass, (ObjectSerializer)this.createJavaBeanSerializer(enumClass));
      }

   }

   public void setPropertyNamingStrategy(PropertyNamingStrategy propertyNamingStrategy) {
      this.propertyNamingStrategy = propertyNamingStrategy;
   }

   public void clearSerializers() {
      this.serializers.clear();
      this.initSerializers();
   }
}
