package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSON;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public abstract class SerializeFilterable {
   protected List<BeforeFilter> beforeFilters = null;
   protected List<AfterFilter> afterFilters = null;
   protected List<PropertyFilter> propertyFilters = null;
   protected List<ValueFilter> valueFilters = null;
   protected List<NameFilter> nameFilters = null;
   protected List<PropertyPreFilter> propertyPreFilters = null;
   protected List<LabelFilter> labelFilters = null;
   protected List<ContextValueFilter> contextValueFilters = null;
   protected boolean writeDirect = true;

   public List<BeforeFilter> getBeforeFilters() {
      if (this.beforeFilters == null) {
         this.beforeFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.beforeFilters;
   }

   public List<AfterFilter> getAfterFilters() {
      if (this.afterFilters == null) {
         this.afterFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.afterFilters;
   }

   public List<NameFilter> getNameFilters() {
      if (this.nameFilters == null) {
         this.nameFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.nameFilters;
   }

   public List<PropertyPreFilter> getPropertyPreFilters() {
      if (this.propertyPreFilters == null) {
         this.propertyPreFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.propertyPreFilters;
   }

   public List<LabelFilter> getLabelFilters() {
      if (this.labelFilters == null) {
         this.labelFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.labelFilters;
   }

   public List<PropertyFilter> getPropertyFilters() {
      if (this.propertyFilters == null) {
         this.propertyFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.propertyFilters;
   }

   public List<ContextValueFilter> getContextValueFilters() {
      if (this.contextValueFilters == null) {
         this.contextValueFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.contextValueFilters;
   }

   public List<ValueFilter> getValueFilters() {
      if (this.valueFilters == null) {
         this.valueFilters = new ArrayList();
         this.writeDirect = false;
      }

      return this.valueFilters;
   }

   public void addFilter(SerializeFilter filter) {
      if (filter != null) {
         if (filter instanceof PropertyPreFilter) {
            this.getPropertyPreFilters().add((PropertyPreFilter)filter);
         }

         if (filter instanceof NameFilter) {
            this.getNameFilters().add((NameFilter)filter);
         }

         if (filter instanceof ValueFilter) {
            this.getValueFilters().add((ValueFilter)filter);
         }

         if (filter instanceof ContextValueFilter) {
            this.getContextValueFilters().add((ContextValueFilter)filter);
         }

         if (filter instanceof PropertyFilter) {
            this.getPropertyFilters().add((PropertyFilter)filter);
         }

         if (filter instanceof BeforeFilter) {
            this.getBeforeFilters().add((BeforeFilter)filter);
         }

         if (filter instanceof AfterFilter) {
            this.getAfterFilters().add((AfterFilter)filter);
         }

         if (filter instanceof LabelFilter) {
            this.getLabelFilters().add((LabelFilter)filter);
         }

      }
   }

   public boolean applyName(JSONSerializer jsonBeanDeser, Object object, String key) {
      Iterator var4;
      PropertyPreFilter filter;
      if (jsonBeanDeser.propertyPreFilters != null) {
         var4 = jsonBeanDeser.propertyPreFilters.iterator();

         while(var4.hasNext()) {
            filter = (PropertyPreFilter)var4.next();
            if (!filter.apply(jsonBeanDeser, object, key)) {
               return false;
            }
         }
      }

      if (this.propertyPreFilters != null) {
         var4 = this.propertyPreFilters.iterator();

         while(var4.hasNext()) {
            filter = (PropertyPreFilter)var4.next();
            if (!filter.apply(jsonBeanDeser, object, key)) {
               return false;
            }
         }
      }

      return true;
   }

   public boolean apply(JSONSerializer jsonBeanDeser, Object object, String key, Object propertyValue) {
      Iterator var5;
      PropertyFilter propertyFilter;
      if (jsonBeanDeser.propertyFilters != null) {
         var5 = jsonBeanDeser.propertyFilters.iterator();

         while(var5.hasNext()) {
            propertyFilter = (PropertyFilter)var5.next();
            if (!propertyFilter.apply(object, key, propertyValue)) {
               return false;
            }
         }
      }

      if (this.propertyFilters != null) {
         var5 = this.propertyFilters.iterator();

         while(var5.hasNext()) {
            propertyFilter = (PropertyFilter)var5.next();
            if (!propertyFilter.apply(object, key, propertyValue)) {
               return false;
            }
         }
      }

      return true;
   }

   protected String processKey(JSONSerializer jsonBeanDeser, Object object, String key, Object propertyValue) {
      Iterator var5;
      NameFilter nameFilter;
      if (jsonBeanDeser.nameFilters != null) {
         for(var5 = jsonBeanDeser.nameFilters.iterator(); var5.hasNext(); key = nameFilter.process(object, key, propertyValue)) {
            nameFilter = (NameFilter)var5.next();
         }
      }

      if (this.nameFilters != null) {
         for(var5 = this.nameFilters.iterator(); var5.hasNext(); key = nameFilter.process(object, key, propertyValue)) {
            nameFilter = (NameFilter)var5.next();
         }
      }

      return key;
   }

   protected Object processValue(JSONSerializer jsonBeanDeser, BeanContext beanContext, Object object, String key, Object propertyValue) {
      if (propertyValue != null) {
         String format;
         if ((jsonBeanDeser.out.writeNonStringValueAsString || beanContext != null && (beanContext.getFeatures() & SerializerFeature.WriteNonStringValueAsString.mask) != 0) && (propertyValue instanceof Number || propertyValue instanceof Boolean)) {
            format = null;
            if (propertyValue instanceof Number && beanContext != null) {
               format = beanContext.getFormat();
            }

            if (format != null) {
               propertyValue = (new DecimalFormat(format)).format(propertyValue);
            } else {
               propertyValue = propertyValue.toString();
            }
         } else if (beanContext != null && beanContext.isJsonDirect()) {
            format = (String)propertyValue;
            propertyValue = JSON.parse(format);
         }
      }

      ValueFilter valueFilter;
      if (jsonBeanDeser.valueFilters != null) {
         for(Iterator var9 = jsonBeanDeser.valueFilters.iterator(); var9.hasNext(); propertyValue = valueFilter.process(object, key, propertyValue)) {
            valueFilter = (ValueFilter)var9.next();
         }
      }

      List<ValueFilter> valueFilters = this.valueFilters;
      ValueFilter valueFilter;
      Iterator var10;
      if (valueFilters != null) {
         for(var10 = valueFilters.iterator(); var10.hasNext(); propertyValue = valueFilter.process(object, key, propertyValue)) {
            valueFilter = (ValueFilter)var10.next();
         }
      }

      ContextValueFilter valueFilter;
      if (jsonBeanDeser.contextValueFilters != null) {
         for(var10 = jsonBeanDeser.contextValueFilters.iterator(); var10.hasNext(); propertyValue = valueFilter.process(beanContext, object, key, propertyValue)) {
            valueFilter = (ContextValueFilter)var10.next();
         }
      }

      if (this.contextValueFilters != null) {
         for(var10 = this.contextValueFilters.iterator(); var10.hasNext(); propertyValue = valueFilter.process(beanContext, object, key, propertyValue)) {
            valueFilter = (ContextValueFilter)var10.next();
         }
      }

      return propertyValue;
   }

   protected boolean writeDirect(JSONSerializer jsonBeanDeser) {
      return jsonBeanDeser.out.writeDirect && this.writeDirect && jsonBeanDeser.writeDirect;
   }
}
