package com.alibaba.fastjson.support.jaxrs;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.serializer.SerializeFilter;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alibaba.fastjson.support.config.FastJsonConfig;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.StreamingOutput;
import javax.ws.rs.ext.ContextResolver;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import javax.ws.rs.ext.Providers;

@Provider
@Consumes({"*/*"})
@Produces({"*/*"})
public class FastJsonProvider implements MessageBodyReader<Object>, MessageBodyWriter<Object> {
   public static final Class<?>[] DEFAULT_UNREADABLES = new Class[]{InputStream.class, Reader.class};
   public static final Class<?>[] DEFAULT_UNWRITABLES = new Class[]{InputStream.class, OutputStream.class, Writer.class, StreamingOutput.class, Response.class};
   /** @deprecated */
   @Deprecated
   protected Charset charset = Charset.forName("UTF-8");
   /** @deprecated */
   @Deprecated
   protected SerializerFeature[] features = new SerializerFeature[0];
   /** @deprecated */
   @Deprecated
   protected SerializeFilter[] filters = new SerializeFilter[0];
   /** @deprecated */
   @Deprecated
   protected String dateFormat;
   @Context
   protected Providers providers;
   private FastJsonConfig fastJsonConfig = new FastJsonConfig();
   private Class<?>[] clazzes = null;
   private boolean pretty;

   public FastJsonConfig getFastJsonConfig() {
      return this.fastJsonConfig;
   }

   public void setFastJsonConfig(FastJsonConfig fastJsonConfig) {
      this.fastJsonConfig = fastJsonConfig;
   }

   public FastJsonProvider() {
   }

   public FastJsonProvider(Class<?>[] clazzes) {
      this.clazzes = clazzes;
   }

   public FastJsonProvider setPretty(boolean p) {
      this.pretty = p;
      return this;
   }

   /** @deprecated */
   @Deprecated
   public FastJsonProvider(String charset) {
      this.fastJsonConfig.setCharset(Charset.forName(charset));
   }

   /** @deprecated */
   @Deprecated
   public Charset getCharset() {
      return this.fastJsonConfig.getCharset();
   }

   /** @deprecated */
   @Deprecated
   public void setCharset(Charset charset) {
      this.fastJsonConfig.setCharset(charset);
   }

   /** @deprecated */
   @Deprecated
   public String getDateFormat() {
      return this.fastJsonConfig.getDateFormat();
   }

   /** @deprecated */
   @Deprecated
   public void setDateFormat(String dateFormat) {
      this.fastJsonConfig.setDateFormat(dateFormat);
   }

   /** @deprecated */
   @Deprecated
   public SerializerFeature[] getFeatures() {
      return this.fastJsonConfig.getSerializerFeatures();
   }

   /** @deprecated */
   @Deprecated
   public void setFeatures(SerializerFeature... features) {
      this.fastJsonConfig.setSerializerFeatures(features);
   }

   /** @deprecated */
   @Deprecated
   public SerializeFilter[] getFilters() {
      return this.fastJsonConfig.getSerializeFilters();
   }

   /** @deprecated */
   @Deprecated
   public void setFilters(SerializeFilter... filters) {
      this.fastJsonConfig.setSerializeFilters(filters);
   }

   protected boolean isAssignableFrom(Class<?> type, Class<?>[] classes) {
      if (type == null) {
         return false;
      } else {
         Class[] var3 = classes;
         int var4 = classes.length;

         for(int var5 = 0; var5 < var4; ++var5) {
            Class<?> cls = var3[var5];
            if (cls.isAssignableFrom(type)) {
               return false;
            }
         }

         return true;
      }
   }

   protected boolean isValidType(Class<?> type, Annotation[] classAnnotations) {
      if (type == null) {
         return false;
      } else if (this.clazzes != null) {
         Class[] var3 = this.clazzes;
         int var4 = var3.length;

         for(int var5 = 0; var5 < var4; ++var5) {
            Class<?> cls = var3[var5];
            if (cls == type) {
               return true;
            }
         }

         return false;
      } else {
         return true;
      }
   }

   protected boolean hasMatchingMediaType(MediaType mediaType) {
      if (mediaType == null) {
         return true;
      } else {
         String subtype = mediaType.getSubtype();
         return "json".equalsIgnoreCase(subtype) || subtype.endsWith("+json") || "javascript".equals(subtype) || "x-javascript".equals(subtype) || "x-json".equals(subtype) || "x-www-form-urlencoded".equalsIgnoreCase(subtype) || subtype.endsWith("x-www-form-urlencoded");
      }
   }

   public boolean isWriteable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
      if (!this.hasMatchingMediaType(mediaType)) {
         return false;
      } else {
         return !this.isAssignableFrom(type, DEFAULT_UNWRITABLES) ? false : this.isValidType(type, annotations);
      }
   }

   public long getSize(Object t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
      return -1L;
   }

   public void writeTo(Object obj, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType, MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream) throws IOException, WebApplicationException {
      FastJsonConfig fastJsonConfig = this.locateConfigProvider(type, mediaType);
      SerializerFeature[] serializerFeatures = fastJsonConfig.getSerializerFeatures();
      if (this.pretty) {
         if (serializerFeatures == null) {
            serializerFeatures = new SerializerFeature[]{SerializerFeature.PrettyFormat};
         } else {
            List<SerializerFeature> featureList = new ArrayList(Arrays.asList(serializerFeatures));
            featureList.add(SerializerFeature.PrettyFormat);
            serializerFeatures = (SerializerFeature[])featureList.toArray(serializerFeatures);
         }

         fastJsonConfig.setSerializerFeatures(serializerFeatures);
      }

      try {
         JSON.writeJSONString(entityStream, fastJsonConfig.getCharset(), obj, fastJsonConfig.getSerializeConfig(), fastJsonConfig.getSerializeFilters(), fastJsonConfig.getDateFormat(), JSON.DEFAULT_GENERATE_FEATURE, fastJsonConfig.getSerializerFeatures());
         entityStream.flush();
      } catch (JSONException var11) {
         throw new WebApplicationException("Could not write JSON: " + var11.getMessage(), var11);
      }
   }

   public boolean isReadable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
      if (!this.hasMatchingMediaType(mediaType)) {
         return false;
      } else {
         return !this.isAssignableFrom(type, DEFAULT_UNREADABLES) ? false : this.isValidType(type, annotations);
      }
   }

   public Object readFrom(Class<Object> type, Type genericType, Annotation[] annotations, MediaType mediaType, MultivaluedMap<String, String> httpHeaders, InputStream entityStream) throws IOException, WebApplicationException {
      try {
         FastJsonConfig fastJsonConfig = this.locateConfigProvider(type, mediaType);
         return JSON.parseObject(entityStream, fastJsonConfig.getCharset(), genericType, fastJsonConfig.getFeatures());
      } catch (JSONException var8) {
         throw new WebApplicationException("JSON parse error: " + var8.getMessage(), var8);
      }
   }

   protected FastJsonConfig locateConfigProvider(Class<?> type, MediaType mediaType) {
      if (this.providers != null) {
         ContextResolver<FastJsonConfig> resolver = this.providers.getContextResolver(FastJsonConfig.class, mediaType);
         if (resolver == null) {
            resolver = this.providers.getContextResolver(FastJsonConfig.class, (MediaType)null);
         }

         if (resolver != null) {
            return (FastJsonConfig)resolver.getContext(type);
         }
      }

      return this.fastJsonConfig;
   }
}
