package com.alibaba.fastjson.support.retrofit;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.alibaba.fastjson.serializer.SerializerFeature;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import okhttp3.MediaType;
import okhttp3.RequestBody;
import okhttp3.ResponseBody;
import retrofit2.Converter;
import retrofit2.Retrofit;
import retrofit2.Converter.Factory;

public class Retrofit2ConverterFactory extends Factory {
   private static final MediaType MEDIA_TYPE = MediaType.parse("application/json; charset=UTF-8");
   private static final Feature[] EMPTY_SERIALIZER_FEATURES = new Feature[0];
   private ParserConfig parserConfig = ParserConfig.getGlobalInstance();
   private int featureValues;
   private Feature[] features;
   private SerializeConfig serializeConfig;
   private SerializerFeature[] serializerFeatures;

   public Retrofit2ConverterFactory() {
      this.featureValues = JSON.DEFAULT_PARSER_FEATURE;
   }

   public Converter<ResponseBody, ?> responseBodyConverter(Type type, Annotation[] annotations, Retrofit retrofit) {
      return new Retrofit2ConverterFactory.ResponseBodyConverter(type);
   }

   public Converter<?, RequestBody> requestBodyConverter(Type type, Annotation[] parameterAnnotations, Annotation[] methodAnnotations, Retrofit retrofit) {
      return new Retrofit2ConverterFactory.RequestBodyConverter();
   }

   public ParserConfig getParserConfig() {
      return this.parserConfig;
   }

   public Retrofit2ConverterFactory setParserConfig(ParserConfig config) {
      this.parserConfig = config;
      return this;
   }

   public int getParserFeatureValues() {
      return this.featureValues;
   }

   public Retrofit2ConverterFactory setParserFeatureValues(int featureValues) {
      this.featureValues = featureValues;
      return this;
   }

   public Feature[] getParserFeatures() {
      return this.features;
   }

   public Retrofit2ConverterFactory setParserFeatures(Feature[] features) {
      this.features = features;
      return this;
   }

   public SerializeConfig getSerializeConfig() {
      return this.serializeConfig;
   }

   public Retrofit2ConverterFactory setSerializeConfig(SerializeConfig serializeConfig) {
      this.serializeConfig = serializeConfig;
      return this;
   }

   public SerializerFeature[] getSerializerFeatures() {
      return this.serializerFeatures;
   }

   public Retrofit2ConverterFactory setSerializerFeatures(SerializerFeature[] features) {
      this.serializerFeatures = features;
      return this;
   }

   final class RequestBodyConverter<T> implements Converter<T, RequestBody> {
      public RequestBody convert(T value) throws IOException {
         byte[] content = JSON.toJSONBytes(value, Retrofit2ConverterFactory.this.serializeConfig == null ? SerializeConfig.globalInstance : Retrofit2ConverterFactory.this.serializeConfig, Retrofit2ConverterFactory.this.serializerFeatures == null ? SerializerFeature.EMPTY : Retrofit2ConverterFactory.this.serializerFeatures);
         return RequestBody.create(Retrofit2ConverterFactory.MEDIA_TYPE, content);
      }
   }

   final class ResponseBodyConverter<T> implements Converter<ResponseBody, T> {
      private Type type;

      ResponseBodyConverter(Type type) {
         this.type = type;
      }

      public T convert(ResponseBody value) throws IOException {
         Object var2;
         try {
            var2 = JSON.parseObject(value.string(), this.type, Retrofit2ConverterFactory.this.parserConfig, Retrofit2ConverterFactory.this.featureValues, Retrofit2ConverterFactory.this.features != null ? Retrofit2ConverterFactory.this.features : Retrofit2ConverterFactory.EMPTY_SERIALIZER_FEATURES);
         } finally {
            value.close();
         }

         return var2;
      }
   }
}
