package com.alibaba.fastjson.support.spring;

import com.alibaba.fastjson.support.spring.annotation.FastJsonFilter;
import com.alibaba.fastjson.support.spring.annotation.FastJsonView;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.Order;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

@Order
@ControllerAdvice
public class FastJsonViewResponseBodyAdvice implements ResponseBodyAdvice<Object> {
   public boolean supports(MethodParameter returnType, Class<? extends HttpMessageConverter<?>> converterType) {
      return FastJsonHttpMessageConverter.class.isAssignableFrom(converterType) && returnType.hasMethodAnnotation(FastJsonView.class);
   }

   public FastJsonContainer beforeBodyWrite(Object body, MethodParameter returnType, MediaType selectedContentType, Class<? extends HttpMessageConverter<?>> selectedConverterType, ServerHttpRequest request, ServerHttpResponse response) {
      FastJsonContainer container = this.getOrCreateContainer(body);
      this.beforeBodyWriteInternal(container, selectedContentType, returnType, request, response);
      return container;
   }

   private FastJsonContainer getOrCreateContainer(Object body) {
      return body instanceof FastJsonContainer ? (FastJsonContainer)body : new FastJsonContainer(body);
   }

   protected void beforeBodyWriteInternal(FastJsonContainer container, MediaType contentType, MethodParameter returnType, ServerHttpRequest request, ServerHttpResponse response) {
      FastJsonView annotation = (FastJsonView)returnType.getMethodAnnotation(FastJsonView.class);
      FastJsonFilter[] include = annotation.include();
      FastJsonFilter[] exclude = annotation.exclude();
      PropertyPreFilters filters = new PropertyPreFilters();
      FastJsonFilter[] var10 = include;
      int var11 = include.length;

      int var12;
      FastJsonFilter item;
      for(var12 = 0; var12 < var11; ++var12) {
         item = var10[var12];
         filters.addFilter(item.clazz(), item.props());
      }

      var10 = exclude;
      var11 = exclude.length;

      for(var12 = 0; var12 < var11; ++var12) {
         item = var10[var12];
         filters.addFilter(item.clazz()).addExcludes(item.props());
      }

      container.setFilters(filters);
   }
}
