package com.alibaba.fastjson.util;

import java.util.Arrays;

public class IdentityHashMap<K, V> {
   private final IdentityHashMap.Entry<K, V>[] buckets;
   private final int indexMask;
   public static final int DEFAULT_SIZE = 8192;

   public IdentityHashMap() {
      this(8192);
   }

   public IdentityHashMap(int tableSize) {
      this.indexMask = tableSize - 1;
      this.buckets = new IdentityHashMap.Entry[tableSize];
   }

   public final V get(K key) {
      int hash = System.identityHashCode(key);
      int bucket = hash & this.indexMask;

      for(IdentityHashMap.Entry entry = this.buckets[bucket]; entry != null; entry = entry.next) {
         if (key == entry.key) {
            return entry.value;
         }
      }

      return null;
   }

   public Class findClass(String keyString) {
      for(int i = 0; i < this.buckets.length; ++i) {
         IdentityHashMap.Entry bucket = this.buckets[i];
         if (bucket != null) {
            for(IdentityHashMap.Entry entry = bucket; entry != null; entry = entry.next) {
               Object key = bucket.key;
               if (key instanceof Class) {
                  Class clazz = (Class)key;
                  String className = clazz.getName();
                  if (className.equals(keyString)) {
                     return clazz;
                  }
               }
            }
         }
      }

      return null;
   }

   public boolean put(K key, V value) {
      int hash = System.identityHashCode(key);
      int bucket = hash & this.indexMask;

      IdentityHashMap.Entry entry;
      for(entry = this.buckets[bucket]; entry != null; entry = entry.next) {
         if (key == entry.key) {
            entry.value = value;
            return true;
         }
      }

      entry = new IdentityHashMap.Entry(key, value, hash, this.buckets[bucket]);
      this.buckets[bucket] = entry;
      return false;
   }

   public void clear() {
      Arrays.fill(this.buckets, (Object)null);
   }

   protected static final class Entry<K, V> {
      public final int hashCode;
      public final K key;
      public V value;
      public final IdentityHashMap.Entry<K, V> next;

      public Entry(K key, V value, int hash, IdentityHashMap.Entry<K, V> next) {
         this.key = key;
         this.value = value;
         this.next = next;
         this.hashCode = hash;
      }
   }
}
