package com.alibaba.fastjson.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.annotation.JSONType;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.parser.JSONScanner;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.parser.deserializer.EnumDeserializer;
import com.alibaba.fastjson.parser.deserializer.JavaBeanDeserializer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import com.alibaba.fastjson.serializer.CalendarCodec;
import com.alibaba.fastjson.serializer.SerializeBeanInfo;
import com.alibaba.fastjson.serializer.SerializerFeature;
import java.lang.annotation.Annotation;
import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Proxy;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;
import java.lang.reflect.WildcardType;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.security.AccessControlException;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.AbstractCollection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.BitSet;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Currency;
import java.util.Date;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.UUID;
import java.util.WeakHashMap;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

public class TypeUtils {
   public static boolean compatibleWithJavaBean = false;
   public static boolean compatibleWithFieldName = false;
   private static boolean setAccessibleEnable = true;
   private static boolean oracleTimestampMethodInited = false;
   private static Method oracleTimestampMethod;
   private static boolean oracleDateMethodInited = false;
   private static Method oracleDateMethod;
   private static boolean optionalClassInited = false;
   private static Class<?> optionalClass;
   private static boolean transientClassInited = false;
   private static Class<? extends Annotation> transientClass;
   private static Class<? extends Annotation> class_OneToMany = null;
   private static boolean class_OneToMany_error = false;
   private static Class<? extends Annotation> class_ManyToMany = null;
   private static boolean class_ManyToMany_error = false;
   private static Method method_HibernateIsInitialized = null;
   private static boolean method_HibernateIsInitialized_error = false;
   private static volatile Class kotlin_metadata;
   private static volatile boolean kotlin_metadata_error;
   private static volatile boolean kotlin_class_klass_error;
   private static volatile Constructor kotlin_kclass_constructor;
   private static volatile Method kotlin_kclass_getConstructors;
   private static volatile Method kotlin_kfunction_getParameters;
   private static volatile Method kotlin_kparameter_getName;
   private static volatile boolean kotlin_error;
   private static volatile Map<Class, String[]> kotlinIgnores;
   private static volatile boolean kotlinIgnores_error;
   private static ConcurrentMap<String, Class<?>> mappings = new ConcurrentHashMap(16, 0.75F, 1);
   private static Class<?> pathClass;
   private static boolean pathClass_error = false;

   public static String castToString(Object value) {
      return value == null ? null : value.toString();
   }

   public static Byte castToByte(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Number) {
         return ((Number)value).byteValue();
      } else if (value instanceof String) {
         String strVal = (String)value;
         return strVal.length() != 0 && !"null".equals(strVal) && !"NULL".equals(strVal) ? Byte.parseByte(strVal) : null;
      } else {
         throw new JSONException("can not cast to byte, value : " + value);
      }
   }

   public static Character castToChar(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Character) {
         return (Character)value;
      } else if (value instanceof String) {
         String strVal = (String)value;
         if (strVal.length() == 0) {
            return null;
         } else if (strVal.length() != 1) {
            throw new JSONException("can not cast to char, value : " + value);
         } else {
            return strVal.charAt(0);
         }
      } else {
         throw new JSONException("can not cast to char, value : " + value);
      }
   }

   public static Short castToShort(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Number) {
         return ((Number)value).shortValue();
      } else if (value instanceof String) {
         String strVal = (String)value;
         return strVal.length() != 0 && !"null".equals(strVal) && !"NULL".equals(strVal) ? Short.parseShort(strVal) : null;
      } else {
         throw new JSONException("can not cast to short, value : " + value);
      }
   }

   public static BigDecimal castToBigDecimal(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof BigDecimal) {
         return (BigDecimal)value;
      } else if (value instanceof BigInteger) {
         return new BigDecimal((BigInteger)value);
      } else {
         String strVal = value.toString();
         if (strVal.length() == 0) {
            return null;
         } else {
            return value instanceof Map && ((Map)value).size() == 0 ? null : new BigDecimal(strVal);
         }
      }
   }

   public static BigInteger castToBigInteger(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof BigInteger) {
         return (BigInteger)value;
      } else if (!(value instanceof Float) && !(value instanceof Double)) {
         String strVal = value.toString();
         return strVal.length() != 0 && !"null".equals(strVal) && !"NULL".equals(strVal) ? new BigInteger(strVal) : null;
      } else {
         return BigInteger.valueOf(((Number)value).longValue());
      }
   }

   public static Float castToFloat(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Number) {
         return ((Number)value).floatValue();
      } else if (value instanceof String) {
         String strVal = value.toString();
         if (strVal.length() != 0 && !"null".equals(strVal) && !"NULL".equals(strVal)) {
            if (strVal.indexOf(44) != 0) {
               strVal = strVal.replaceAll(",", "");
            }

            return Float.parseFloat(strVal);
         } else {
            return null;
         }
      } else {
         throw new JSONException("can not cast to float, value : " + value);
      }
   }

   public static Double castToDouble(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Number) {
         return ((Number)value).doubleValue();
      } else if (value instanceof String) {
         String strVal = value.toString();
         if (strVal.length() != 0 && !"null".equals(strVal) && !"NULL".equals(strVal)) {
            if (strVal.indexOf(44) != 0) {
               strVal = strVal.replaceAll(",", "");
            }

            return Double.parseDouble(strVal);
         } else {
            return null;
         }
      } else {
         throw new JSONException("can not cast to double, value : " + value);
      }
   }

   public static Date castToDate(Object value) {
      return castToDate(value, (String)null);
   }

   public static Date castToDate(Object value, String format) {
      if (value == null) {
         return null;
      } else if (value instanceof Date) {
         return (Date)value;
      } else if (value instanceof Calendar) {
         return ((Calendar)value).getTime();
      } else {
         long longValue = -1L;
         if (value instanceof Number) {
            longValue = ((Number)value).longValue();
            return new Date(longValue);
         } else {
            if (value instanceof String) {
               String strVal = (String)value;
               JSONScanner dateLexer = new JSONScanner(strVal);

               try {
                  if (dateLexer.scanISO8601DateIfMatch(false)) {
                     Calendar calendar = dateLexer.getCalendar();
                     Date var7 = calendar.getTime();
                     return var7;
                  }
               } finally {
                  dateLexer.close();
               }

               if (strVal.startsWith("/Date(") && strVal.endsWith(")/")) {
                  strVal = strVal.substring(6, strVal.length() - 2);
               }

               if (strVal.indexOf(45) != -1) {
                  if (format == null) {
                     if (strVal.length() != JSON.DEFFAULT_DATE_FORMAT.length() && (strVal.length() != 22 || !JSON.DEFFAULT_DATE_FORMAT.equals("yyyyMMddHHmmssSSSZ"))) {
                        if (strVal.length() == 10) {
                           format = "yyyy-MM-dd";
                        } else if (strVal.length() == "yyyy-MM-dd HH:mm:ss".length()) {
                           format = "yyyy-MM-dd HH:mm:ss";
                        } else if (strVal.length() == 29 && strVal.charAt(26) == ':' && strVal.charAt(28) == '0') {
                           format = "yyyy-MM-dd'T'HH:mm:ss.SSSXXX";
                        } else {
                           format = "yyyy-MM-dd HH:mm:ss.SSS";
                        }
                     } else {
                        format = JSON.DEFFAULT_DATE_FORMAT;
                     }
                  }

                  SimpleDateFormat dateFormat = new SimpleDateFormat(format, JSON.defaultLocale);
                  dateFormat.setTimeZone(JSON.defaultTimeZone);

                  try {
                     return dateFormat.parse(strVal);
                  } catch (ParseException var35) {
                     throw new JSONException("can not cast to Date, value : " + strVal);
                  }
               }

               if (strVal.length() == 0) {
                  return null;
               }

               longValue = Long.parseLong(strVal);
            }

            if (longValue < 0L) {
               Class<?> clazz = value.getClass();
               Object result;
               if ("oracle.sql.TIMESTAMP".equals(clazz.getName())) {
                  if (oracleTimestampMethod == null && !oracleTimestampMethodInited) {
                     try {
                        oracleTimestampMethod = clazz.getMethod("toJdbc");
                     } catch (NoSuchMethodException var37) {
                     } finally {
                        oracleTimestampMethodInited = true;
                     }
                  }

                  try {
                     result = oracleTimestampMethod.invoke(value);
                  } catch (Exception var36) {
                     throw new JSONException("can not cast oracle.sql.TIMESTAMP to Date", var36);
                  }

                  return (Date)result;
               } else if ("oracle.sql.DATE".equals(clazz.getName())) {
                  if (oracleDateMethod == null && !oracleDateMethodInited) {
                     try {
                        oracleDateMethod = clazz.getMethod("toJdbc");
                     } catch (NoSuchMethodException var40) {
                     } finally {
                        oracleDateMethodInited = true;
                     }
                  }

                  try {
                     result = oracleDateMethod.invoke(value);
                  } catch (Exception var39) {
                     throw new JSONException("can not cast oracle.sql.DATE to Date", var39);
                  }

                  return (Date)result;
               } else {
                  throw new JSONException("can not cast to Date, value : " + value);
               }
            } else {
               return new Date(longValue);
            }
         }
      }
   }

   public static java.sql.Date castToSqlDate(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof java.sql.Date) {
         return (java.sql.Date)value;
      } else if (value instanceof Date) {
         return new java.sql.Date(((Date)value).getTime());
      } else if (value instanceof Calendar) {
         return new java.sql.Date(((Calendar)value).getTimeInMillis());
      } else {
         long longValue = 0L;
         if (value instanceof Number) {
            longValue = ((Number)value).longValue();
         }

         if (value instanceof String) {
            String strVal = (String)value;
            if (strVal.length() == 0 || "null".equals(strVal) || "NULL".equals(strVal)) {
               return null;
            }

            if (isNumber(strVal)) {
               longValue = Long.parseLong(strVal);
            } else {
               JSONScanner scanner = new JSONScanner(strVal);
               if (!scanner.scanISO8601DateIfMatch(false)) {
                  throw new JSONException("can not cast to Timestamp, value : " + strVal);
               }

               longValue = scanner.getCalendar().getTime().getTime();
            }
         }

         if (longValue <= 0L) {
            throw new JSONException("can not cast to Date, value : " + value);
         } else {
            return new java.sql.Date(longValue);
         }
      }
   }

   public static Time castToSqlTime(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Time) {
         return (Time)value;
      } else if (value instanceof Date) {
         return new Time(((Date)value).getTime());
      } else if (value instanceof Calendar) {
         return new Time(((Calendar)value).getTimeInMillis());
      } else {
         long longValue = 0L;
         if (value instanceof Number) {
            longValue = ((Number)value).longValue();
         }

         if (value instanceof String) {
            String strVal = (String)value;
            if (strVal.length() == 0 || "null".equalsIgnoreCase(strVal)) {
               return null;
            }

            if (isNumber(strVal)) {
               longValue = Long.parseLong(strVal);
            } else {
               JSONScanner scanner = new JSONScanner(strVal);
               if (!scanner.scanISO8601DateIfMatch(false)) {
                  throw new JSONException("can not cast to Timestamp, value : " + strVal);
               }

               longValue = scanner.getCalendar().getTime().getTime();
            }
         }

         if (longValue <= 0L) {
            throw new JSONException("can not cast to Date, value : " + value);
         } else {
            return new Time(longValue);
         }
      }
   }

   public static Timestamp castToTimestamp(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Calendar) {
         return new Timestamp(((Calendar)value).getTimeInMillis());
      } else if (value instanceof Timestamp) {
         return (Timestamp)value;
      } else if (value instanceof Date) {
         return new Timestamp(((Date)value).getTime());
      } else {
         long longValue = 0L;
         if (value instanceof Number) {
            longValue = ((Number)value).longValue();
         }

         if (value instanceof String) {
            String strVal = (String)value;
            if (strVal.length() == 0 || "null".equals(strVal) || "NULL".equals(strVal)) {
               return null;
            }

            if (strVal.endsWith(".000000000")) {
               strVal = strVal.substring(0, strVal.length() - 10);
            } else if (strVal.endsWith(".000000")) {
               strVal = strVal.substring(0, strVal.length() - 7);
            }

            if (isNumber(strVal)) {
               longValue = Long.parseLong(strVal);
            } else {
               JSONScanner scanner = new JSONScanner(strVal);
               if (!scanner.scanISO8601DateIfMatch(false)) {
                  throw new JSONException("can not cast to Timestamp, value : " + strVal);
               }

               longValue = scanner.getCalendar().getTime().getTime();
            }
         }

         if (longValue <= 0L) {
            throw new JSONException("can not cast to Timestamp, value : " + value);
         } else {
            return new Timestamp(longValue);
         }
      }
   }

   public static boolean isNumber(String str) {
      for(int i = 0; i < str.length(); ++i) {
         char ch = str.charAt(i);
         if (ch != '+' && ch != '-') {
            if (ch < '0' || ch > '9') {
               return false;
            }
         } else if (i != 0) {
            return false;
         }
      }

      return true;
   }

   public static Long castToLong(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Number) {
         return ((Number)value).longValue();
      } else {
         if (value instanceof String) {
            String strVal = (String)value;
            if (strVal.length() == 0 || "null".equals(strVal) || "NULL".equals(strVal)) {
               return null;
            }

            if (strVal.indexOf(44) != 0) {
               strVal = strVal.replaceAll(",", "");
            }

            try {
               return Long.parseLong(strVal);
            } catch (NumberFormatException var4) {
               JSONScanner dateParser = new JSONScanner(strVal);
               Calendar calendar = null;
               if (dateParser.scanISO8601DateIfMatch(false)) {
                  calendar = dateParser.getCalendar();
               }

               dateParser.close();
               if (calendar != null) {
                  return calendar.getTimeInMillis();
               }
            }
         }

         if (value instanceof Map) {
            Map map = (Map)value;
            if (map.size() == 2 && map.containsKey("andIncrement") && map.containsKey("andDecrement")) {
               Iterator iter = map.values().iterator();
               iter.next();
               Object value2 = iter.next();
               return castToLong(value2);
            }
         }

         throw new JSONException("can not cast to long, value : " + value);
      }
   }

   public static Integer castToInt(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Integer) {
         return (Integer)value;
      } else if (value instanceof Number) {
         return ((Number)value).intValue();
      } else if (value instanceof String) {
         String strVal = (String)value;
         if (strVal.length() != 0 && !"null".equals(strVal) && !"NULL".equals(strVal)) {
            if (strVal.indexOf(44) != 0) {
               strVal = strVal.replaceAll(",", "");
            }

            return Integer.parseInt(strVal);
         } else {
            return null;
         }
      } else if (value instanceof Boolean) {
         return (Boolean)value ? 1 : 0;
      } else {
         if (value instanceof Map) {
            Map map = (Map)value;
            if (map.size() == 2 && map.containsKey("andIncrement") && map.containsKey("andDecrement")) {
               Iterator iter = map.values().iterator();
               iter.next();
               Object value2 = iter.next();
               return castToInt(value2);
            }
         }

         throw new JSONException("can not cast to int, value : " + value);
      }
   }

   public static byte[] castToBytes(Object value) {
      if (value instanceof byte[]) {
         return (byte[])((byte[])value);
      } else if (value instanceof String) {
         return IOUtils.decodeBase64((String)value);
      } else {
         throw new JSONException("can not cast to int, value : " + value);
      }
   }

   public static Boolean castToBoolean(Object value) {
      if (value == null) {
         return null;
      } else if (value instanceof Boolean) {
         return (Boolean)value;
      } else if (value instanceof Number) {
         return ((Number)value).intValue() == 1;
      } else {
         if (value instanceof String) {
            String strVal = (String)value;
            if (strVal.length() == 0 || "null".equals(strVal) || "NULL".equals(strVal)) {
               return null;
            }

            if ("true".equalsIgnoreCase(strVal) || "1".equals(strVal)) {
               return Boolean.TRUE;
            }

            if ("false".equalsIgnoreCase(strVal) || "0".equals(strVal)) {
               return Boolean.FALSE;
            }

            if ("Y".equalsIgnoreCase(strVal) || "T".equals(strVal)) {
               return Boolean.TRUE;
            }

            if ("F".equalsIgnoreCase(strVal) || "N".equals(strVal)) {
               return Boolean.FALSE;
            }
         }

         throw new JSONException("can not cast to boolean, value : " + value);
      }
   }

   public static <T> T castToJavaBean(Object obj, Class<T> clazz) {
      return cast(obj, clazz, ParserConfig.getGlobalInstance());
   }

   public static <T> T cast(Object obj, Class<T> clazz, ParserConfig config) {
      if (obj == null) {
         if (clazz == Integer.TYPE) {
            return 0;
         } else if (clazz == Long.TYPE) {
            return 0L;
         } else if (clazz == Short.TYPE) {
            return Short.valueOf((short)0);
         } else if (clazz == Byte.TYPE) {
            return 0;
         } else if (clazz == Float.TYPE) {
            return 0.0F;
         } else if (clazz == Double.TYPE) {
            return 0.0D;
         } else {
            return clazz == Boolean.TYPE ? Boolean.FALSE : null;
         }
      } else if (clazz == null) {
         throw new IllegalArgumentException("clazz is null");
      } else if (clazz == obj.getClass()) {
         return obj;
      } else if (obj instanceof Map) {
         if (clazz == Map.class) {
            return obj;
         } else {
            Map map = (Map)obj;
            return clazz == Object.class && !map.containsKey(JSON.DEFAULT_TYPE_KEY) ? obj : castToJavaBean((Map)obj, clazz, config);
         }
      } else {
         if (clazz.isArray()) {
            if (obj instanceof Collection) {
               Collection collection = (Collection)obj;
               int index = 0;
               Object array = Array.newInstance(clazz.getComponentType(), collection.size());

               for(Iterator var6 = collection.iterator(); var6.hasNext(); ++index) {
                  Object item = var6.next();
                  Object value = cast(item, clazz.getComponentType(), config);
                  Array.set(array, index, value);
               }

               return array;
            }

            if (clazz == byte[].class) {
               return castToBytes(obj);
            }
         }

         if (clazz.isAssignableFrom(obj.getClass())) {
            return obj;
         } else if (clazz != Boolean.TYPE && clazz != Boolean.class) {
            if (clazz != Byte.TYPE && clazz != Byte.class) {
               if (clazz != Character.TYPE && clazz != Character.class) {
                  if (clazz != Short.TYPE && clazz != Short.class) {
                     if (clazz != Integer.TYPE && clazz != Integer.class) {
                        if (clazz != Long.TYPE && clazz != Long.class) {
                           if (clazz != Float.TYPE && clazz != Float.class) {
                              if (clazz != Double.TYPE && clazz != Double.class) {
                                 if (clazz == String.class) {
                                    return castToString(obj);
                                 } else if (clazz == BigDecimal.class) {
                                    return castToBigDecimal(obj);
                                 } else if (clazz == BigInteger.class) {
                                    return castToBigInteger(obj);
                                 } else if (clazz == Date.class) {
                                    return castToDate(obj);
                                 } else if (clazz == java.sql.Date.class) {
                                    return castToSqlDate(obj);
                                 } else if (clazz == Time.class) {
                                    return castToSqlTime(obj);
                                 } else if (clazz == Timestamp.class) {
                                    return castToTimestamp(obj);
                                 } else if (clazz.isEnum()) {
                                    return castToEnum(obj, clazz, config);
                                 } else if (Calendar.class.isAssignableFrom(clazz)) {
                                    Date date = castToDate(obj);
                                    Calendar calendar;
                                    if (clazz == Calendar.class) {
                                       calendar = Calendar.getInstance(JSON.defaultTimeZone, JSON.defaultLocale);
                                    } else {
                                       try {
                                          calendar = (Calendar)clazz.newInstance();
                                       } catch (Exception var9) {
                                          throw new JSONException("can not cast to : " + clazz.getName(), var9);
                                       }
                                    }

                                    calendar.setTime(date);
                                    return calendar;
                                 } else {
                                    String className = clazz.getName();
                                    if (className.equals("javax.xml.datatype.XMLGregorianCalendar")) {
                                       Date date = castToDate(obj);
                                       Calendar calendar = Calendar.getInstance(JSON.defaultTimeZone, JSON.defaultLocale);
                                       calendar.setTime(date);
                                       return CalendarCodec.instance.createXMLGregorianCalendar(calendar);
                                    } else {
                                       if (obj instanceof String) {
                                          String strVal = (String)obj;
                                          if (strVal.length() == 0 || "null".equals(strVal) || "NULL".equals(strVal)) {
                                             return null;
                                          }

                                          if (clazz == Currency.class) {
                                             return Currency.getInstance(strVal);
                                          }

                                          if (clazz == Locale.class) {
                                             return toLocale(strVal);
                                          }

                                          if (className.startsWith("java.time.")) {
                                             String json = JSON.toJSONString(strVal);
                                             return JSON.parseObject(json, clazz);
                                          }
                                       }

                                       throw new JSONException("can not cast to : " + clazz.getName());
                                    }
                                 }
                              } else {
                                 return castToDouble(obj);
                              }
                           } else {
                              return castToFloat(obj);
                           }
                        } else {
                           return castToLong(obj);
                        }
                     } else {
                        return castToInt(obj);
                     }
                  } else {
                     return castToShort(obj);
                  }
               } else {
                  return castToChar(obj);
               }
            } else {
               return castToByte(obj);
            }
         } else {
            return castToBoolean(obj);
         }
      }
   }

   public static Locale toLocale(String strVal) {
      String[] items = strVal.split("_");
      if (items.length == 1) {
         return new Locale(items[0]);
      } else {
         return items.length == 2 ? new Locale(items[0], items[1]) : new Locale(items[0], items[1], items[2]);
      }
   }

   public static <T> T castToEnum(Object obj, Class<T> clazz, ParserConfig mapping) {
      try {
         if (obj instanceof String) {
            String name = (String)obj;
            if (name.length() == 0) {
               return null;
            }

            if (mapping == null) {
               mapping = ParserConfig.getGlobalInstance();
            }

            ObjectDeserializer derializer = mapping.getDeserializer((Type)clazz);
            if (derializer instanceof EnumDeserializer) {
               EnumDeserializer enumDeserializer = (EnumDeserializer)derializer;
               return enumDeserializer.getEnumByHashCode(fnv1a_64(name));
            }

            return Enum.valueOf(clazz, name);
         }

         if (obj instanceof Number) {
            int ordinal = ((Number)obj).intValue();
            Object[] values = clazz.getEnumConstants();
            if (ordinal < values.length) {
               return values[ordinal];
            }
         }
      } catch (Exception var6) {
         throw new JSONException("can not cast to : " + clazz.getName(), var6);
      }

      throw new JSONException("can not cast to : " + clazz.getName());
   }

   public static <T> T cast(Object obj, Type type, ParserConfig mapping) {
      if (obj == null) {
         return null;
      } else if (type instanceof Class) {
         return cast(obj, (Class)type, mapping);
      } else if (type instanceof ParameterizedType) {
         return cast(obj, (ParameterizedType)type, mapping);
      } else {
         if (obj instanceof String) {
            String strVal = (String)obj;
            if (strVal.length() == 0 || "null".equals(strVal) || "NULL".equals(strVal)) {
               return null;
            }
         }

         if (type instanceof TypeVariable) {
            return obj;
         } else {
            throw new JSONException("can not cast to : " + type);
         }
      }
   }

   public static <T> T cast(Object obj, ParameterizedType type, ParserConfig mapping) {
      Type rawTye = type.getRawType();
      Type argType;
      if (rawTye == Set.class || rawTye == HashSet.class || rawTye == TreeSet.class || rawTye == Collection.class || rawTye == List.class || rawTye == ArrayList.class) {
         argType = type.getActualTypeArguments()[0];
         if (obj instanceof Iterable) {
            Object collection;
            if (rawTye != Set.class && rawTye != HashSet.class) {
               if (rawTye == TreeSet.class) {
                  collection = new TreeSet();
               } else {
                  collection = new ArrayList();
               }
            } else {
               collection = new HashSet();
            }

            Iterator it = ((Iterable)obj).iterator();

            while(it.hasNext()) {
               Object item = it.next();
               ((Collection)collection).add(cast(item, argType, mapping));
            }

            return collection;
         }
      }

      if (rawTye == Map.class || rawTye == HashMap.class) {
         argType = type.getActualTypeArguments()[0];
         Type valueType = type.getActualTypeArguments()[1];
         if (obj instanceof Map) {
            Map map = new HashMap();
            Iterator var7 = ((Map)obj).entrySet().iterator();

            while(var7.hasNext()) {
               Entry entry = (Entry)var7.next();
               Object key = cast(entry.getKey(), argType, mapping);
               Object value = cast(entry.getValue(), valueType, mapping);
               map.put(key, value);
            }

            return map;
         }
      }

      if (obj instanceof String) {
         String strVal = (String)obj;
         if (strVal.length() == 0) {
            return null;
         }
      }

      if (type.getActualTypeArguments().length == 1) {
         argType = type.getActualTypeArguments()[0];
         if (argType instanceof WildcardType) {
            return cast(obj, rawTye, mapping);
         }
      }

      if (rawTye == Entry.class && obj instanceof Map && ((Map)obj).size() == 1) {
         Entry entry = (Entry)((Map)obj).entrySet().iterator().next();
         return entry;
      } else {
         if (rawTye instanceof Class) {
            if (mapping == null) {
               mapping = ParserConfig.global;
            }

            ObjectDeserializer deserializer = mapping.getDeserializer(rawTye);
            if (deserializer != null) {
               String str = JSON.toJSONString(obj);
               DefaultJSONParser parser = new DefaultJSONParser(str, mapping);
               return deserializer.deserialze(parser, type, (Object)null);
            }
         }

         throw new JSONException("can not cast to : " + type);
      }
   }

   public static <T> T castToJavaBean(Map<String, Object> map, Class<T> clazz, ParserConfig config) {
      try {
         String className;
         String language;
         if (clazz == StackTraceElement.class) {
            String declaringClass = (String)map.get("className");
            className = (String)map.get("methodName");
            language = (String)map.get("fileName");
            Number value = (Number)map.get("lineNumber");
            int lineNumber;
            if (value == null) {
               lineNumber = 0;
            } else {
               lineNumber = value.intValue();
            }

            return new StackTraceElement(declaringClass, className, language, lineNumber);
         } else {
            Object arg0 = map.get(JSON.DEFAULT_TYPE_KEY);
            if (arg0 instanceof String) {
               className = (String)arg0;
               if (config == null) {
                  config = ParserConfig.global;
               }

               Class<?> loadClazz = config.checkAutoType(className, (Class)null);
               if (loadClazz == null) {
                  throw new ClassNotFoundException(className + " not found");
               }

               if (!loadClazz.equals(clazz)) {
                  return castToJavaBean(map, loadClazz, config);
               }
            }

            JSONObject object;
            ObjectDeserializer deserizer;
            if (clazz.isInterface()) {
               if (map instanceof JSONObject) {
                  object = (JSONObject)map;
               } else {
                  object = new JSONObject(map);
               }

               if (config == null) {
                  config = ParserConfig.getGlobalInstance();
               }

               deserizer = (ObjectDeserializer)config.getDeserializers().get(clazz);
               if (deserizer != null) {
                  language = JSON.toJSONString(object);
                  return JSON.parseObject(language, clazz);
               } else {
                  return Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(), new Class[]{clazz}, object);
               }
            } else {
               if (clazz == Locale.class) {
                  arg0 = map.get("language");
                  Object arg1 = map.get("country");
                  if (arg0 instanceof String) {
                     language = (String)arg0;
                     if (arg1 instanceof String) {
                        String country = (String)arg1;
                        return new Locale(language, country);
                     }

                     if (arg1 == null) {
                        return new Locale(language);
                     }
                  }
               }

               if (clazz == String.class && map instanceof JSONObject) {
                  return map.toString();
               } else {
                  if (clazz == LinkedHashMap.class && map instanceof JSONObject) {
                     object = (JSONObject)map;
                     Map innerMap = object.getInnerMap();
                     if (innerMap instanceof LinkedHashMap) {
                        return innerMap;
                     }

                     LinkedHashMap linkedHashMap = new LinkedHashMap();
                     linkedHashMap.putAll(innerMap);
                  }

                  if (config == null) {
                     config = ParserConfig.getGlobalInstance();
                  }

                  JavaBeanDeserializer javaBeanDeser = null;
                  deserizer = config.getDeserializer((Type)clazz);
                  if (deserizer instanceof JavaBeanDeserializer) {
                     javaBeanDeser = (JavaBeanDeserializer)deserizer;
                  }

                  if (javaBeanDeser == null) {
                     throw new JSONException("can not get javaBeanDeserializer. " + clazz.getName());
                  } else {
                     return javaBeanDeser.createInstance(map, config);
                  }
               }
            }
         }
      } catch (Exception var8) {
         throw new JSONException(var8.getMessage(), var8);
      }
   }

   private static void addBaseClassMappings() {
      mappings.put("byte", Byte.TYPE);
      mappings.put("short", Short.TYPE);
      mappings.put("int", Integer.TYPE);
      mappings.put("long", Long.TYPE);
      mappings.put("float", Float.TYPE);
      mappings.put("double", Double.TYPE);
      mappings.put("boolean", Boolean.TYPE);
      mappings.put("char", Character.TYPE);
      mappings.put("[byte", byte[].class);
      mappings.put("[short", short[].class);
      mappings.put("[int", int[].class);
      mappings.put("[long", long[].class);
      mappings.put("[float", float[].class);
      mappings.put("[double", double[].class);
      mappings.put("[boolean", boolean[].class);
      mappings.put("[char", char[].class);
      mappings.put("[B", byte[].class);
      mappings.put("[S", short[].class);
      mappings.put("[I", int[].class);
      mappings.put("[J", long[].class);
      mappings.put("[F", float[].class);
      mappings.put("[D", double[].class);
      mappings.put("[C", char[].class);
      mappings.put("[Z", boolean[].class);
      Class<?>[] classes = new Class[]{Object.class, Cloneable.class, loadClass("java.lang.AutoCloseable"), Exception.class, RuntimeException.class, IllegalAccessError.class, IllegalAccessException.class, IllegalArgumentException.class, IllegalMonitorStateException.class, IllegalStateException.class, IllegalThreadStateException.class, IndexOutOfBoundsException.class, InstantiationError.class, InstantiationException.class, InternalError.class, InterruptedException.class, LinkageError.class, NegativeArraySizeException.class, NoClassDefFoundError.class, NoSuchFieldError.class, NoSuchFieldException.class, NoSuchMethodError.class, NoSuchMethodException.class, NullPointerException.class, NumberFormatException.class, OutOfMemoryError.class, SecurityException.class, StackOverflowError.class, StringIndexOutOfBoundsException.class, TypeNotPresentException.class, VerifyError.class, StackTraceElement.class, HashMap.class, Hashtable.class, TreeMap.class, java.util.IdentityHashMap.class, WeakHashMap.class, LinkedHashMap.class, HashSet.class, LinkedHashSet.class, TreeSet.class, TimeUnit.class, ConcurrentHashMap.class, loadClass("java.util.concurrent.ConcurrentSkipListMap"), loadClass("java.util.concurrent.ConcurrentSkipListSet"), AtomicInteger.class, AtomicLong.class, Collections.EMPTY_MAP.getClass(), BitSet.class, Calendar.class, Date.class, Locale.class, UUID.class, Time.class, java.sql.Date.class, Timestamp.class, SimpleDateFormat.class, JSONObject.class};
      Class[] var1 = classes;
      int var2 = classes.length;

      int var3;
      for(var3 = 0; var3 < var2; ++var3) {
         Class clazz = var1[var3];
         if (clazz != null) {
            mappings.put(clazz.getName(), clazz);
         }
      }

      String[] awt = new String[]{"java.awt.Rectangle", "java.awt.Point", "java.awt.Font", "java.awt.Color"};
      String[] spring = awt;
      var3 = awt.length;

      int var11;
      for(var11 = 0; var11 < var3; ++var11) {
         String className = spring[var11];
         Class<?> clazz = loadClass(className);
         if (clazz == null) {
            break;
         }

         mappings.put(clazz.getName(), clazz);
      }

      spring = new String[]{"org.springframework.util.LinkedMultiValueMap", "org.springframework.util.LinkedCaseInsensitiveMap", "org.springframework.remoting.support.RemoteInvocation", "org.springframework.remoting.support.RemoteInvocationResult", "org.springframework.security.web.savedrequest.DefaultSavedRequest", "org.springframework.security.web.savedrequest.SavedCookie", "org.springframework.security.web.csrf.DefaultCsrfToken", "org.springframework.security.web.authentication.WebAuthenticationDetails", "org.springframework.security.core.context.SecurityContextImpl", "org.springframework.security.authentication.UsernamePasswordAuthenticationToken", "org.springframework.security.core.authority.SimpleGrantedAuthority", "org.springframework.security.core.userdetails.User"};
      String[] var10 = spring;
      var11 = spring.length;

      for(int var12 = 0; var12 < var11; ++var12) {
         String className = var10[var12];
         Class<?> clazz = loadClass(className);
         if (clazz == null) {
            break;
         }

         mappings.put(clazz.getName(), clazz);
      }

   }

   public static void clearClassMapping() {
      mappings.clear();
      addBaseClassMappings();
   }

   public static Class<?> loadClass(String className) {
      return loadClass(className, (ClassLoader)null);
   }

   public static boolean isPath(Class<?> clazz) {
      if (pathClass == null && !pathClass_error) {
         try {
            pathClass = Class.forName("java.nio.file.Path");
         } catch (Throwable var2) {
            pathClass_error = true;
         }
      }

      return pathClass != null ? pathClass.isAssignableFrom(clazz) : false;
   }

   public static Class<?> getClassFromMapping(String className) {
      return (Class)mappings.get(className);
   }

   public static Class<?> loadClass(String className, ClassLoader classLoader) {
      return loadClass(className, classLoader, true);
   }

   public static Class<?> loadClass(String className, ClassLoader classLoader, boolean cache) {
      if (className != null && className.length() != 0) {
         Class<?> clazz = (Class)mappings.get(className);
         if (clazz != null) {
            return clazz;
         } else if (className.charAt(0) == '[') {
            Class<?> componentType = loadClass(className.substring(1), classLoader);
            return Array.newInstance(componentType, 0).getClass();
         } else if (className.startsWith("L") && className.endsWith(";")) {
            String newClassName = className.substring(1, className.length() - 1);
            return loadClass(newClassName, classLoader);
         } else {
            try {
               if (classLoader != null) {
                  clazz = classLoader.loadClass(className);
                  if (cache) {
                     mappings.put(className, clazz);
                  }

                  return clazz;
               }
            } catch (Throwable var7) {
               var7.printStackTrace();
            }

            try {
               ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
               if (contextClassLoader != null && contextClassLoader != classLoader) {
                  clazz = contextClassLoader.loadClass(className);
                  if (cache) {
                     mappings.put(className, clazz);
                  }

                  return clazz;
               }
            } catch (Throwable var6) {
            }

            try {
               clazz = Class.forName(className);
               mappings.put(className, clazz);
               return clazz;
            } catch (Throwable var5) {
               return clazz;
            }
         }
      } else {
         return null;
      }
   }

   public static SerializeBeanInfo buildBeanInfo(Class<?> beanType, Map<String, String> aliasMap, PropertyNamingStrategy propertyNamingStrategy) {
      return buildBeanInfo(beanType, aliasMap, propertyNamingStrategy, false);
   }

   public static SerializeBeanInfo buildBeanInfo(Class<?> beanType, Map<String, String> aliasMap, PropertyNamingStrategy propertyNamingStrategy, boolean fieldBased) {
      JSONType jsonType = (JSONType)getAnnotation(beanType, JSONType.class);
      String[] orders = null;
      String typeName = null;
      String typeKey = null;
      int features;
      if (jsonType != null) {
         orders = jsonType.orders();
         typeName = jsonType.typeName();
         if (typeName.length() == 0) {
            typeName = null;
         }

         PropertyNamingStrategy jsonTypeNaming = jsonType.naming();
         if (jsonTypeNaming != PropertyNamingStrategy.CamelCase) {
            propertyNamingStrategy = jsonTypeNaming;
         }

         features = SerializerFeature.of(jsonType.serialzeFeatures());

         for(Class supperClass = beanType.getSuperclass(); supperClass != null && supperClass != Object.class; supperClass = supperClass.getSuperclass()) {
            JSONType superJsonType = (JSONType)getAnnotation(supperClass, JSONType.class);
            if (superJsonType == null) {
               break;
            }

            typeKey = superJsonType.typeKey();
            if (typeKey.length() != 0) {
               break;
            }
         }

         Class[] var16 = beanType.getInterfaces();
         int var18 = var16.length;

         for(int var12 = 0; var12 < var18; ++var12) {
            Class<?> interfaceClass = var16[var12];
            JSONType superJsonType = (JSONType)getAnnotation(interfaceClass, JSONType.class);
            if (superJsonType != null) {
               typeKey = superJsonType.typeKey();
               if (typeKey.length() != 0) {
                  break;
               }
            }
         }

         if (typeKey != null && typeKey.length() == 0) {
            typeKey = null;
         }
      } else {
         features = 0;
      }

      Map<String, Field> fieldCacheMap = new HashMap();
      ParserConfig.parserAllFieldToCache(beanType, fieldCacheMap);
      List<FieldInfo> fieldInfoList = fieldBased ? computeGettersWithFieldBase(beanType, aliasMap, false, propertyNamingStrategy) : computeGetters(beanType, jsonType, aliasMap, fieldCacheMap, false, propertyNamingStrategy);
      FieldInfo[] fields = new FieldInfo[fieldInfoList.size()];
      fieldInfoList.toArray(fields);
      Object sortedFieldList;
      if (orders != null && orders.length != 0) {
         sortedFieldList = fieldBased ? computeGettersWithFieldBase(beanType, aliasMap, true, propertyNamingStrategy) : computeGetters(beanType, jsonType, aliasMap, fieldCacheMap, true, propertyNamingStrategy);
      } else {
         sortedFieldList = new ArrayList(fieldInfoList);
         Collections.sort((List)sortedFieldList);
      }

      FieldInfo[] sortedFields = new FieldInfo[((List)sortedFieldList).size()];
      ((List)sortedFieldList).toArray(sortedFields);
      if (Arrays.equals(sortedFields, fields)) {
         sortedFields = fields;
      }

      return new SerializeBeanInfo(beanType, jsonType, typeName, typeKey, features, fields, sortedFields);
   }

   public static List<FieldInfo> computeGettersWithFieldBase(Class<?> clazz, Map<String, String> aliasMap, boolean sorted, PropertyNamingStrategy propertyNamingStrategy) {
      Map<String, FieldInfo> fieldInfoMap = new LinkedHashMap();

      for(Class currentClass = clazz; currentClass != null; currentClass = currentClass.getSuperclass()) {
         Field[] fields = currentClass.getDeclaredFields();
         computeFields(currentClass, aliasMap, propertyNamingStrategy, fieldInfoMap, fields);
      }

      return getFieldInfos(clazz, sorted, fieldInfoMap);
   }

   public static List<FieldInfo> computeGetters(Class<?> clazz, Map<String, String> aliasMap) {
      return computeGetters(clazz, aliasMap, true);
   }

   public static List<FieldInfo> computeGetters(Class<?> clazz, Map<String, String> aliasMap, boolean sorted) {
      JSONType jsonType = (JSONType)getAnnotation(clazz, JSONType.class);
      Map<String, Field> fieldCacheMap = new HashMap();
      ParserConfig.parserAllFieldToCache(clazz, fieldCacheMap);
      return computeGetters(clazz, jsonType, aliasMap, fieldCacheMap, sorted, PropertyNamingStrategy.CamelCase);
   }

   public static List<FieldInfo> computeGetters(Class<?> clazz, JSONType jsonType, Map<String, String> aliasMap, Map<String, Field> fieldCacheMap, boolean sorted, PropertyNamingStrategy propertyNamingStrategy) {
      Map<String, FieldInfo> fieldInfoMap = new LinkedHashMap();
      boolean kotlin = isKotlin(clazz);
      Constructor[] constructors = null;
      Annotation[][] paramAnnotationArrays = (Annotation[][])null;
      String[] paramNames = null;
      short[] paramNameMapping = null;
      Method[] methods = clazz.getMethods();
      Method[] var13 = methods;
      int var14 = methods.length;

      for(int var15 = 0; var15 < var14; ++var15) {
         Method method = var13[var15];
         String methodName = method.getName();
         int ordinal = 0;
         int serialzeFeatures = 0;
         int parserFeatures = 0;
         String label = null;
         if (!Modifier.isStatic(method.getModifiers()) && !method.getReturnType().equals(Void.TYPE) && method.getParameterTypes().length == 0 && method.getReturnType() != ClassLoader.class && (!methodName.equals("getMetaClass") || !method.getReturnType().getName().equals("groovy.lang.MetaClass")) && (!methodName.equals("getSuppressed") || method.getDeclaringClass() != Throwable.class) && (!kotlin || !isKotlinIgnore(clazz, methodName))) {
            Boolean fieldAnnotationAndNameExists = false;
            JSONField annotation = (JSONField)method.getAnnotation(JSONField.class);
            if (annotation == null) {
               annotation = getSuperMethodAnnotation(clazz, method);
            }

            String propertyName;
            if (annotation == null && kotlin) {
               short p;
               if (constructors == null) {
                  constructors = clazz.getDeclaredConstructors();
                  Constructor creatorConstructor = getKoltinConstructor(constructors);
                  if (creatorConstructor != null) {
                     paramAnnotationArrays = creatorConstructor.getParameterAnnotations();
                     paramNames = getKoltinConstructorParameters(clazz);
                     if (paramNames != null) {
                        String[] paramNames_sorted = new String[paramNames.length];
                        System.arraycopy(paramNames, 0, paramNames_sorted, 0, paramNames.length);
                        Arrays.sort(paramNames_sorted);
                        paramNameMapping = new short[paramNames.length];

                        int index;
                        for(p = 0; p < paramNames.length; paramNameMapping[index] = p++) {
                           index = Arrays.binarySearch(paramNames_sorted, paramNames[p]);
                        }

                        paramNames = paramNames_sorted;
                     }
                  }
               }

               if (paramNames != null && paramNameMapping != null && methodName.startsWith("get")) {
                  propertyName = decapitalize(methodName.substring(3));
                  int p = Arrays.binarySearch(paramNames, propertyName);
                  if (p < 0) {
                     for(int i = 0; i < paramNames.length; ++i) {
                        if (propertyName.equalsIgnoreCase(paramNames[i])) {
                           p = i;
                           break;
                        }
                     }
                  }

                  if (p >= 0) {
                     p = paramNameMapping[p];
                     Annotation[] paramAnnotations = paramAnnotationArrays[p];
                     if (paramAnnotations != null) {
                        Annotation[] var28 = paramAnnotations;
                        int var29 = paramAnnotations.length;

                        for(int var30 = 0; var30 < var29; ++var30) {
                           Annotation paramAnnotation = var28[var30];
                           if (paramAnnotation instanceof JSONField) {
                              annotation = (JSONField)paramAnnotation;
                              break;
                           }
                        }
                     }

                     if (annotation == null) {
                        Field field = ParserConfig.getFieldFromCache(propertyName, fieldCacheMap);
                        if (field != null) {
                           annotation = (JSONField)field.getAnnotation(JSONField.class);
                        }
                     }
                  }
               }
            }

            if (annotation != null) {
               if (!annotation.serialize()) {
                  continue;
               }

               ordinal = annotation.ordinal();
               serialzeFeatures = SerializerFeature.of(annotation.serialzeFeatures());
               parserFeatures = Feature.of(annotation.parseFeatures());
               if (annotation.name().length() != 0) {
                  propertyName = annotation.name();
                  if (aliasMap != null) {
                     propertyName = (String)aliasMap.get(propertyName);
                     if (propertyName == null) {
                        continue;
                     }
                  }

                  FieldInfo fieldInfo = new FieldInfo(propertyName, method, (Field)null, clazz, (Type)null, ordinal, serialzeFeatures, parserFeatures, annotation, (JSONField)null, label);
                  fieldInfoMap.put(propertyName, fieldInfo);
                  continue;
               }

               if (annotation.label().length() != 0) {
                  label = annotation.label();
               }
            }

            char c2;
            String propertyName;
            boolean ignore;
            Field field;
            JSONField fieldAnnotation;
            FieldInfo fieldInfo;
            if (methodName.startsWith("get")) {
               if (methodName.length() < 4 || methodName.equals("getClass") || methodName.equals("getDeclaringClass") && clazz.isEnum()) {
                  continue;
               }

               c2 = methodName.charAt(3);
               if (!Character.isUpperCase(c2) && c2 <= 512) {
                  if (c2 == '_') {
                     propertyName = methodName.substring(4);
                  } else if (c2 == 'f') {
                     propertyName = methodName.substring(3);
                  } else {
                     if (methodName.length() < 5 || !Character.isUpperCase(methodName.charAt(4))) {
                        continue;
                     }

                     propertyName = decapitalize(methodName.substring(3));
                  }
               } else {
                  if (compatibleWithJavaBean) {
                     propertyName = decapitalize(methodName.substring(3));
                  } else {
                     propertyName = Character.toLowerCase(methodName.charAt(3)) + methodName.substring(4);
                  }

                  propertyName = getPropertyNameByCompatibleFieldName(fieldCacheMap, methodName, propertyName, 3);
               }

               ignore = isJSONTypeIgnore(clazz, propertyName);
               if (ignore) {
                  continue;
               }

               field = ParserConfig.getFieldFromCache(propertyName, fieldCacheMap);
               if (field == null && propertyName.length() > 1) {
                  char ch = propertyName.charAt(1);
                  if (ch >= 'A' && ch <= 'Z') {
                     String javaBeanCompatiblePropertyName = decapitalize(methodName.substring(3));
                     field = ParserConfig.getFieldFromCache(javaBeanCompatiblePropertyName, fieldCacheMap);
                  }
               }

               fieldAnnotation = null;
               if (field != null) {
                  fieldAnnotation = (JSONField)field.getAnnotation(JSONField.class);
                  if (fieldAnnotation != null) {
                     if (!fieldAnnotation.serialize()) {
                        continue;
                     }

                     ordinal = fieldAnnotation.ordinal();
                     serialzeFeatures = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                     parserFeatures = Feature.of(fieldAnnotation.parseFeatures());
                     if (fieldAnnotation.name().length() != 0) {
                        fieldAnnotationAndNameExists = true;
                        propertyName = fieldAnnotation.name();
                        if (aliasMap != null) {
                           propertyName = (String)aliasMap.get(propertyName);
                           if (propertyName == null) {
                              continue;
                           }
                        }
                     }

                     if (fieldAnnotation.label().length() != 0) {
                        label = fieldAnnotation.label();
                     }
                  }
               }

               if (aliasMap != null) {
                  propertyName = (String)aliasMap.get(propertyName);
                  if (propertyName == null) {
                     continue;
                  }
               }

               if (propertyNamingStrategy != null && !fieldAnnotationAndNameExists) {
                  propertyName = propertyNamingStrategy.translate(propertyName);
               }

               fieldInfo = new FieldInfo(propertyName, method, field, clazz, (Type)null, ordinal, serialzeFeatures, parserFeatures, annotation, fieldAnnotation, label);
               fieldInfoMap.put(propertyName, fieldInfo);
            }

            if (methodName.startsWith("is") && methodName.length() >= 3 && (method.getReturnType() == Boolean.TYPE || method.getReturnType() == Boolean.class)) {
               c2 = methodName.charAt(2);
               if (Character.isUpperCase(c2)) {
                  if (compatibleWithJavaBean) {
                     propertyName = decapitalize(methodName.substring(2));
                  } else {
                     propertyName = Character.toLowerCase(methodName.charAt(2)) + methodName.substring(3);
                  }

                  propertyName = getPropertyNameByCompatibleFieldName(fieldCacheMap, methodName, propertyName, 2);
               } else if (c2 == '_') {
                  propertyName = methodName.substring(3);
               } else {
                  if (c2 != 'f') {
                     continue;
                  }

                  propertyName = methodName.substring(2);
               }

               ignore = isJSONTypeIgnore(clazz, propertyName);
               if (!ignore) {
                  field = ParserConfig.getFieldFromCache(propertyName, fieldCacheMap);
                  if (field == null) {
                     field = ParserConfig.getFieldFromCache(methodName, fieldCacheMap);
                  }

                  fieldAnnotation = null;
                  if (field != null) {
                     fieldAnnotation = (JSONField)field.getAnnotation(JSONField.class);
                     if (fieldAnnotation != null) {
                        if (!fieldAnnotation.serialize()) {
                           continue;
                        }

                        ordinal = fieldAnnotation.ordinal();
                        serialzeFeatures = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                        parserFeatures = Feature.of(fieldAnnotation.parseFeatures());
                        if (fieldAnnotation.name().length() != 0) {
                           propertyName = fieldAnnotation.name();
                           if (aliasMap != null) {
                              propertyName = (String)aliasMap.get(propertyName);
                              if (propertyName == null) {
                                 continue;
                              }
                           }
                        }

                        if (fieldAnnotation.label().length() != 0) {
                           label = fieldAnnotation.label();
                        }
                     }
                  }

                  if (aliasMap != null) {
                     propertyName = (String)aliasMap.get(propertyName);
                     if (propertyName == null) {
                        continue;
                     }
                  }

                  if (propertyNamingStrategy != null) {
                     propertyName = propertyNamingStrategy.translate(propertyName);
                  }

                  if (!fieldInfoMap.containsKey(propertyName)) {
                     fieldInfo = new FieldInfo(propertyName, method, field, clazz, (Type)null, ordinal, serialzeFeatures, parserFeatures, annotation, fieldAnnotation, label);
                     fieldInfoMap.put(propertyName, fieldInfo);
                  }
               }
            }
         }
      }

      Field[] fields = clazz.getFields();
      computeFields(clazz, aliasMap, propertyNamingStrategy, fieldInfoMap, fields);
      return getFieldInfos(clazz, sorted, fieldInfoMap);
   }

   private static List<FieldInfo> getFieldInfos(Class<?> clazz, boolean sorted, Map<String, FieldInfo> fieldInfoMap) {
      List<FieldInfo> fieldInfoList = new ArrayList();
      String[] orders = null;
      JSONType annotation = (JSONType)getAnnotation(clazz, JSONType.class);
      if (annotation != null) {
         orders = annotation.orders();
      }

      if (orders != null && orders.length > 0) {
         LinkedHashMap<String, FieldInfo> map = new LinkedHashMap(fieldInfoList.size());
         Iterator var14 = fieldInfoMap.values().iterator();

         while(var14.hasNext()) {
            FieldInfo field = (FieldInfo)var14.next();
            map.put(field.name, field);
         }

         int i = false;
         String[] var16 = orders;
         int var9 = orders.length;

         for(int var10 = 0; var10 < var9; ++var10) {
            String item = var16[var10];
            FieldInfo field = (FieldInfo)map.get(item);
            if (field != null) {
               fieldInfoList.add(field);
               map.remove(item);
            }
         }

         Iterator var17 = map.values().iterator();

         while(var17.hasNext()) {
            FieldInfo field = (FieldInfo)var17.next();
            fieldInfoList.add(field);
         }
      } else {
         Iterator var6 = fieldInfoMap.values().iterator();

         while(var6.hasNext()) {
            FieldInfo fieldInfo = (FieldInfo)var6.next();
            fieldInfoList.add(fieldInfo);
         }

         if (sorted) {
            Collections.sort(fieldInfoList);
         }
      }

      return fieldInfoList;
   }

   private static void computeFields(Class<?> clazz, Map<String, String> aliasMap, PropertyNamingStrategy propertyNamingStrategy, Map<String, FieldInfo> fieldInfoMap, Field[] fields) {
      Field[] var5 = fields;
      int var6 = fields.length;

      for(int var7 = 0; var7 < var6; ++var7) {
         Field field = var5[var7];
         if (!Modifier.isStatic(field.getModifiers())) {
            JSONField fieldAnnotation = (JSONField)field.getAnnotation(JSONField.class);
            int ordinal = 0;
            int serialzeFeatures = 0;
            int parserFeatures = 0;
            String propertyName = field.getName();
            String label = null;
            if (fieldAnnotation != null) {
               if (!fieldAnnotation.serialize()) {
                  continue;
               }

               ordinal = fieldAnnotation.ordinal();
               serialzeFeatures = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
               parserFeatures = Feature.of(fieldAnnotation.parseFeatures());
               if (fieldAnnotation.name().length() != 0) {
                  propertyName = fieldAnnotation.name();
               }

               if (fieldAnnotation.label().length() != 0) {
                  label = fieldAnnotation.label();
               }
            }

            if (aliasMap != null) {
               propertyName = (String)aliasMap.get(propertyName);
               if (propertyName == null) {
                  continue;
               }
            }

            if (propertyNamingStrategy != null) {
               propertyName = propertyNamingStrategy.translate(propertyName);
            }

            if (!fieldInfoMap.containsKey(propertyName)) {
               FieldInfo fieldInfo = new FieldInfo(propertyName, (Method)null, field, clazz, (Type)null, ordinal, serialzeFeatures, parserFeatures, (JSONField)null, fieldAnnotation, label);
               fieldInfoMap.put(propertyName, fieldInfo);
            }
         }
      }

   }

   private static String getPropertyNameByCompatibleFieldName(Map<String, Field> fieldCacheMap, String methodName, String propertyName, int fromIdx) {
      if (compatibleWithFieldName && !fieldCacheMap.containsKey(propertyName)) {
         String tempPropertyName = methodName.substring(fromIdx);
         return fieldCacheMap.containsKey(tempPropertyName) ? tempPropertyName : propertyName;
      } else {
         return propertyName;
      }
   }

   public static JSONField getSuperMethodAnnotation(Class<?> clazz, Method method) {
      Class<?>[] interfaces = clazz.getInterfaces();
      Class[] types;
      int var6;
      if (interfaces.length > 0) {
         Class<?>[] types = method.getParameterTypes();
         types = interfaces;
         int var5 = interfaces.length;

         for(var6 = 0; var6 < var5; ++var6) {
            Class<?> interfaceClass = types[var6];
            Method[] var8 = interfaceClass.getMethods();
            int var9 = var8.length;

            for(int var10 = 0; var10 < var9; ++var10) {
               Method interfaceMethod = var8[var10];
               Class<?>[] interfaceTypes = interfaceMethod.getParameterTypes();
               if (interfaceTypes.length == types.length && interfaceMethod.getName().equals(method.getName())) {
                  boolean match = true;

                  for(int i = 0; i < types.length; ++i) {
                     if (!interfaceTypes[i].equals(types[i])) {
                        match = false;
                        break;
                     }
                  }

                  if (match) {
                     JSONField annotation = (JSONField)interfaceMethod.getAnnotation(JSONField.class);
                     if (annotation != null) {
                        return annotation;
                     }
                  }
               }
            }
         }
      }

      Class<?> superClass = clazz.getSuperclass();
      if (superClass == null) {
         return null;
      } else {
         if (Modifier.isAbstract(superClass.getModifiers())) {
            types = method.getParameterTypes();
            Method[] var16 = superClass.getMethods();
            var6 = var16.length;

            for(int var17 = 0; var17 < var6; ++var17) {
               Method interfaceMethod = var16[var17];
               Class<?>[] interfaceTypes = interfaceMethod.getParameterTypes();
               if (interfaceTypes.length == types.length && interfaceMethod.getName().equals(method.getName())) {
                  boolean match = true;

                  for(int i = 0; i < types.length; ++i) {
                     if (!interfaceTypes[i].equals(types[i])) {
                        match = false;
                        break;
                     }
                  }

                  if (match) {
                     JSONField annotation = (JSONField)interfaceMethod.getAnnotation(JSONField.class);
                     if (annotation != null) {
                        return annotation;
                     }
                  }
               }
            }
         }

         return null;
      }
   }

   private static boolean isJSONTypeIgnore(Class<?> clazz, String propertyName) {
      JSONType jsonType = (JSONType)getAnnotation(clazz, JSONType.class);
      if (jsonType != null) {
         String[] fields = jsonType.includes();
         int i;
         if (fields.length > 0) {
            for(i = 0; i < fields.length; ++i) {
               if (propertyName.equals(fields[i])) {
                  return false;
               }
            }

            return true;
         }

         fields = jsonType.ignores();

         for(i = 0; i < fields.length; ++i) {
            if (propertyName.equals(fields[i])) {
               return true;
            }
         }
      }

      return clazz.getSuperclass() != Object.class && clazz.getSuperclass() != null && isJSONTypeIgnore(clazz.getSuperclass(), propertyName);
   }

   public static boolean isGenericParamType(Type type) {
      if (type instanceof ParameterizedType) {
         return true;
      } else if (!(type instanceof Class)) {
         return false;
      } else {
         Type superType = ((Class)type).getGenericSuperclass();
         return superType != Object.class && isGenericParamType(superType);
      }
   }

   public static Type getGenericParamType(Type type) {
      if (type instanceof ParameterizedType) {
         return type;
      } else {
         return type instanceof Class ? getGenericParamType(((Class)type).getGenericSuperclass()) : type;
      }
   }

   public static Type unwrapOptional(Type type) {
      if (!optionalClassInited) {
         try {
            optionalClass = Class.forName("java.util.Optional");
         } catch (Exception var5) {
         } finally {
            optionalClassInited = true;
         }
      }

      if (type instanceof ParameterizedType) {
         ParameterizedType parameterizedType = (ParameterizedType)type;
         if (parameterizedType.getRawType() == optionalClass) {
            return parameterizedType.getActualTypeArguments()[0];
         }
      }

      return type;
   }

   public static Class<?> getClass(Type type) {
      if (type.getClass() == Class.class) {
         return (Class)type;
      } else if (type instanceof ParameterizedType) {
         return getClass(((ParameterizedType)type).getRawType());
      } else if (type instanceof TypeVariable) {
         Type boundType = ((TypeVariable)type).getBounds()[0];
         return (Class)boundType;
      } else {
         if (type instanceof WildcardType) {
            Type[] upperBounds = ((WildcardType)type).getUpperBounds();
            if (upperBounds.length == 1) {
               return getClass(upperBounds[0]);
            }
         }

         return Object.class;
      }
   }

   public static Field getField(Class<?> clazz, String fieldName, Field[] declaredFields) {
      Field[] var3 = declaredFields;
      int var4 = declaredFields.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         Field field = var3[var5];
         String itemName = field.getName();
         if (fieldName.equals(itemName)) {
            return field;
         }

         char c0;
         char c1;
         if (fieldName.length() > 2 && (c0 = fieldName.charAt(0)) >= 'a' && c0 <= 'z' && (c1 = fieldName.charAt(1)) >= 'A' && c1 <= 'Z' && fieldName.equalsIgnoreCase(itemName)) {
            return field;
         }
      }

      Class<?> superClass = clazz.getSuperclass();
      if (superClass != null && superClass != Object.class) {
         return getField(superClass, fieldName, superClass.getDeclaredFields());
      } else {
         return null;
      }
   }

   public static int getSerializeFeatures(Class<?> clazz) {
      JSONType annotation = (JSONType)getAnnotation(clazz, JSONType.class);
      return annotation == null ? 0 : SerializerFeature.of(annotation.serialzeFeatures());
   }

   public static int getParserFeatures(Class<?> clazz) {
      JSONType annotation = (JSONType)getAnnotation(clazz, JSONType.class);
      return annotation == null ? 0 : Feature.of(annotation.parseFeatures());
   }

   public static String decapitalize(String name) {
      if (name != null && name.length() != 0) {
         if (name.length() > 1 && Character.isUpperCase(name.charAt(1)) && Character.isUpperCase(name.charAt(0))) {
            return name;
         } else {
            char[] chars = name.toCharArray();
            chars[0] = Character.toLowerCase(chars[0]);
            return new String(chars);
         }
      } else {
         return name;
      }
   }

   static void setAccessible(AccessibleObject obj) {
      if (setAccessibleEnable) {
         if (!obj.isAccessible()) {
            try {
               obj.setAccessible(true);
            } catch (AccessControlException var2) {
               setAccessibleEnable = false;
            }

         }
      }
   }

   public static Type getCollectionItemType(Type fieldType) {
      Type itemType = null;
      Type actualTypeArgument;
      if (fieldType instanceof ParameterizedType) {
         actualTypeArgument = ((ParameterizedType)fieldType).getActualTypeArguments()[0];
         if (actualTypeArgument instanceof WildcardType) {
            WildcardType wildcardType = (WildcardType)actualTypeArgument;
            Type[] upperBounds = wildcardType.getUpperBounds();
            if (upperBounds.length == 1) {
               actualTypeArgument = upperBounds[0];
            }
         }

         itemType = actualTypeArgument;
      } else {
         Class clazz;
         if (fieldType instanceof Class && !(clazz = (Class)fieldType).getName().startsWith("java.")) {
            actualTypeArgument = clazz.getGenericSuperclass();
            itemType = getCollectionItemType(actualTypeArgument);
         }
      }

      if (itemType == null) {
         itemType = Object.class;
      }

      return (Type)itemType;
   }

   public static Class<?> getCollectionItemClass(Type fieldType) {
      if (fieldType instanceof ParameterizedType) {
         Type actualTypeArgument = ((ParameterizedType)fieldType).getActualTypeArguments()[0];
         if (actualTypeArgument instanceof WildcardType) {
            WildcardType wildcardType = (WildcardType)actualTypeArgument;
            Type[] upperBounds = wildcardType.getUpperBounds();
            if (upperBounds.length == 1) {
               actualTypeArgument = upperBounds[0];
            }
         }

         if (actualTypeArgument instanceof Class) {
            Class<?> itemClass = (Class)actualTypeArgument;
            if (!Modifier.isPublic(itemClass.getModifiers())) {
               throw new JSONException("can not create ASMParser");
            } else {
               return itemClass;
            }
         } else {
            throw new JSONException("can not create ASMParser");
         }
      } else {
         return Object.class;
      }
   }

   public static Type checkPrimitiveArray(GenericArrayType genericArrayType) {
      Type clz = genericArrayType;
      Type genericComponentType = genericArrayType.getGenericComponentType();

      String prefix;
      for(prefix = "["; genericComponentType instanceof GenericArrayType; prefix = prefix + prefix) {
         genericComponentType = ((GenericArrayType)genericComponentType).getGenericComponentType();
      }

      if (genericComponentType instanceof Class) {
         Class<?> ck = (Class)genericComponentType;
         if (ck.isPrimitive()) {
            try {
               if (ck == Boolean.TYPE) {
                  clz = Class.forName(prefix + "Z");
               } else if (ck == Character.TYPE) {
                  clz = Class.forName(prefix + "C");
               } else if (ck == Byte.TYPE) {
                  clz = Class.forName(prefix + "B");
               } else if (ck == Short.TYPE) {
                  clz = Class.forName(prefix + "S");
               } else if (ck == Integer.TYPE) {
                  clz = Class.forName(prefix + "I");
               } else if (ck == Long.TYPE) {
                  clz = Class.forName(prefix + "J");
               } else if (ck == Float.TYPE) {
                  clz = Class.forName(prefix + "F");
               } else if (ck == Double.TYPE) {
                  clz = Class.forName(prefix + "D");
               }
            } catch (ClassNotFoundException var6) {
            }
         }
      }

      return (Type)clz;
   }

   public static Collection createCollection(Type type) {
      Class<?> rawClass = getRawClass(type);
      Object list;
      if (rawClass != AbstractCollection.class && rawClass != Collection.class) {
         if (rawClass.isAssignableFrom(HashSet.class)) {
            list = new HashSet();
         } else if (rawClass.isAssignableFrom(LinkedHashSet.class)) {
            list = new LinkedHashSet();
         } else if (rawClass.isAssignableFrom(TreeSet.class)) {
            list = new TreeSet();
         } else if (rawClass.isAssignableFrom(ArrayList.class)) {
            list = new ArrayList();
         } else if (rawClass.isAssignableFrom(EnumSet.class)) {
            Object itemType;
            if (type instanceof ParameterizedType) {
               itemType = ((ParameterizedType)type).getActualTypeArguments()[0];
            } else {
               itemType = Object.class;
            }

            list = EnumSet.noneOf((Class)itemType);
         } else {
            try {
               list = (Collection)rawClass.newInstance();
            } catch (Exception var4) {
               throw new JSONException("create instance error, class " + rawClass.getName());
            }
         }
      } else {
         list = new ArrayList();
      }

      return (Collection)list;
   }

   public static Class<?> getRawClass(Type type) {
      if (type instanceof Class) {
         return (Class)type;
      } else if (type instanceof ParameterizedType) {
         return getRawClass(((ParameterizedType)type).getRawType());
      } else {
         throw new JSONException("TODO");
      }
   }

   public static boolean isProxy(Class<?> clazz) {
      Class[] var1 = clazz.getInterfaces();
      int var2 = var1.length;
      int var3 = 0;

      while(var3 < var2) {
         Class<?> item = var1[var3];
         String interfaceName = item.getName();
         if (!interfaceName.equals("net.sf.cglib.proxy.Factory") && !interfaceName.equals("org.springframework.cglib.proxy.Factory")) {
            if (!interfaceName.equals("javassist.util.proxy.ProxyObject") && !interfaceName.equals("org.apache.ibatis.javassist.util.proxy.ProxyObject")) {
               ++var3;
               continue;
            }

            return true;
         }

         return true;
      }

      return false;
   }

   public static boolean isTransient(Method method) {
      if (method == null) {
         return false;
      } else {
         if (!transientClassInited) {
            try {
               transientClass = Class.forName("java.beans.Transient");
            } catch (Exception var5) {
            } finally {
               transientClassInited = true;
            }
         }

         if (transientClass != null) {
            Annotation annotation = method.getAnnotation(transientClass);
            return annotation != null;
         } else {
            return false;
         }
      }
   }

   public static boolean isAnnotationPresentOneToMany(Method method) {
      if (method == null) {
         return false;
      } else {
         if (class_OneToMany == null && !class_OneToMany_error) {
            try {
               class_OneToMany = Class.forName("javax.persistence.OneToMany");
            } catch (Throwable var2) {
               class_OneToMany_error = true;
            }
         }

         return class_OneToMany != null && method.isAnnotationPresent(class_OneToMany);
      }
   }

   public static boolean isAnnotationPresentManyToMany(Method method) {
      if (method == null) {
         return false;
      } else {
         if (class_ManyToMany == null && !class_ManyToMany_error) {
            try {
               class_ManyToMany = Class.forName("javax.persistence.ManyToMany");
            } catch (Throwable var2) {
               class_ManyToMany_error = true;
            }
         }

         return class_ManyToMany != null && (method.isAnnotationPresent(class_OneToMany) || method.isAnnotationPresent(class_ManyToMany));
      }
   }

   public static boolean isHibernateInitialized(Object object) {
      if (object == null) {
         return false;
      } else {
         if (method_HibernateIsInitialized == null && !method_HibernateIsInitialized_error) {
            try {
               Class<?> class_Hibernate = Class.forName("org.hibernate.Hibernate");
               method_HibernateIsInitialized = class_Hibernate.getMethod("isInitialized", Object.class);
            } catch (Throwable var2) {
               method_HibernateIsInitialized_error = true;
            }
         }

         if (method_HibernateIsInitialized != null) {
            try {
               Boolean initialized = (Boolean)method_HibernateIsInitialized.invoke((Object)null, object);
               return initialized;
            } catch (Throwable var3) {
            }
         }

         return true;
      }
   }

   public static long fnv1a_64_lower(String key) {
      long hashCode = -3750763034362895579L;

      for(int i = 0; i < key.length(); ++i) {
         char ch = key.charAt(i);
         if (ch != '_' && ch != '-') {
            if (ch >= 'A' && ch <= 'Z') {
               ch = (char)(ch + 32);
            }

            hashCode ^= (long)ch;
            hashCode *= 1099511628211L;
         }
      }

      return hashCode;
   }

   public static long fnv1a_64(String key) {
      long hashCode = -3750763034362895579L;

      for(int i = 0; i < key.length(); ++i) {
         char ch = key.charAt(i);
         hashCode ^= (long)ch;
         hashCode *= 1099511628211L;
      }

      return hashCode;
   }

   public static boolean isKotlin(Class clazz) {
      if (kotlin_metadata == null && !kotlin_metadata_error) {
         try {
            kotlin_metadata = Class.forName("kotlin.Metadata");
         } catch (Throwable var2) {
            kotlin_metadata_error = true;
         }
      }

      return kotlin_metadata != null && clazz.isAnnotationPresent(kotlin_metadata);
   }

   public static Constructor getKoltinConstructor(Constructor[] constructors) {
      Constructor creatorConstructor = null;
      Constructor[] var2 = constructors;
      int var3 = constructors.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Constructor<?> constructor = var2[var4];
         Class<?>[] parameterTypes = constructor.getParameterTypes();
         if ((parameterTypes.length <= 0 || !parameterTypes[parameterTypes.length - 1].getName().equals("kotlin.jvm.internal.DefaultConstructorMarker")) && (creatorConstructor == null || creatorConstructor.getParameterTypes().length < parameterTypes.length)) {
            creatorConstructor = constructor;
         }
      }

      return creatorConstructor;
   }

   public static String[] getKoltinConstructorParameters(Class clazz) {
      Class class_kotlinn_kparameter;
      if (kotlin_kclass_constructor == null && !kotlin_class_klass_error) {
         try {
            class_kotlinn_kparameter = Class.forName("kotlin.reflect.jvm.internal.KClassImpl");
            kotlin_kclass_constructor = class_kotlinn_kparameter.getConstructor(Class.class);
         } catch (Throwable var11) {
            kotlin_class_klass_error = true;
         }
      }

      if (kotlin_kclass_constructor == null) {
         return null;
      } else {
         if (kotlin_kclass_getConstructors == null && !kotlin_class_klass_error) {
            try {
               class_kotlinn_kparameter = Class.forName("kotlin.reflect.jvm.internal.KClassImpl");
               kotlin_kclass_getConstructors = class_kotlinn_kparameter.getMethod("getConstructors");
            } catch (Throwable var10) {
               kotlin_class_klass_error = true;
            }
         }

         if (kotlin_kfunction_getParameters == null && !kotlin_class_klass_error) {
            try {
               class_kotlinn_kparameter = Class.forName("kotlin.reflect.KFunction");
               kotlin_kfunction_getParameters = class_kotlinn_kparameter.getMethod("getParameters");
            } catch (Throwable var9) {
               kotlin_class_klass_error = true;
            }
         }

         if (kotlin_kparameter_getName == null && !kotlin_class_klass_error) {
            try {
               class_kotlinn_kparameter = Class.forName("kotlin.reflect.KParameter");
               kotlin_kparameter_getName = class_kotlinn_kparameter.getMethod("getName");
            } catch (Throwable var8) {
               kotlin_class_klass_error = true;
            }
         }

         if (kotlin_error) {
            return null;
         } else {
            try {
               Object constructor = null;
               Object kclassImpl = kotlin_kclass_constructor.newInstance(clazz);
               Iterable it = (Iterable)kotlin_kclass_getConstructors.invoke(kclassImpl);

               for(Iterator iterator = it.iterator(); iterator.hasNext(); iterator.hasNext()) {
                  Object item = iterator.next();
                  List parameters = (List)kotlin_kfunction_getParameters.invoke(item);
                  if (constructor == null || parameters.size() != 0) {
                     constructor = item;
                  }
               }

               List parameters = (List)kotlin_kfunction_getParameters.invoke(constructor);
               String[] names = new String[parameters.size()];

               for(int i = 0; i < parameters.size(); ++i) {
                  Object param = parameters.get(i);
                  names[i] = (String)kotlin_kparameter_getName.invoke(param);
               }

               return names;
            } catch (Throwable var12) {
               var12.printStackTrace();
               kotlin_error = true;
               return null;
            }
         }
      }
   }

   private static boolean isKotlinIgnore(Class clazz, String methodName) {
      if (kotlinIgnores == null && !kotlinIgnores_error) {
         try {
            Map<Class, String[]> map = new HashMap();
            Class charRangeClass = Class.forName("kotlin.ranges.CharRange");
            map.put(charRangeClass, new String[]{"getEndInclusive", "isEmpty"});
            Class intRangeClass = Class.forName("kotlin.ranges.IntRange");
            map.put(intRangeClass, new String[]{"getEndInclusive", "isEmpty"});
            Class longRangeClass = Class.forName("kotlin.ranges.LongRange");
            map.put(longRangeClass, new String[]{"getEndInclusive", "isEmpty"});
            Class floatRangeClass = Class.forName("kotlin.ranges.ClosedFloatRange");
            map.put(floatRangeClass, new String[]{"getEndInclusive", "isEmpty"});
            Class doubleRangeClass = Class.forName("kotlin.ranges.ClosedDoubleRange");
            map.put(doubleRangeClass, new String[]{"getEndInclusive", "isEmpty"});
            kotlinIgnores = map;
         } catch (Throwable var8) {
            kotlinIgnores_error = true;
         }
      }

      if (kotlinIgnores == null) {
         return false;
      } else {
         String[] ignores = (String[])kotlinIgnores.get(clazz);
         return ignores != null && Arrays.binarySearch(ignores, methodName) >= 0;
      }
   }

   public static <A extends Annotation> A getAnnotation(Class<?> clazz, Class<A> annotationClass) {
      A a = clazz.getAnnotation(annotationClass);
      if (a != null) {
         return a;
      } else {
         if (clazz.getAnnotations().length > 0) {
            Annotation[] var3 = clazz.getAnnotations();
            int var4 = var3.length;

            for(int var5 = 0; var5 < var4; ++var5) {
               Annotation annotation = var3[var5];
               a = annotation.annotationType().getAnnotation(annotationClass);
               if (a != null) {
                  return a;
               }
            }
         }

         return null;
      }
   }

   static {
      try {
         compatibleWithJavaBean = "true".equals(IOUtils.getStringProperty("fastjson.compatibleWithJavaBean"));
         compatibleWithFieldName = "true".equals(IOUtils.getStringProperty("fastjson.compatibleWithFieldName"));
      } catch (Throwable var1) {
      }

      addBaseClassMappings();
   }
}
